@@ -3,8 +3,6 @@
  *  This file is part of the KDE libraries
  *  Copyright (c) 2001 Waldo Bastian <bastian@kde.org>
  *
- * $Id: kconf_update.cpp 465272 2005-09-29 09:47:40Z mueller $
- *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU Library General Public
  *  License version 2 as published by the Free Software Foundation.
@@ -24,27 +22,23 @@
 #include <sys/stat.h>
 #include <unistd.h>
 #include <stdlib.h>
+#include <kde_file.h>
 
-#include <qfile.h>
-#include <qtextstream.h>
+#include <QtCore/QDate>
+#include <QtCore/QFile>
+#include <QtCore/QTextStream>
+#include <QtCore/QTextCodec>
 
 #include <kconfig.h>
-#include <ksimpleconfig.h>
+#include <kconfiggroup.h>
 #include <klocale.h>
 #include <kcmdlineargs.h>
 #include <kglobal.h>
 #include <kstandarddirs.h>
 #include <kaboutdata.h>
-#include <kinstance.h>
-#include <ktempfile.h>
-
-static KCmdLineOptions options[] =
-{
-        { "debug", I18N_NOOP("Keep output results from scripts"), 0 },
-	{ "check <update-file>", I18N_NOOP("Check whether config file itself requires updating"), 0 },
-	{ "+[file]", I18N_NOOP("File to read update instructions from"), 0 },
-        KCmdLineLastOption
-};
+#include <kcomponentdata.h>
+#include <ktemporaryfile.h>
+#include <kurl.h>
 
 class KonfUpdate
 {
@@ -73,13 +67,14 @@ public:
    void gotScriptArguments(const QString &_arguments);
    void resetOptions();
 
-   void copyGroup(KConfigBase *cfg1, const QString &grp1, 
-                  KConfigBase *cfg2, const QString &grp2);
+   void copyGroup(KConfig *cfg1, const QString &group1,
+                  KConfig *cfg2, const QString &group2);
 
 protected:
    KConfig *config;
    QString currentFilename;
    bool skip;
+   bool skipFile;
    bool debug;
    QString id;
 
@@ -114,21 +109,22 @@ KonfUpdate::KonfUpdate()
    newConfig = 0;
 
    config = new KConfig("kconf_updaterc");
+   KConfigGroup cg(config, QString());
 
    QStringList updateFiles;
    KCmdLineArgs *args=KCmdLineArgs::parsedArgs();
-   
+
    debug = args->isSet("debug");
 
    m_bUseConfigInfo = false;
    if (args->isSet("check"))
    {
       m_bUseConfigInfo = true;
-      QString file = locate("data", "kconf_update/"+QFile::decodeName(args->getOption("check")));
+      QString file = KStandardDirs::locate("data", "kconf_update/"+args->getOption("check"));
       if (file.isEmpty())
       {
-         qWarning("File '%s' not found.", args->getOption("check").data());
-         log() << "File '" << QFile::decodeName(args->getOption("check")) << "' passed on command line not found" << endl;
+         qWarning("File '%s' not found.", args->getOption("check").toLocal8Bit().data());
+         log() << "File '" << args->getOption("check") << "' passed on command line not found" << endl;
          return;
       }
       updateFiles.append(file);
@@ -137,15 +133,15 @@ KonfUpdate::KonfUpdate()
    {
       for(int i = 0; i < args->count(); i++)
       {
-         KURL url = args->url(i);
+         KUrl url = args->url(i);
          if (!url.isLocalFile())
-            KCmdLineArgs::usage(i18n("Only local files are supported."));
+            KCmdLineArgs::usageError(i18n("Only local files are supported."));
          updateFiles.append(url.path());
       }
    }
    else
    {
-      if (config->readBoolEntry("autoUpdateDisabled", false))
+      if (cg.readEntry("autoUpdateDisabled", false))
          return;
       updateFiles = findUpdateFiles(true);
       updateAll = true;
@@ -159,10 +155,9 @@ KonfUpdate::KonfUpdate()
       updateFile(file);
    }
 
-   config->setGroup(QString::null);
-   if (updateAll && !config->readBoolEntry("updateInfoAdded", false))
+   if (updateAll && !cg.readEntry("updateInfoAdded", false))
    {
-       config->writeEntry("updateInfoAdded", true);
+       cg.writeEntry("updateInfoAdded", true);
        updateFiles = findUpdateFiles(false);
 
        for(QStringList::ConstIterator it = updateFiles.begin();
@@ -188,42 +183,43 @@ KonfUpdate::log()
 {
    if (!m_textStream)
    {
-      QString file = locateLocal("data", "kconf_update/log/update.log");
+      QString file = KStandardDirs::locateLocal("data", "kconf_update/log/update.log");
       m_file = new QFile(file);
-      if (m_file->open(IO_WriteOnly | IO_Append))
+      if (m_file->open(QIODevice::WriteOnly | QIODevice::Append))
       {
         m_textStream = new QTextStream(m_file);
       }
       else
       {
         // Error
-        m_textStream = new QTextStream(stderr, IO_WriteOnly);
+        m_textStream = new QTextStream(stderr, QIODevice::WriteOnly);
       }
    }
-   
+
    (*m_textStream) << QDateTime::currentDateTime().toString( Qt::ISODate ) << " ";
-   
+
    return *m_textStream;
 }
 
 QStringList KonfUpdate::findUpdateFiles(bool dirtyOnly)
 {
    QStringList result;
-   QStringList list = KGlobal::dirs()->findAllResources("data", "kconf_update/*.upd", false, true);
+   QStringList list = KGlobal::dirs()->findAllResources("data", "kconf_update/*.upd",
+                                                        KStandardDirs::NoDuplicates);
    for(QStringList::ConstIterator it = list.begin();
        it != list.end();
        ++it)
    {
       QString file = *it;
-      struct stat buff;
-      if (stat( QFile::encodeName(file), &buff) == 0)
+      KDE_struct_stat buff;
+      if (KDE_stat( QFile::encodeName(file), &buff) == 0)
       {
-         int i = file.findRev('/');
-         if (i != -1) 
+         int i = file.lastIndexOf('/');
+         if (i != -1)
             file = file.mid(i+1);
-         config->setGroup(file);
-         time_t ctime = config->readUnsignedLongNumEntry("ctime");
-         time_t mtime = config->readUnsignedLongNumEntry("mtime");
+         KConfigGroup cg(config, file);
+         time_t ctime = cg.readEntry("ctime", 0);
+         time_t mtime = cg.readEntry("mtime", 0);
          if (!dirtyOnly ||
              (ctime != buff.st_ctime) || (mtime != buff.st_mtime))
          {
@@ -237,56 +233,56 @@ QStringList KonfUpdate::findUpdateFiles(bool dirtyOnly)
 bool KonfUpdate::checkFile(const QString &filename)
 {
    currentFilename = filename;
-   int i = currentFilename.findRev('/');
-   if (i != -1) 
+   int i = currentFilename.lastIndexOf('/');
+   if (i != -1)
       currentFilename = currentFilename.mid(i+1);
    skip = true;
    QFile file(filename);
-   if (!file.open(IO_ReadOnly))
+   if (!file.open(QIODevice::ReadOnly))
       return false;
 
    QTextStream ts(&file);
-   ts.setEncoding(QTextStream::Latin1);
+   ts.setCodec(QTextCodec::codecForName("ISO-8859-1"));
    int lineCount = 0;
    resetOptions();
    QString id;
    while(!ts.atEnd())
    {
-      QString line = ts.readLine().stripWhiteSpace();
+      QString line = ts.readLine().trimmed();
       lineCount++;
       if (line.isEmpty() || (line[0] == '#'))
          continue;
       if (line.startsWith("Id="))
-         id = currentFilename+":"+line.mid(3);
+         id = currentFilename+':'+line.mid(3);
       else if (line.startsWith("File="))
          checkGotFile(line.mid(5), id);
    }
-  
+
    return true;
 }
 
 void KonfUpdate::checkGotFile(const QString &_file, const QString &id)
 {
    QString file;
-   int i = _file.find(',');
+   int i = _file.indexOf(',');
    if (i == -1)
    {
-      file = _file.stripWhiteSpace();
+      file = _file.trimmed();
    }
    else
    {
-      file = _file.mid(i+1).stripWhiteSpace();
+      file = _file.mid(i+1).trimmed();
    }
 
-//   qDebug("File %s, id %s", file.latin1(), id.latin1());
+//   qDebug("File %s, id %s", file.toLatin1().constData(), id.toLatin1().constData());
 
-   KSimpleConfig cfg(file);
-   cfg.setGroup("$Version");
-   QStringList ids = cfg.readListEntry("update_info");
+   KConfig cfg(file, KConfig::SimpleConfig);
+   KConfigGroup cg(&cfg, "$Version");
+   QStringList ids = cg.readEntry("update_info", QStringList());
    if (ids.contains(id))
        return;
    ids.append(id);
-   cfg.writeEntry("update_info", ids);
+   cg.writeEntry("update_info", ids);
 }
 
 /**
@@ -311,23 +307,23 @@ void KonfUpdate::checkGotFile(const QString &_file, const QString &id)
 bool KonfUpdate::updateFile(const QString &filename)
 {
    currentFilename = filename;
-   int i = currentFilename.findRev('/');
-   if (i != -1) 
+   int i = currentFilename.lastIndexOf('/');
+   if (i != -1)
        currentFilename = currentFilename.mid(i+1);
    skip = true;
    QFile file(filename);
-   if (!file.open(IO_ReadOnly))
+   if (!file.open(QIODevice::ReadOnly))
       return false;
 
-   log() << "Checking update-file '" << filename << "' for new updates" << endl; 
+   log() << "Checking update-file '" << filename << "' for new updates" << endl;
 
    QTextStream ts(&file);
-   ts.setEncoding(QTextStream::Latin1);
+   ts.setCodec(QTextCodec::codecForName("ISO-8859-1"));
    m_lineCount = 0;
    resetOptions();
    while(!ts.atEnd())
    {
-      m_line = ts.readLine().stripWhiteSpace();
+      m_line = ts.readLine().trimmed();
       m_lineCount++;
       if (m_line.isEmpty() || (m_line[0] == '#'))
          continue;
@@ -339,6 +335,8 @@ bool KonfUpdate::updateFile(const QString &filename)
          gotOptions(m_line.mid(8));
       else if (m_line.startsWith("File="))
          gotFile(m_line.mid(5));
+      else if(skipFile)
+         continue;
       else if (m_line.startsWith("Group="))
          gotGroup(m_line.mid(6));
       else if (m_line.startsWith("RemoveGroup="))
@@ -379,14 +377,14 @@ bool KonfUpdate::updateFile(const QString &filename)
       }
    }
    // Flush.
-   gotId(QString::null);
-  
-   struct stat buff;
-   stat( QFile::encodeName(filename), &buff);
-   config->setGroup(currentFilename);
-   config->writeEntry("ctime", buff.st_ctime);
-   config->writeEntry("mtime", buff.st_mtime);
-   config->sync();
+   gotId(QString());
+
+   KDE_struct_stat buff;
+   KDE_stat( QFile::encodeName(filename), &buff);
+   KConfigGroup cg(config, currentFilename);
+   cg.writeEntry("ctime", int(buff.st_ctime));
+   cg.writeEntry("mtime", int(buff.st_mtime));
+   cg.sync();
    return true;
 }
 
@@ -396,26 +394,27 @@ void KonfUpdate::gotId(const QString &_id)
 {
    if (!id.isEmpty() && !skip)
    {
-       config->setGroup(currentFilename);
-       QStringList ids = config->readListEntry("done");
+       KConfigGroup cg(config, currentFilename);
+
+       QStringList ids = cg.readEntry("done", QStringList());
        if (!ids.contains(id))
        {
           ids.append(id);
-          config->writeEntry("done", ids);
-          config->sync();
+          cg.writeEntry("done", ids);
+          cg.sync();
        }
    }
 
    // Flush pending changes
-   gotFile(QString::null);
+   gotFile(QString());
+   KConfigGroup cg(config, currentFilename);
 
-   config->setGroup(currentFilename);
-   QStringList ids = config->readListEntry("done");
+   QStringList ids = cg.readEntry("done", QStringList());
    if (!_id.isEmpty())
    {
        if (ids.contains(_id))
        {
-          //qDebug("Id '%s' was already in done-list", _id.latin1());
+          //qDebug("Id '%s' was already in done-list", _id.toLatin1().constData());
           if (!m_bUseConfigInfo)
           {
              skip = true;
@@ -423,6 +422,7 @@ void KonfUpdate::gotId(const QString &_id)
           }
        }
        skip = false;
+       skipFile = false;
        id = _id;
        if (m_bUseConfigInfo)
           log() << currentFilename << ": Checking update '" << _id << "'" << endl;
@@ -434,89 +434,89 @@ void KonfUpdate::gotId(const QString &_id)
 void KonfUpdate::gotFile(const QString &_file)
 {
    // Reset group
-   gotGroup(QString::null);
- 
+   gotGroup(QString());
+
    if (!oldFile.isEmpty())
    {
       // Close old file.
       delete oldConfig1;
       oldConfig1 = 0;
 
-      oldConfig2->setGroup("$Version");
-      QStringList ids = oldConfig2->readListEntry("update_info");
-      QString cfg_id = currentFilename + ":" + id;
+      KConfigGroup cg(oldConfig2, "$Version");
+      QStringList ids = cg.readEntry("update_info", QStringList());
+      QString cfg_id = currentFilename + ':' + id;
       if (!ids.contains(cfg_id) && !skip)
       {
          ids.append(cfg_id);
-         oldConfig2->writeEntry("update_info", ids);
+         cg.writeEntry("update_info", ids);
       }
-      oldConfig2->sync();
+      cg.sync();
       delete oldConfig2;
       oldConfig2 = 0;
-      
-      QString file = locateLocal("config", oldFile);
-      struct stat s_buf;
-      if (stat(QFile::encodeName(file), &s_buf) == 0)
+
+      QString file = KStandardDirs::locateLocal("config", oldFile);
+      KDE_struct_stat s_buf;
+      if (KDE_stat(QFile::encodeName(file), &s_buf) == 0)
       {
          if (s_buf.st_size == 0)
          {
             // Delete empty file.
             unlink(QFile::encodeName(file));
-         }   
+         }
       }
 
-      oldFile = QString::null;
+      oldFile.clear();
    }
    if (!newFile.isEmpty())
    {
       // Close new file.
-      newConfig->setGroup("$Version");
-      QStringList ids = newConfig->readListEntry("update_info");
-      QString cfg_id = currentFilename + ":" + id;
+      KConfigGroup cg(newConfig, "$Version");
+      QStringList ids = cg.readEntry("update_info", QStringList());
+      QString cfg_id = currentFilename + ':' + id;
       if (!ids.contains(cfg_id) && !skip)
       {
          ids.append(cfg_id);
-         newConfig->writeEntry("update_info", ids);
+         cg.writeEntry("update_info", ids);
       }
       newConfig->sync();
       delete newConfig;
       newConfig = 0;
 
-      newFile = QString::null;
+      newFile.clear();
    }
-   newConfig = 0; 
+   newConfig = 0;
 
-   int i = _file.find(',');
+   int i = _file.indexOf(',');
    if (i == -1)
    {
-      oldFile = _file.stripWhiteSpace();
+      oldFile = _file.trimmed();
    }
    else
    {
-      oldFile = _file.left(i).stripWhiteSpace();
-      newFile = _file.mid(i+1).stripWhiteSpace();
+      oldFile = _file.left(i).trimmed();
+      newFile = _file.mid(i+1).trimmed();
       if (oldFile == newFile)
-         newFile = QString::null;
+         newFile.clear();
    }
-   
+
    if (!oldFile.isEmpty())
    {
-      oldConfig2 = new KConfig(oldFile, false, false);
-      QString cfg_id = currentFilename + ":" + id;
-      oldConfig2->setGroup("$Version");
-      QStringList ids = oldConfig2->readListEntry("update_info");
+      oldConfig2 = new KConfig(oldFile, KConfig::NoGlobals);
+      QString cfg_id = currentFilename + ':' + id;
+      KConfigGroup cg(oldConfig2, "$Version");
+      QStringList ids = cg.readEntry("update_info", QStringList());
       if (ids.contains(cfg_id))
       {
          skip = true;
-         newFile = QString::null;
+         newFile.clear();
          log() << currentFilename << ": Skipping update '" << id << "'" << endl;
       }
 
       if (!newFile.isEmpty())
       {
-         newConfig = new KConfig(newFile, false, false);
-         newConfig->setGroup("$Version");
-         ids = newConfig->readListEntry("update_info");
+         newConfig = new KConfig(newFile, KConfig::NoGlobals);
+         KConfigGroup cg(newConfig, "$Version");
+         ids = cg.readEntry("update_info", QStringList());
          if (ids.contains(cfg_id))
          {
             skip = true;
@@ -528,35 +528,47 @@ void KonfUpdate::gotFile(const QString &_file)
          newConfig = oldConfig2;
       }
 
-      oldConfig1 = new KConfig(oldFile, true, false);
+      oldConfig1 = new KConfig(oldFile, KConfig::NoGlobals);
    }
    else
    {
-      newFile = QString::null;
+      newFile.clear();
    }
    newFileName = newFile;
    if (newFileName.isEmpty())
       newFileName = oldFile;
+
+   skipFile = false;
+   if( !oldFile.isEmpty())
+   { // if File= is specified, it doesn't exist, is empty or contains only kconf_update's [$Version] group, skip
+      if( oldConfig1 != NULL
+          && ( oldConfig1->groupList().isEmpty()
+              || ( oldConfig1->groupList().count() == 1 && oldConfig1->groupList().first() == "$Version" )))
+      {
+         log() << currentFilename << ": File '" << oldFile << "' does not exist or empty, skipping" << endl;
+         skipFile = true;
+      }
+   }
 }
 
 void KonfUpdate::gotGroup(const QString &_group)
 {
-   int i = _group.find(',');
+   int i = _group.indexOf(',');
    if (i == -1)
    {
-      oldGroup = _group.stripWhiteSpace();
+      oldGroup = _group.trimmed();
       newGroup = oldGroup;
    }
    else
    {
-      oldGroup = _group.left(i).stripWhiteSpace();
-      newGroup = _group.mid(i+1).stripWhiteSpace();
+      oldGroup = _group.left(i).trimmed();
+      newGroup = _group.mid(i+1).trimmed();
    }
 }
 
 void KonfUpdate::gotRemoveGroup(const QString &_group)
 {
-   oldGroup = _group.stripWhiteSpace();
+   oldGroup = _group.trimmed();
 
    if (!oldConfig1)
    {
@@ -567,23 +579,23 @@ void KonfUpdate::gotRemoveGroup(const QString &_group)
    if (!oldConfig1->hasGroup(oldGroup))
       return;
    // Delete group.
-   oldConfig2->deleteGroup(oldGroup, true);
+   oldConfig2->deleteGroup(oldGroup);
    log() << currentFilename << ": RemoveGroup removes group " << oldFile << ":" << oldGroup << endl;
 }
 
 
 void KonfUpdate::gotKey(const QString &_key)
 {
-   int i = _key.find(',');
+   int i = _key.indexOf(',');
    if (i == -1)
    {
-      oldKey = _key.stripWhiteSpace();
+      oldKey = _key.trimmed();
       newKey = oldKey;
    }
    else
    {
-      oldKey = _key.left(i).stripWhiteSpace();
-      newKey = _key.mid(i+1).stripWhiteSpace();
+      oldKey = _key.left(i).trimmed();
+      newKey = _key.mid(i+1).trimmed();
    }
 
    if (oldKey.isEmpty() || newKey.isEmpty())
@@ -596,38 +608,38 @@ void KonfUpdate::gotKey(const QString &_key)
       log() << currentFilename << ": !! Key without previous File specification in line " << m_lineCount << " : '" << m_line << "'" << endl;
       return;
    }
-   oldConfig1->setGroup(oldGroup);
-   if (!oldConfig1->hasKey(oldKey))
+   KConfigGroup cg1( oldConfig1, oldGroup);
+   if (!cg1.hasKey(oldKey))
       return;
-   QString value = oldConfig1->readEntry(oldKey);
-   newConfig->setGroup(newGroup);
-   if (!m_bOverwrite && newConfig->hasKey(newKey))
+   QString value = cg1.readEntry(oldKey, QString());
+   KConfigGroup newFGroup( newConfig, newGroup);
+   if (!m_bOverwrite && newFGroup.hasKey(newKey))
    {
       log() << currentFilename << ": Skipping " << newFileName << ":" << newGroup << ":" << newKey << ", already exists."<< endl;
       return;
    }
    log() << currentFilename << ": Updating " << newFileName << ":" << newGroup << ":" << newKey << " to '" << value << "'" << endl;
-   newConfig->writeEntry(newKey, value);
+   newFGroup.writeEntry(newKey, value);
 
    if (m_bCopy)
       return; // Done.
 
    // Delete old entry
-   if ((oldConfig2 == newConfig) && 
+   if ((oldConfig2 == newConfig) &&
        (oldGroup == newGroup) &&
        (oldKey == newKey))
       return; // Don't delete!
-   oldConfig2->setGroup(oldGroup);
-   oldConfig2->deleteEntry(oldKey, false);
+   KConfigGroup oldGroup2( oldConfig2, oldGroup);
+   oldGroup2.deleteEntry(oldKey);
    log() << currentFilename << ": Removing " << oldFile << ":" << oldGroup << ":" << oldKey << ", moved." << endl;
-   if (oldConfig2->deleteGroup(oldGroup, false)) { // Delete group if empty.
+   /*if (oldConfig2->deleteGroup(oldGroup, KConfig::Normal)) { // Delete group if empty.
       log() << currentFilename << ": Removing empty group " << oldFile << ":" << oldGroup << endl;
-   }
+   }  (this should be automatic)  */
 }
 
 void KonfUpdate::gotRemoveKey(const QString &_key)
 {
-   oldKey = _key.stripWhiteSpace();
+   oldKey = _key.trimmed();
 
    if (oldKey.isEmpty())
    {
@@ -641,17 +653,17 @@ void KonfUpdate::gotRemoveKey(const QString &_key)
       return;
    }
 
-   oldConfig1->setGroup(oldGroup);
-   if (!oldConfig1->hasKey(oldKey))
+   KConfigGroup cg1(oldConfig1, oldGroup);
+   if (!cg1.hasKey(oldKey))
       return;
    log() << currentFilename << ": RemoveKey removes " << oldFile << ":" << oldGroup << ":" << oldKey << endl;
 
    // Delete old entry
-   oldConfig2->setGroup(oldGroup);
-   oldConfig2->deleteEntry(oldKey, false);
-   if (oldConfig2->deleteGroup(oldGroup, false)) { // Delete group if empty.
+   KConfigGroup cg2( oldConfig2, oldGroup);
+   cg2.deleteEntry(oldKey);
+   /*if (oldConfig2->deleteGroup(oldGroup, KConfig::Normal)) { // Delete group if empty.
       log() << currentFilename << ": Removing empty group " << oldFile << ":" << oldGroup << endl;
-   }
+   }   (this should be automatic)*/
 }
 
 void KonfUpdate::gotAllKeys()
@@ -690,29 +702,29 @@ void KonfUpdate::gotAllGroups()
 
 void KonfUpdate::gotOptions(const QString &_options)
 {
-   QStringList options = QStringList::split(',', _options);
+   QStringList options = _options.split(',');
    for(QStringList::ConstIterator it = options.begin();
        it != options.end();
        ++it)
    {
-       if ( (*it).lower().stripWhiteSpace() == "copy")
+       if ( (*it).toLower().trimmed() == "copy")
           m_bCopy = true;
 
-       if ( (*it).lower().stripWhiteSpace() == "overwrite")
+       if ( (*it).toLower().trimmed() == "overwrite")
           m_bOverwrite = true;
    }
 }
 
-void KonfUpdate::copyGroup(KConfigBase *cfg1, const QString &grp1, 
-                           KConfigBase *cfg2, const QString &grp2)
+void KonfUpdate::copyGroup(KConfig *cfg1, const QString &group1,
+                           KConfig *cfg2, const QString &group2)
 {
-   cfg1->setGroup(grp1);
-   cfg2->setGroup(grp2);
-   QMap<QString, QString> list = cfg1->entryMap(grp1);
+   KConfigGroup cg1(cfg1, group1);
+   KConfigGroup cg2(cfg2, group2);
+   QMap<QString, QString> list = cg1.entryMap();
    for(QMap<QString, QString>::Iterator it = list.begin();
        it != list.end(); ++it)
    {
-      cfg2->writeEntry(it.key(), cfg1->readEntry(it.key()));
+      cg2.writeEntry(it.key(), cg1.readEntry(it.key(), QString()));
    }
 }
 
@@ -724,32 +736,32 @@ void KonfUpdate::gotScriptArguments(const QString &_arguments)
 void KonfUpdate::gotScript(const QString &_script)
 {
    QString script, interpreter;
-   int i = _script.find(',');
+   int i = _script.indexOf(',');
    if (i == -1)
    {
-      script = _script.stripWhiteSpace();
+      script = _script.trimmed();
    }
    else
    {
-      script = _script.left(i).stripWhiteSpace();
-      interpreter = _script.mid(i+1).stripWhiteSpace();
+      script = _script.left(i).trimmed();
+      interpreter = _script.mid(i+1).trimmed();
    }
 
 
    if (script.isEmpty())
    {
-      log() << currentFilename << ": !! Script fails to specifiy filename in line " << m_lineCount << " : '" << m_line << "'" << endl;
+      log() << currentFilename << ": !! Script fails to specify filename in line " << m_lineCount << " : '" << m_line << "'" << endl;
       skip = true;
       return;
-   } 
+   }
 
 
 
-   QString path = locate("data","kconf_update/"+script);
+   QString path = KStandardDirs::locate("data","kconf_update/"+script);
    if (path.isEmpty())
    {
       if (interpreter.isEmpty())
-         path = locate("lib", "kconf_update_bin/"+script);
+         path = KStandardDirs::locate("lib", "kconf_update_bin/"+script);
 
       if (path.isEmpty())
       {
@@ -768,7 +780,7 @@ void KonfUpdate::gotScript(const QString &_script)
    if (interpreter.isEmpty())
       cmd = path;
    else
-      cmd = interpreter + " " + path;
+      cmd = interpreter + ' ' + path;
 
    if( !m_arguments.isNull())
    {
@@ -776,22 +788,22 @@ void KonfUpdate::gotScript(const QString &_script)
       cmd += m_arguments;
    }
 
-   KTempFile tmp1;
-   tmp1.setAutoDelete(true);
-   KTempFile tmp2;
-   tmp2.setAutoDelete(true);
-   KTempFile tmp3;
-   tmp3.setAutoDelete(true);
+   KTemporaryFile tmp1;
+   tmp1.open();
+   KTemporaryFile tmp2;
+   tmp2.open();
+   KTemporaryFile tmp3;
+   tmp3.open();
 
    int result;
    if (oldConfig1)
    {
        if (debug)
        {
-           tmp1.setAutoDelete(false);
-           log() << "Script input stored in " << tmp1.name() << endl;
+           tmp1.setAutoRemove(false);
+           log() << "Script input stored in " << tmp1.fileName() << endl;
        }
-       KSimpleConfig cfg(tmp1.name());
+       KConfig cfg(tmp1.fileName(), KConfig::SimpleConfig);
 
        if (oldGroup.isEmpty())
        {
@@ -804,26 +816,26 @@ void KonfUpdate::gotScript(const QString &_script)
                copyGroup(oldConfig1, *it, &cfg, *it);
            }
        }
-       else 
+       else
        {
-           copyGroup(oldConfig1, oldGroup, &cfg, QString::null);
+           copyGroup(oldConfig1, oldGroup, &cfg, QString());
        }
        cfg.sync();
-       result = system(QFile::encodeName(QString("%1 < %2 > %3 2> %4").arg(cmd, tmp1.name(), tmp2.name(), tmp3.name())));
+       result = system(QFile::encodeName(QString("%1 < %2 > %3 2> %4").arg(cmd, tmp1.fileName(), tmp2.fileName(), tmp3.fileName())));
    }
    else
    {
        // No config file
-       result = system(QFile::encodeName(QString("%1 2> %2").arg(cmd, tmp3.name())));
+       result = system(QFile::encodeName(QString("%1 2> %2").arg(cmd, tmp3.fileName())));
    }
 
    // Copy script stderr to log file
    {
-     QFile output(tmp3.name());
-     if (output.open(IO_ReadOnly))
-     { 
+     QFile output(tmp3.fileName());
+     if (output.open(QIODevice::ReadOnly))
+     {
        QTextStream ts( &output );
-       ts.setEncoding(QTextStream::UnicodeUTF8);
+       ts.setCodec(QTextCodec::codecForName("UTF-8"));
        while(!ts.atEnd())
        {
          QString line = ts.readLine();
@@ -834,7 +846,7 @@ void KonfUpdate::gotScript(const QString &_script)
 
    if (result)
    {
-      log() << currentFilename << ": !! An error occured while running '" << cmd << "'" << endl;
+      log() << currentFilename << ": !! An error occurred while running '" << cmd << "'" << endl;
       return;
    }
 
@@ -843,60 +855,59 @@ void KonfUpdate::gotScript(const QString &_script)
 
    if (debug)
    {
-      tmp2.setAutoDelete(false);
-      log() << "Script output stored in " << tmp2.name() << endl;
+      tmp2.setAutoRemove(false);
+      log() << "Script output stored in " << tmp2.fileName() << endl;
    }
 
    // Deleting old entries
    {
      QString group = oldGroup;
-     QFile output(tmp2.name());
-     if (output.open(IO_ReadOnly))
-     { 
+     QFile output(tmp2.fileName());
+     if (output.open(QIODevice::ReadOnly))
+     {
        QTextStream ts( &output );
-       ts.setEncoding(QTextStream::UnicodeUTF8);
+       ts.setCodec(QTextCodec::codecForName("UTF-8"));
        while(!ts.atEnd())
        {
          QString line = ts.readLine();
-         if (line.startsWith("["))
+         if (line.startsWith('['))
          {
-            int j = line.find(']')+1;
+            int j = line.indexOf(']')+1;
             if (j > 0)
                group = line.mid(1, j-2);
          }
          else if (line.startsWith("# DELETE "))
-         {  
+         {
             QString key = line.mid(9);
             if (key[0] == '[')
             {
-               int j = key.find(']')+1;
+               int j = key.indexOf(']')+1;
                if (j > 0)
                {
                   group = key.mid(1,j-2);
                   key = key.mid(j);
                }
             }
-            oldConfig2->setGroup(group);
-            oldConfig2->deleteEntry(key, false);
+            KConfigGroup cg(oldConfig2, group);
+            cg.deleteEntry(key);
             log() << currentFilename << ": Script removes " << oldFile << ":" << group << ":" << key << endl;
-            if (oldConfig2->deleteGroup(group, false)) { // Delete group if empty.
+            /*if (oldConfig2->deleteGroup(group, KConfig::Normal)) { // Delete group if empty.
                log() << currentFilename << ": Removing empty group " << oldFile << ":" << group << endl;
-	    }
+	    } (this should be automatic)*/
          }
          else if (line.startsWith("# DELETEGROUP"))
          {
-            QString key = line.mid(13).stripWhiteSpace();
+            QString key = line.mid(13).trimmed();
             if (key[0] == '[')
             {
-               int j = key.find(']')+1;
+               int j = key.indexOf(']')+1;
                if (j > 0)
                {
                   group = key.mid(1,j-2);
                }
             }
-            if (oldConfig2->deleteGroup(group, true)) { // Delete group
-               log() << currentFilename << ": Script removes group " << oldFile << ":" << group << endl;
-	    }
+            oldConfig2->deleteGroup(group);
+            log() << currentFilename << ": Script removes group " << oldFile << ":" << group << endl;
           }
        }
      }
@@ -908,7 +919,7 @@ void KonfUpdate::gotScript(const QString &_script)
      KConfig *saveOldConfig1 = oldConfig1;
      QString saveOldGroup = oldGroup;
      QString saveNewGroup = newGroup;
-     oldConfig1 = new KConfig(tmp2.name(), true, false);
+     oldConfig1 = new KConfig(tmp2.fileName(), KConfig::NoGlobals);
 
      // For all groups...
      QStringList grpList = oldConfig1->groupList();
@@ -934,24 +945,29 @@ void KonfUpdate::resetOptions()
 {
    m_bCopy = false;
    m_bOverwrite = false;
-   m_arguments = QString::null;
+   m_arguments.clear();
 }
 
 
 extern "C" KDE_EXPORT int kdemain(int argc, char **argv)
 {
-   KAboutData aboutData("kconf_update", I18N_NOOP("KConf Update"),
-                        "1.0.2",
-                        I18N_NOOP("KDE Tool for updating user configuration files"),
-                        KAboutData::License_GPL,
-                        "(c) 2001, Waldo Bastian");
+   KCmdLineOptions options;
+   options.add("debug", ki18n("Keep output results from scripts"));
+   options.add("check <update-file>", ki18n("Check whether config file itself requires updating"));
+   options.add("+[file]", ki18n("File to read update instructions from"));
 
-   aboutData.addAuthor("Waldo Bastian", 0, "bastian@kde.org");
+   KAboutData aboutData("kconf_update", 0, ki18n("KConf Update"),
+                        "1.0.2",
+                        ki18n("KDE Tool for updating user configuration files"),
+                        KAboutData::License_GPL,
+                        ki18n("(c) 2001, Waldo Bastian"));
+
+   aboutData.addAuthor(ki18n("Waldo Bastian"), KLocalizedString(), "bastian@kde.org");
 
    KCmdLineArgs::init(argc, argv, &aboutData);
    KCmdLineArgs::addCmdLineOptions(options);
 
-   KInstance instance(&aboutData);
+   KComponentData componentData(&aboutData);
 
    KonfUpdate konfUpdate;
 
