@@ -169,6 +169,10 @@ void Kded::messageFilter(const QDBusMessage &message)
   if (obj == "ksycoca")
      return; // Ignore this one.
 
+  // This happens when kded goes down and some modules try to clean up.
+  if (!self())
+     return;
+
   if (self()->m_dontLoad.value(obj, 0))
      return;
 
@@ -186,7 +190,7 @@ void Kded::initModules()
     if( !sessionUID.isEmpty() && uid_t( sessionUID.toInt() ) != getuid())
         kde_running = false;
      // Preload kded modules.
-     KService::List kdedModules = KServiceTypeTrader::self()->query("KDEDModule");
+     const KService::List kdedModules = KServiceTypeTrader::self()->query("KDEDModule");
      for(KService::List::ConstIterator it = kdedModules.begin(); it != kdedModules.end(); ++it)
      {
          KService::Ptr service = *it;
@@ -230,7 +234,7 @@ void Kded::loadSecondPhase()
      kDebug(7020) << "Loading second phase autoload";
      KSharedConfig::Ptr config = KGlobal::config();
      KService::List kdedModules = KServiceTypeTrader::self()->query("KDEDModule");
-     for(KService::List::ConstIterator it = kdedModules.begin(); it != kdedModules.end(); ++it)
+     for(KService::List::ConstIterator it = kdedModules.constBegin(); it != kdedModules.constEnd(); ++it)
      {
          KService::Ptr service = *it;
          bool autoload = service->property("X-KDE-Kded-autoload", QVariant::Bool).toBool();
@@ -382,8 +386,8 @@ void Kded::updateDirWatch()
            this, SLOT(dirDeleted(const QString&)));
 
   // For each resource
-  for( QStringList::ConstIterator it = m_allResourceDirs.begin();
-       it != m_allResourceDirs.end();
+  for( QStringList::ConstIterator it = m_allResourceDirs.constBegin();
+       it != m_allResourceDirs.constEnd();
        ++it )
   {
      readDirectory( *it );
@@ -605,7 +609,7 @@ KUpdateD::KUpdateD()
     QObject::connect( m_pDirWatch, SIGNAL(dirty(const QString&)),
            this, SLOT(slotNewUpdateFile()));
 
-    QStringList dirs = KGlobal::dirs()->findDirs("data", "kconf_update");
+    const QStringList dirs = KGlobal::dirs()->findDirs("data", "kconf_update");
     for( QStringList::ConstIterator it = dirs.begin();
          it != dirs.end();
          ++it )
@@ -737,7 +741,7 @@ extern "C" KDE_EXPORT int kdemain(int argc, char *argv[])
 {
      KAboutData aboutData( "kded" /*don't change this one to kded4! dbus registration should be org.kde.kded etc.*/,
         "kdelibs4", ki18n("KDE Daemon"),
-        "$Id: kded.cpp 825113 2008-06-27 13:30:46Z dfaure $",
+        "$Id: kded.cpp 901205 2008-12-24 16:51:16Z dfaure $",
         ki18n("KDE Daemon - triggers Sycoca database updates when needed"));
 
      KCmdLineOptions options;
