@@ -54,16 +54,11 @@ KCompletionBox::KCompletionBox( QWidget *parent )
     d->upwardBox       = false;
     d->emitSelected    = true;
 
-    setWindowFlags( Qt::Popup ); // calls setVisible, so must be done after initializations
+    setWindowFlags( Qt::ToolTip ); // calls setVisible, so must be done after initializations
 
     setLineWidth( 1 );
     setFrameStyle( QFrame::Box | QFrame::Plain );
 
-    if ( parent ) {
-        setFocusProxy( parent );
-    } else
-        setFocusPolicy( Qt::NoFocus );
-
     setVerticalScrollBarPolicy(Qt::ScrollBarAsNeeded);
     setHorizontalScrollBarPolicy(Qt::ScrollBarAlwaysOff);
 
@@ -113,14 +108,29 @@ void KCompletionBox::slotActivated( QListWidgetItem *item )
 bool KCompletionBox::eventFilter( QObject *o, QEvent *e )
 {
     int type = e->type();
+    QWidget *wid = qobject_cast<QWidget*>(o);
 
-    if ((o==this) && (type==QEvent::KeyPress)) {
-	//not sure if this is a work around for a Qt bug or a fix for changed qt behaviour, at least it makes the kcombobox work again for me
-	QApplication::sendEvent(d->m_parent,e);
-	return true;
+    if (o == this) {
+        return false;
     }
 
-    if ( o == d->m_parent ) {
+    if (wid && (wid == d->m_parent || wid->windowFlags() & Qt::Window) && 
+               (type == QEvent::Move || type == QEvent::Resize)) {
+        hide();
+        return false;
+    }
+
+    if (type == QEvent::MouseButtonPress && (wid && !isAncestorOf(wid))) {
+        if (!d->emitSelected && currentItem() && !qobject_cast<QScrollBar*>(o)) {
+          Q_ASSERT(currentItem());
+          emit currentTextChanged(currentItem()->text() );
+        }
+        hide();
+        e->accept();
+        return true;
+    }
+
+    if (wid && wid->isAncestorOf(d->m_parent)) {
         if ( isVisible() ) {
             if ( type == QEvent::KeyPress ) {
                 QKeyEvent *ev = static_cast<QKeyEvent *>( e );
@@ -192,8 +202,7 @@ bool KCompletionBox::eventFilter( QObject *o, QEvent *e )
                     default:
                         break;
                 }
-            }
-            else if ( type == QEvent::ShortcutOverride ) {
+            } else if ( type == QEvent::ShortcutOverride ) {
                 // Override any accelerators that match
                 // the key sequences we use here...
                 QKeyEvent *ev = static_cast<QKeyEvent *>( e );
@@ -228,44 +237,17 @@ bool KCompletionBox::eventFilter( QObject *o, QEvent *e )
                     default:
                         break;
                 }
-            }
-
-            // parent gets a click -> we hide
-            else if ( type == QEvent::Resize ||
-                      type == QEvent::Close || type == QEvent::Hide ) {
-                hide();
-            }
-            
-            else if ( type == QEvent::FocusOut ) {
+            } else if ( type == QEvent::FocusOut ) {
                   QFocusEvent* event = static_cast<QFocusEvent*>( e );
                   if (event->reason() != Qt::PopupFocusReason)
                     hide();
             }
-            
-        }
-    }
-
-    // any mouse-click on something else than "this" makes us hide
-    else if ( type == QEvent::MouseButtonPress ) {
-        QMouseEvent *ev = static_cast<QMouseEvent *>( e );
-        if ( !rect().contains( ev->pos() )) // this widget
-            hide();
-
-        if ( !d->emitSelected && currentItem() && !qobject_cast<QScrollBar*>(o) )
-        {
-          Q_ASSERT( currentItem() );
-
-          emit currentTextChanged( currentItem()->text() );
-          hide();
-          ev->accept();  // Consume the mouse click event...
-          return true;
         }
     }
 
     return KListWidget::eventFilter( o, e );
 }
 
-
 void KCompletionBox::popup()
 {
     if ( count() == 0 )
@@ -282,12 +264,6 @@ void KCompletionBox::popup()
         else if ( size().height() != sizeHint().height() )
             sizeAndPosition();
     }
-
-    // Make sure we give the focus back to the parent widget (ereslibre)
-    if ( parentWidget() ) {
-        QFocusEvent focusEvent( QEvent::FocusIn );
-        QApplication::sendEvent( parentWidget(), &focusEvent );
-    }
 }
 
 void KCompletionBox::sizeAndPosition()
@@ -362,14 +338,12 @@ void KCompletionBox::setVisible( bool visible )
 
 QRect KCompletionBox::calculateGeometry() const
 {
-    if (count() == 0)
+    QRect visualRect;
+    if (count() == 0 || !(visualRect = visualItemRect(item(0))).isValid())
         return QRect();
 
     int x = 0, y = 0;
-
-    Q_ASSERT( visualItemRect(item(0)).isValid() );
-
-    int ih = visualItemRect(item(0)).height();
+    int ih = visualRect.height();
     int h = qMin( 15 * ih, (int) count() * ih ) + 2*frameWidth();
 
     int w = (d->m_parent) ? d->m_parent->width() : KListWidget::minimumSizeHint().width();
