@@ -75,6 +75,8 @@ public:
         BuildState::operator=( m_stateStack.pop() );
     }
 
+    bool emptyState() const { return m_stateStack.isEmpty(); }
+
     QWidget *findRecursive( KXMLGUI::ContainerNode *node, bool tag );
     QList<QWidget*> findRecursive( KXMLGUI::ContainerNode *node, const QString &tagName );
     void applyActionProperties( const QDomElement &element );
@@ -204,6 +206,8 @@ void KXMLGUIFactory::addClient( KXMLGUIClient *client )
             client->factory()->removeClient( client ); //just in case someone does stupid things ;-)
     }
 
+    if (d->emptyState())
+        emit makingChanges(true);
     d->pushState();
 
 //    QTime dt; dt.start();
@@ -281,6 +285,8 @@ void KXMLGUIFactory::addClient( KXMLGUIClient *client )
     foreach (KXMLGUIClient *child, client->childClients())
         addClient( child );
 
+    if (d->emptyState())
+        emit makingChanges(false);
 /*
     QString unaddedActions;
     foreach (KActionCollection* ac, KActionCollection::allCollections())
@@ -303,6 +309,9 @@ void KXMLGUIFactory::removeClient( KXMLGUIClient *client )
     if ( !client || client->factory() != this )
         return;
 
+    if (d->emptyState())
+        emit makingChanges(true);
+
     // remove this client from our client list
     d->m_clients.removeAll( client );
 
@@ -344,6 +353,9 @@ void KXMLGUIFactory::removeClient( KXMLGUIClient *client )
 
     d->popState();
 
+    if (d->emptyState())
+        emit makingChanges(false);
+
     emit clientRemoved( client );
 }
 
