@@ -901,7 +901,9 @@ void KDirOperator::setUrl(const KUrl& _newurl, bool clearforward)
 
     if (!Private::isReadable(newurl)) {
         // maybe newurl is a file? check its parent directory
-        newurl.cd(QLatin1String(".."));
+        newurl.setPath(newurl.directory());
+        if (newurl.equals(d->currUrl, KUrl::CompareWithoutTrailingSlash))
+            return; // parent is current dir, nothing to do (fixes #173454, too)
         KIO::UDSEntry entry;
         bool res = KIO::NetAccess::stat(newurl, entry, this);
         KFileItem i(entry, newurl);
@@ -944,7 +946,9 @@ void KDirOperator::setUrl(const KUrl& _newurl, bool clearforward)
 
 void KDirOperator::updateDir()
 {
+    QApplication::setOverrideCursor(Qt::WaitCursor);
     d->dirLister->emitChanges();
+    QApplication::restoreOverrideCursor();
 }
 
 void KDirOperator::rereadDir()
@@ -1994,7 +1998,9 @@ void KDirOperator::Private::_k_slotPressed(const QModelIndex&)
     // Remember whether the left mouse button has been pressed, to prevent
     // that a right-click on an item opens an item (see _k_slotClicked(),
     // _k_slotDoubleClicked() and _k_openContextMenu()).
-    leftButtonPressed = (QApplication::mouseButtons() & Qt::LeftButton);
+    const Qt::KeyboardModifiers modifiers = QApplication::keyboardModifiers();
+    leftButtonPressed = (QApplication::mouseButtons() & Qt::LeftButton) &&
+                        !(modifiers & Qt::ShiftModifier) && !(modifiers & Qt::ControlModifier);
 }
 
 void KDirOperator::Private::_k_slotClicked(const QModelIndex& index)
@@ -2015,7 +2021,8 @@ void KDirOperator::Private::_k_slotActivated(const QModelIndex& index)
     KFileItem item = dirModel->itemForIndex(dirIndex);
     bool selectDir = false;
 
-    if (item.isNull())
+    const Qt::KeyboardModifiers modifiers = QApplication::keyboardModifiers();
+    if (item.isNull() || (modifiers & Qt::ShiftModifier) || (modifiers & Qt::ControlModifier))
         return;
 
     if (item.isDir()) {
