@@ -1,7 +1,7 @@
 /*
  * This file is part of the CSS implementation for KDE.
  *
- * Copyright (C) 1999 Lars Knoll (knoll@kde.org)
+ * Copyright 1999 Lars Knoll (knoll@kde.org)
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Library General Public
@@ -15,31 +15,22 @@
  *
  * You should have received a copy of the GNU Library General Public License
  * along with this library; see the file COPYING.LIB.  If not, write to
- * the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+ * Boston, MA 02110-1301, USA.
  *
- * $Id: csshelper.h,v 1.13 2003/04/17 16:49:26 mueller Exp $
  */
 #ifndef css_helper_h
 #define css_helper_h
 
-#include <qcolor.h>
-#include <qfont.h>
+#include <QtGui/QColor>
+#include <QtGui/QFont>
 
 #include "dom/dom_string.h"
 
-class QPaintDeviceMetrics;
-class KHTMLSettings;
-
-namespace DOM
-{
-    class CSSPrimitiveValueImpl;
-}
 
 namespace khtml
 {
-    class RenderStyle;
-
+    
     /*
      * mostly just removes the url("...") brace
      */
