@@ -16,20 +16,21 @@
  *
  *  You should have received a copy of the GNU Library General Public
  *  License along with this library; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
  */
 
 #ifndef _KJS_PROXY_H_
 #define _KJS_PROXY_H_
 
-#include <qvariant.h>
-#include <qstring.h>
+#include <QtCore/QVariant>
+#include <QtCore/QString>
 #include <sys/time.h>
 
 class KHTMLPart;
 
 namespace DOM {
   class Node;
+  class NodeImpl;
   class EventListener;
   class Event;
 }
@@ -38,7 +39,6 @@ namespace KJS {
   class List;
   class Interpreter;
   class Completion;
-  class KJSDebugWin;
 }
 
 namespace khtml {
@@ -57,11 +57,12 @@ public:
   virtual QVariant evaluate(QString filename, int baseLine, const QString &, const DOM::Node &n,
 			    KJS::Completion *completion = 0) = 0;
   virtual void clear() = 0;
-  virtual DOM::EventListener *createHTMLEventHandler(QString sourceUrl, QString name, QString code) = 0;
+  virtual DOM::EventListener *createHTMLEventHandler(QString sourceUrl, QString name, QString code, DOM::NodeImpl* node) = 0;
   virtual void finishedWithEvent(const DOM::Event &event) = 0;
   virtual KJS::Interpreter *interpreter() = 0;
 
   virtual void setDebugEnabled(bool enabled) = 0;
+  virtual bool debugEnabled() const = 0;
   virtual void showDebugWindow(bool show=true) = 0;
   virtual bool paused() const = 0;
   virtual void dataReceived() = 0;
@@ -70,18 +71,10 @@ public:
 
   khtml::ChildFrame *m_frame;
   int m_handlerLineno;
+
+  // Helper method, to access the private KHTMLPart::jScript()
+  static KJSProxy *proxy( KHTMLPart *part );
 };
 
-class KJSCPUGuard {
-public:
-  KJSCPUGuard() {}
-  void start(unsigned int msec=5000, unsigned int i_msec=10000);
-  void stop();
-private:
-  void (*oldAlarmHandler)(int);
-  static void alarmHandler(int);
-  static bool confirmTerminate();
-  itimerval oldtv; 
-};
 
 #endif
