@@ -4,7 +4,7 @@
  * Copyright (C) 1999 Lars Knoll (knoll@kde.org)
  *           (C) 1999 Antti Koivisto (koivisto@kde.org)
  *           (C) 2001 Dirk Mueller (mueller@kde.org)
- *           (C) 2002-2003 Apple Computer, Inc.
+ *           (C) 2003, 2004, 2005, 2006, 2007 Apple Computer, Inc.
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Library General Public
@@ -26,6 +26,7 @@
 #include "html/html_headimpl.h"
 #include "html/html_documentimpl.h"
 #include "xml/dom_textimpl.h"
+#include "xml/dom2_eventsimpl.h"
 
 #include "khtmlview.h"
 #include "khtml_part.h"
@@ -37,6 +38,7 @@
 #include "css/cssstyleselector.h"
 #include "css/css_stylesheetimpl.h"
 #include "css/csshelper.h"
+#include "css/css_mediaquery.h"
 
 #include "ecma/kjs_proxy.h"
 
@@ -79,8 +81,8 @@ void HTMLBaseElementImpl::removedFromDocument()
 
     // Since the document doesn't have a base element...
     // (This will break in the case of multiple base elements, but that's not valid anyway (?))
-    getDocument()->setBaseURL( KURL() );
-    getDocument()->setBaseTarget( QString::null );
+    document()->setBaseURL( KUrl() );
+    document()->setBaseTarget( QString() );
 }
 
 void HTMLBaseElementImpl::process()
@@ -88,11 +90,11 @@ void HTMLBaseElementImpl::process()
     if (!inDocument())
 	return;
 
-    if(!m_href.isEmpty() && getDocument()->view())
-	getDocument()->setBaseURL( KURL( getDocument()->view()->part()->url(), m_href.string() ) );
+    if(!m_href.isEmpty() && document()->view())
+	document()->setBaseURL( KUrl( document()->view()->part()->url(), m_href.string() ) );
 
     if(!m_target.isEmpty())
-	getDocument()->setBaseTarget( m_target.string() );
+	document()->setBaseTarget( m_target.string() );
 
     // ### should changing a document's base URL dynamically automatically update all images, stylesheets etc?
 }
@@ -116,7 +118,7 @@ void HTMLLinkElementImpl::parseAttribute(AttributeImpl *attr)
     switch (attr->id())
     {
     case ATTR_HREF:
-        m_url = getDocument()->completeURL( khtml::parseURL(attr->value()).string() );
+        m_url = document()->completeURL( khtml::parseURL(attr->value()).string() );
 	process();
         break;
     case ATTR_REL:
@@ -128,7 +130,7 @@ void HTMLLinkElementImpl::parseAttribute(AttributeImpl *attr)
         // stylesheet choice
         break;
     case ATTR_MEDIA:
-        m_media = attr->value().string().lower();
+        m_media = attr->value().string().toLower();
         process();
         break;
     case ATTR_DISABLED: {
@@ -137,17 +139,17 @@ void HTMLLinkElementImpl::parseAttribute(AttributeImpl *attr)
         if (m_oldisDisabled != m_isDisabled) {
             if (isLoading()) {
                 if (m_oldisDisabled)
-                    getDocument()->addPendingSheet();
+                    document()->addPendingSheet();
                 else if (!m_alternate)
-                    getDocument()->styleSheetLoaded();
+                    document()->styleSheetLoaded();
             }
             if (m_oldisDisabled) {
                 // enabling: if it's an alternate sheet, pretend it's not.
                 m_alternate = false;
             } else if (!m_alternate) {
                 // disabling: recheck alternate status
-                QString rel =  getAttribute(ATTR_REL).string().lower();
-                QString type = getAttribute(ATTR_TYPE).string().lower();
+                QString rel =  getAttribute(ATTR_REL).string().toLower();
+                QString type = getAttribute(ATTR_TYPE).string().toLower();
                 m_alternate = (type.contains("text/css") || rel.contains("stylesheet")) && rel.contains("alternate");
             }
             if (isLoading())
@@ -155,10 +157,10 @@ void HTMLLinkElementImpl::parseAttribute(AttributeImpl *attr)
             if (!m_sheet && !m_isDisabled) {
                 process();
                 if (isLoading() && m_alternate)
-                    getDocument()->addPendingSheet();
+                    document()->addPendingSheet();
                 m_alternate = false;
             } else
-                getDocument()->updateStyleSelector(); // Update the style selector.
+                document()->updateStyleSelector(); // Update the style selector.
         }
         break;
     }
@@ -172,52 +174,61 @@ void HTMLLinkElementImpl::process()
     if (!inDocument())
         return;
 
-    QString type = getAttribute(ATTR_TYPE).string().lower();
-    QString rel = getAttribute(ATTR_REL).string().lower();
+    QString type = getAttribute(ATTR_TYPE).string().toLower();
+    QString rel = getAttribute(ATTR_REL).string().toLower();
 
-    KHTMLPart* part = getDocument()->view() ? getDocument()->view()->part() : 0;
+    KHTMLPart* part = document()->view() ? document()->view()->part() : 0;
 
     // IE extension: location of small icon for locationbar / bookmarks
     // Uses both "shortcut icon" and "icon"
     if ( part && rel.contains("icon") && !m_url.isEmpty() && !part->parentPart())
-        part->browserExtension()->setIconURL( KURL(m_url.string()) );
+        part->browserExtension()->setIconUrl( KUrl(m_url.string()) );
 
     // Stylesheet
-    else if (!m_isDisabled &&
-        (type.contains("text/css") || rel.contains("stylesheet"))) {
+    else if (!m_isDisabled && rel.contains("stylesheet")) {
         // no need to load style sheets which aren't for the screen output
         // ### there may be in some situations e.g. for an editor or script to manipulate
-        if( m_media.isNull() || m_media.contains("screen") || m_media.contains("all") || m_media.contains("print") ) {
-            m_loading = true;
+        khtml::MediaQueryEvaluator allEval(true);
+        khtml::MediaQueryEvaluator screenEval("screen", true);
+        khtml::MediaQueryEvaluator printEval("print", true);
+        MediaListImpl* media = new MediaListImpl((CSSStyleSheetImpl*)0, m_media, true);
+        media->ref();
+        if (allEval.eval(media) || screenEval.eval(media) || printEval.eval(media)) {
             // Add ourselves as a pending sheet, but only if we aren't an alternate
             // stylesheet.  Alternate stylesheets don't hold up render tree construction.
             m_alternate = rel.contains("alternate");
             if (!isAlternate())
-                getDocument()->addPendingSheet();
+                document()->addPendingSheet();
 
             QString chset = getAttribute( ATTR_CHARSET ).string();
             // set chset to charset of referring document when attribute CHARSET is absent.
             // http://www.w3.org/TR/CSS21/syndata.html(4.4)
             if (chset.isEmpty() && part) chset = part->encoding();
-            if (m_cachedSheet)
+            if (m_cachedSheet) {
+                if (m_loading)
+                    document()->styleSheetLoaded();
 		m_cachedSheet->deref(this);
-            m_cachedSheet = getDocument()->docLoader()->requestStyleSheet(m_url, chset);
+            }
+            m_loading = true;
+            m_cachedSheet = document()->docLoader()->requestStyleSheet(m_url, chset);
             if (m_cachedSheet) {
                 m_isCSSSheet = true;
 		m_cachedSheet->ref(this);
             }
             else if (!isAlternate()) {
                 // Error requesting sheet; decrement pending sheet count
-                getDocument()->styleSheetLoaded();
+                m_loading = false;
+                document()->styleSheetLoaded();
             }
         }
+        media->deref();
     }
     else if (m_sheet) {
 	// we no longer contain a stylesheet, e.g. perhaps rel or type was changed
 	m_sheet->deref();
 	m_sheet = 0;
         m_isCSSSheet = false;
-	getDocument()->updateStyleSelector();
+	document()->updateStyleSelector();
     }
 }
 
@@ -230,19 +241,23 @@ void HTMLLinkElementImpl::insertedIntoDocument()
 void HTMLLinkElementImpl::removedFromDocument()
 {
     HTMLElementImpl::removedFromDocument();
-    getDocument()->updateStyleSelector();
+    document()->updateStyleSelector();
 }
 
-void HTMLLinkElementImpl::setStyleSheet(const DOM::DOMString &url, const DOM::DOMString &sheetStr, const DOM::DOMString &charset)
+void HTMLLinkElementImpl::setStyleSheet(const DOM::DOMString &url, const DOM::DOMString &sheetStr, const DOM::DOMString &charset, const DOM::DOMString &mimetype)
 {
     if (m_sheet)
         m_sheet->deref();
+    bool strict = !document()->inCompatMode();
+    DOMString sheet = sheetStr;
+    if (strict && !khtml::isAcceptableCSSMimetype(mimetype))
+         sheet = "";
     m_sheet = new CSSStyleSheetImpl(this, url);
     m_sheet->ref();
     m_sheet->setCharset(charset);
-    m_sheet->parseString( sheetStr, !getDocument()->inCompatMode() );
+    m_sheet->parseString( sheet, strict );
 
-    MediaListImpl *media = new MediaListImpl( m_sheet, m_media );
+    MediaListImpl *media = new MediaListImpl( (CSSStyleSheetImpl*)0, m_media );
     m_sheet->setMedia( media );
 
     finished();
@@ -254,13 +269,13 @@ void HTMLLinkElementImpl::finished()
 
     // Tell the doc about the sheet.
     if (!isLoading() && !isDisabled() && !isAlternate())
-        getDocument()->styleSheetLoaded();
+        document()->styleSheetLoaded();
 
     // ### major inefficiency, but currently necessary for proper
     // alternate styles support. don't recalc the styleselector
     // when nothing actually changed!
     if ( isAlternate() && m_sheet && !isDisabled())
-        getDocument()->updateStyleSelector();
+        document()->updateStyleSelector();
 }
 
 void HTMLLinkElementImpl::error( int, const QString& )
@@ -270,17 +285,29 @@ void HTMLLinkElementImpl::error( int, const QString& )
 
 bool HTMLLinkElementImpl::isLoading() const
 {
-//    kdDebug( 6030 ) << "link: checking if loading!" << endl;
+//    kDebug( 6030 ) << "link: checking if loading!";
     if(m_loading) return true;
     if(!m_sheet) return false;
     //if(!m_sheet->isCSSStyleSheet()) return false;
     return static_cast<CSSStyleSheetImpl *>(m_sheet)->isLoading();
 }
 
-void HTMLLinkElementImpl::sheetLoaded()
+bool HTMLLinkElementImpl::checkAddPendingSheet()
 {
-    if (!isLoading() && !isDisabled() && !isAlternate())
-        getDocument()->styleSheetLoaded();
+    if (!isLoading() && !isDisabled() && !isAlternate()) {
+        document()->addPendingSheet();
+        return true;
+    }
+    return false;
+}
+
+bool HTMLLinkElementImpl::checkRemovePendingSheet()
+{
+    if (!isLoading() && !isDisabled() && !isAlternate()) {
+        document()->styleSheetLoaded();
+	return true;
+    }
+    return false;
 }
 
 // -------------------------------------------------------------------------
@@ -318,12 +345,12 @@ void HTMLMetaElementImpl::process()
     // Get the document to process the tag, but only if we're actually part of DOM tree (changing a meta tag while
     // it's not in the tree shouldn't have any effect on the document)
     if (inDocument() && !m_equiv.isNull() && !m_content.isNull())
-	getDocument()->processHttpEquiv(m_equiv,m_content);
+	document()->processHttpEquiv(m_equiv,m_content);
 }
 
 // -------------------------------------------------------------------------
 
-HTMLScriptElementImpl::HTMLScriptElementImpl(DocumentPtr *doc)
+HTMLScriptElementImpl::HTMLScriptElementImpl(DocumentImpl *doc)
     : HTMLElementImpl(doc), m_cachedScript(0), m_createdByParser(false), m_evaluated(false)
 {
 }
@@ -339,6 +366,31 @@ NodeImpl::Id HTMLScriptElementImpl::id() const
     return ID_SCRIPT;
 }
 
+void HTMLScriptElementImpl::parseAttribute(AttributeImpl *attr)
+{
+    switch(attr->id())
+    {
+    case ATTR_ONLOAD:
+        setHTMLEventListener(EventImpl::LOAD_EVENT,
+            document()->createHTMLEventListener(attr->value().string(), "onload", this));
+        break;
+    case ATTR_SRC: {
+        // We want to evaluate scripts on src attr change when a fresh script element
+        // is inserted into document, and then has its source changed -after-.
+        // If the source is manipulated while we're outside the document,
+        // we'll only start doing things once we get insertedIntoDocument()
+        if (m_evaluated || m_cachedScript || m_createdByParser || !inDocument())
+            return;
+        DOMString url = attr->value();
+        if (!url.isEmpty())
+            loadFromUrl(url);
+        break;
+    }
+    default:
+        HTMLElementImpl::parseAttribute(attr);
+    }
+}
+
 bool HTMLScriptElementImpl::isURLAttribute(AttributeImpl *attr) const
 {
     return attr->id() == ATTR_SRC;
@@ -350,7 +402,15 @@ void HTMLScriptElementImpl::childrenChanged()
     // and the script element has been inserted in the document
     // we evaluate the script.
     if (!m_createdByParser && inDocument() && firstChild())
-        evaluateScript(getDocument()->URL().url(), text());
+        evaluateScript(document()->URL().url(), text());
+}
+
+void HTMLScriptElementImpl::loadFromUrl(const DOMString &url)
+{
+    QString charset = getAttribute(ATTR_CHARSET).string();
+    m_cachedScript = document()->docLoader()->requestScript(url, charset);
+    if (m_cachedScript)
+        m_cachedScript->ref(this);    
 }
 
 void HTMLScriptElementImpl::insertedIntoDocument()
@@ -362,12 +422,9 @@ void HTMLScriptElementImpl::insertedIntoDocument()
     if (m_createdByParser)
         return;
 
-    QString url = getAttribute(ATTR_SRC).string();
+    DOMString url = getAttribute(ATTR_SRC).string();
     if (!url.isEmpty()) {
-        QString charset = getAttribute(ATTR_CHARSET).string();
-        m_cachedScript = getDocument()->docLoader()->requestScript(DOMString(url), charset);
-        if (m_cachedScript)
-            m_cachedScript->ref(this);
+        loadFromUrl(url);
         return;
     }
 
@@ -376,7 +433,7 @@ void HTMLScriptElementImpl::insertedIntoDocument()
     // it should be evaluated, and evaluateScript only evaluates a script once.
     DOMString scriptString = text();
     if (!scriptString.isEmpty())
-        evaluateScript(getDocument()->URL().url(), scriptString);
+        evaluateScript(document()->URL().url(), scriptString);
 }
 
 void HTMLScriptElementImpl::removedFromDocument()
@@ -400,7 +457,12 @@ void HTMLScriptElementImpl::notifyFinished(CachedObject* o)
     cs->deref(this);
     m_cachedScript = 0;
 
-    evaluateScript(URL, script);
+    ref(); // Pin so we don't destroy oursleves.
+    if (!cs->hadError()) {
+        evaluateScript(URL, script);
+        dispatchHTMLEvent(EventImpl::LOAD_EVENT, false, false);
+    }
+    deref();
 }
 
 void HTMLScriptElementImpl::evaluateScript(const QString &URL, const DOMString &script)
@@ -408,7 +470,7 @@ void HTMLScriptElementImpl::evaluateScript(const QString &URL, const DOMString &
     if (m_evaluated)
         return;
 
-    KHTMLPart *part = getDocument()->part();
+    KHTMLPart *part = document()->part();
     if (part) {
         KJSProxy *proxy = KJSProxy::proxy(part);
         if (proxy) {
@@ -445,7 +507,7 @@ void HTMLScriptElementImpl::setText(const DOMString &value)
         removeChildren();
     }
 
-    appendChild(getDocument()->createTextNode(value.implementation()), exceptioncode);
+    appendChild(document()->createTextNode(value.implementation()), exceptioncode);
 }
 
 DOMString HTMLScriptElementImpl::htmlFor() const
@@ -492,7 +554,7 @@ void HTMLScriptElementImpl::setDefer(bool defer)
 
 DOMString HTMLScriptElementImpl::src() const
 {
-    return getDocument()->completeURL(getAttribute(ATTR_SRC).string());
+    return document()->completeURL(getAttribute(ATTR_SRC).string());
 }
 
 void HTMLScriptElementImpl::setSrc(const DOMString &value)
@@ -531,7 +593,7 @@ void HTMLStyleElementImpl::parseAttribute(AttributeImpl *attr)
         m_type = attr->value().lower();
         break;
     case ATTR_MEDIA:
-        m_media = attr->value().string().lower();
+        m_media = attr->value().string().toLower();
         break;
     default:
         HTMLElementImpl::parseAttribute(attr);
@@ -542,14 +604,14 @@ void HTMLStyleElementImpl::insertedIntoDocument()
 {
     HTMLElementImpl::insertedIntoDocument();
     if (m_sheet)
-        getDocument()->updateStyleSelector();
+        document()->updateStyleSelector();
 }
 
 void HTMLStyleElementImpl::removedFromDocument()
 {
     HTMLElementImpl::removedFromDocument();
     if (m_sheet)
-        getDocument()->updateStyleSelector();
+        document()->updateStyleSelector();
 }
 
 void HTMLStyleElementImpl::childrenChanged()
@@ -571,20 +633,26 @@ void HTMLStyleElementImpl::childrenChanged()
     }
 
     m_loading = false;
-    if ((m_type.isEmpty() || m_type == "text/css") // Type must be empty or CSS
-         && (m_media.isNull() || m_media.contains("screen") || m_media.contains("all") || m_media.contains("print"))) {
-        getDocument()->addPendingSheet();
-        m_loading = true;
-        m_sheet = new CSSStyleSheetImpl(this);
-        m_sheet->ref();
-        m_sheet->parseString( text, !getDocument()->inCompatMode() );
-        MediaListImpl* media = new MediaListImpl( m_sheet, DOMString( m_media ) );
-        m_sheet->setMedia( media );
-        m_loading = false;
+    if (m_type.isEmpty() || m_type == "text/css") // Type must be empty or CSS
+    {
+        MediaListImpl* media = new MediaListImpl((CSSStyleSheetImpl*)0, m_media, true);
+        media->ref();
+        khtml::MediaQueryEvaluator screenEval("screen", true);
+        khtml::MediaQueryEvaluator printEval("print", true);
+        if (screenEval.eval(media) || printEval.eval(media)) {
+            document()->addPendingSheet();
+            m_loading = true;
+            m_sheet = new CSSStyleSheetImpl(this);
+            m_sheet->ref();
+            m_sheet->parseString( text, !document()->inCompatMode() );
+            m_sheet->setMedia( media );
+            m_loading = false;
+        }
+        media->deref();
     }
 
     if (!isLoading() && m_sheet)
-        getDocument()->styleSheetLoaded();
+        document()->styleSheetLoaded();
 }
 
 bool HTMLStyleElementImpl::isLoading() const
@@ -594,10 +662,22 @@ bool HTMLStyleElementImpl::isLoading() const
     return static_cast<CSSStyleSheetImpl *>(m_sheet)->isLoading();
 }
 
-void HTMLStyleElementImpl::sheetLoaded()
+bool HTMLStyleElementImpl::checkRemovePendingSheet()
 {
-    if (!isLoading())
-        getDocument()->styleSheetLoaded();
+    if (!isLoading()) {
+        document()->styleSheetLoaded();
+        return true;
+    }
+    return false;
+}
+
+bool HTMLStyleElementImpl::checkAddPendingSheet()
+{
+    if (!isLoading()) {
+        document()->addPendingSheet();
+        return true;
+    }
+    return false;
 }
 
 // -------------------------------------------------------------------------
@@ -617,7 +697,7 @@ void HTMLTitleElementImpl::childrenChanged()
 	    m_title += c->nodeValue();
     }
     if ( !m_title.isEmpty() && inDocument())
-        getDocument()->setTitle(m_title);
+        document()->setTitle(m_title);
 }
 
 DOMString HTMLTitleElementImpl::text()
@@ -644,6 +724,6 @@ void HTMLTitleElementImpl::setText( const DOMString& str )
         delete nl;
     }
     // No child text node found, creating one
-    DOM::TextImpl* t = getDocument()->createTextNode(str.implementation());
+    DOM::TextImpl* t = document()->createTextNode(str.implementation());
     appendChild(t, exceptioncode);
 }
