@@ -224,7 +224,7 @@ void HTMLBRElementImpl::attach()
     assert(!m_render);
     assert(parentNode());
 
-    if (parentNode()->renderer()) {
+    if (parentNode()->renderer() && parentNode()->renderer()->childAllowed()) {
         RenderStyle* style = document()->styleSelector()->styleForElement( this );
         style->ref();
         if( style->display() != NONE ) {
@@ -250,7 +250,7 @@ void HTMLWBRElementImpl::attach()
     assert(!m_render);
     assert(parentNode());
 
-    if (parentNode()->renderer()) {
+    if (parentNode()->renderer() && parentNode()->renderer()->childAllowed()) {
         RenderStyle* style = document()->styleSelector()->styleForElement( this );
         style->ref();
         if( style->display() != NONE ) {
