@@ -193,6 +193,7 @@ public final class KJASAppletStub
                         app.setBounds( 0, 0, appletSize.width, appletSize.height );
                     else
                         app.setBounds( 0, 0, panel.getSize().width, panel.getSize().height );
+                    active = true;
                     app.init();
                     loader.removeStatusListener(panel);
                     // stop the loading... animation 
@@ -200,7 +201,6 @@ public final class KJASAppletStub
                     app.setVisible(true);
                     break;
                 case STARTED:
-                    active = true;
                     app.start();
                     frame.validate();
                     app.repaint();
@@ -462,10 +462,7 @@ public final class KJASAppletStub
     * @return true if the applet has been completely loaded.
     */
     boolean isLoaded() {
-        if (runThread == null)
-            return false;
-        int state = runThread.getAppletState();
-        return (state >= INSTANCIATED && state < DESTROYED);
+        return runThread != null && runThread.getAppletState() >= INSTANCIATED;
     }
     
     public void appletResize( int width, int height )
