@@ -32,10 +32,11 @@ enum WidthType { Width, MinWidth, MaxWidth };
 enum HeightType { Height, MinHeight, MaxHeight };
 
 class RenderBlock;
+class RenderTableCell;
 
 class RenderBox : public RenderContainer
 {
-
+    friend class RenderTableCell;
 
 // combines ElemImpl & PosElImpl (all rendering objects are positioned)
 // should contain all border and padding handling
@@ -54,6 +55,10 @@ public:
 
     virtual InlineBox* createInlineBox(bool makePlaceHolderBox, bool isRootLineBox);
     virtual void deleteInlineBoxes(RenderArena* arena=0);
+    virtual void dirtyInlineBoxes(bool fullLayout, bool isRootLineBox = false);
+    virtual void removeInlineBox(InlineBox* _box) { if (m_placeHolderBox == _box) m_placeHolderBox = 0; }
+    
+    virtual void removeChild(RenderObject*);
 
     virtual void detach();
 
@@ -91,7 +96,7 @@ public:
 
     virtual void repaintRectangle(int x, int y, int w, int h, Priority p=NormalPriority, bool f=false);
 
-    virtual short containingBlockWidth() const;
+    virtual short containingBlockWidth(RenderObject* providedCB=0) const;
     void relativePositionOffset(int &tx, int &ty) const;
 
     virtual void calcWidth();
@@ -119,11 +124,17 @@ public:
 
     virtual int pageTopAfter(int y) const;
     virtual int crossesPageBreak(int t, int b) const;
+    
+    virtual bool handleEvent(const DOM::EventImpl& ev);
 
     int calcBoxWidth(int w) const;
     int calcBoxHeight(int h) const;
     int calcContentWidth(int w) const;
     int calcContentHeight(int h) const;
+    
+    InlineBox *placeHolderBox() { return m_placeHolderBox; }
+    void setPlaceHolderBox(InlineBox* placeHolder) { m_placeHolderBox = placeHolder; /* assert !m_placeHolderBox */ }
+    QRect getFixedBackgroundImageRect( const BackgroundLayer* bglayer, int& sx, int& sy, int& scaledImageWidth, int& scaledImageHeight );
 
 protected:
     int calcWidthUsing(WidthType widthType, int cw, LengthType& lengthType);
@@ -133,7 +144,7 @@ protected:
     int calcPercentageHeight(const Length& height, bool treatAsReplaced = false) const;
     int availableHeightUsing(const Length& h) const;
     int availableWidthUsing(const Length& w) const;
-    int calcImplicitHeight() const;
+    int calcImplicitContentHeight() const;
     bool hasImplicitHeight() const {
         return isPositioned() && !style()->top().isVariable() && !style()->bottom().isVariable();
     }
@@ -142,13 +153,12 @@ protected:
     virtual void paintBoxDecorations(PaintInfo& paintInfo, int _tx, int _ty);
     void paintRootBoxDecorations( PaintInfo& paintInfo, int _tx, int _ty);
 
-    void paintBackgrounds(QPainter *p, const QColor& c, const BackgroundLayer* bgLayer, int clipy, int cliph, int _tx, int _ty, int w, int h);
-    void paintBackground(QPainter *p, const QColor& c, const BackgroundLayer* bgLayer, int clipy, int cliph, int _tx, int _ty, int w, int h);
-
+    void paintAllBackgrounds(QPainter *p, const QColor& c, const BackgroundLayer* bgLayer, QRect clipr, int _tx, int _ty, int w, int h);
+    virtual void paintOneBackground(QPainter *p, const QColor& c, const BackgroundLayer* bgLayer, QRect clipr, int _tx, int _ty, int w, int h);
     virtual void paintBackgroundExtended(QPainter* /*p*/, const QColor& /*c*/, const BackgroundLayer* /*bgLayer*/,
-                                         int /*clipy*/, int /*cliph*/, int /*_tx*/, int /*_ty*/,
-                                         int /*w*/, int /*height*/, int /*bleft*/, int /*bright*/, int /*pleft*/, int /*pright*/ );
-
+                                         QRect clipr, int /*_tx*/, int /*_ty*/,
+                                         int /*w*/, int /*height*/, int /*bleft*/, int /*bright*/, int /*pleft*/, int /*pright*/,
+                                         int /*btop*/, int /*bbottom*/, int /*ptop*/, int /*pbottom*/ );
     void outlineBox(QPainter *p, int _tx, int _ty, const char *color = "red");
 
     void calcAbsoluteHorizontal();
@@ -165,10 +175,11 @@ protected:
     void calcAbsoluteVerticalReplaced();
     void calcAbsoluteHorizontalReplaced();
 
-    QRect getOverflowClipRect(int tx, int ty);
-    QRect getClipRect(int tx, int ty);
+    QRect overflowClipRect(int tx, int ty);
+    QRect clipRect(int tx, int ty);
 
     void restructureParentFlow();
+    void detachRemainingChildren();
 
 
     // the actual height of the contents + borders + padding (border-box)
