@@ -27,10 +27,10 @@
 #include "render_image.h"
 #include "render_canvas.h"
 
-#include <qdrawutil.h>
-#include <qpainter.h>
+#include <QtGui/qdrawutil.h>
+#include <QtGui/QPainter>
+#include <QtGui/QApplication>
 
-#include <kapplication.h>
 #include <kdebug.h>
 #include <kglobalsettings.h>
 
@@ -48,23 +48,26 @@
 #include "khtmlview.h"
 #include "khtml_part.h"
 #include <math.h>
+#include "imload/imagepainter.h"
 
 #include "loading_icon.cpp"
 
 using namespace DOM;
 using namespace khtml;
+using namespace khtmlImLoad;
 
 // -------------------------------------------------------------------------
 
 RenderImage::RenderImage(NodeImpl *_element)
     : RenderReplaced(_element)
 {
-    m_oldImage = m_cachedImage = 0;
+    m_cachedImage  = 0;
+    m_imagePainter = 0;
 
     m_selectionState = SelectionNone;
     berrorPic = false;
 
-    const KHTMLSettings *settings = _element->getDocument()->view()->part()->settings();
+    const KHTMLSettings *settings = _element->document()->view()->part()->settings();
     bUnfinishedImageFrame = settings->unfinishedImageFrame();
 
     setIntrinsicWidth( 0 );
@@ -73,14 +76,14 @@ RenderImage::RenderImage(NodeImpl *_element)
 
 RenderImage::~RenderImage()
 {
+    delete m_imagePainter;
     if(m_cachedImage) m_cachedImage->deref( this );
-    if(m_oldImage) m_oldImage->deref( this );
 }
 
-QPixmap RenderImage::pixmap() const
-{
-    return m_cachedImage ? m_cachedImage->pixmap() : QPixmap();
-}
+// QPixmap RenderImage::pixmap() const
+// {
+//     return image ? image->pixmap() : QPixmap();
+// }
 
 void RenderImage::setStyle(RenderStyle* _style)
 {
@@ -96,28 +99,30 @@ void RenderImage::setContentObject(CachedObject* co )
         updateImage( static_cast<CachedImage*>( co ) );
 }
 
-void RenderImage::setPixmap( const QPixmap &p, const QRect& r, CachedImage *o)
+void RenderImage::updatePixmap( const QRect& r, CachedImage *o)
 {
-    if ( o == m_oldImage )
-        return;
-
     if(o != m_cachedImage) {
-        RenderReplaced::setPixmap(p, r, o);
+        RenderReplaced::updatePixmap(r, o);
         return;
     }
 
     bool iwchanged = false;
 
     if(o->isErrorImage()) {
-        int iw = p.width() + 8;
-        int ih = p.height() + 8;
+        int iw = o->pixmap_size().width() + 8;
+        int ih = o->pixmap_size().height() + 8;
 
         // we have an alt and the user meant it (its not a text we invented)
         if ( element() && !alt.isEmpty() && !element()->getAttribute( ATTR_ALT ).isNull()) {
             const QFontMetrics &fm = style()->fontMetrics();
-            QRect br = fm.boundingRect (  0, 0, 1024, 256, Qt::AlignAuto|Qt::WordBreak, alt.string() );
+            QRect br = fm.boundingRect (  0, 0, 1024, 256, Qt::AlignLeft|Qt::TextWordWrap, alt.string() );
             if ( br.width() > iw )
-                iw = br.width();
+                //iw = br.width();
+#ifdef __GNUC__
+  #warning "KDE4: hack for testregression, remove (use above instead) when main branch"
+#endif
+              iw = br.width() + qMax(-fm.minLeftBearing(), 0) + qMax(-fm.minRightBearing(), 0);
+
             if ( br.height() > ih )
                 ih = br.height();
         }
@@ -184,24 +189,12 @@ void RenderImage::setPixmap( const QPixmap &p, const QRect& r, CachedImage *o)
     }
     else
     {
-        bool completeRepaint = !resizeCache.isNull();
         int cHeight = contentHeight();
-        int scaledHeight = intrinsicHeight() ? ((o->valid_rect().height()*cHeight)/intrinsicHeight()) : 0;
+        //### FIXME!
+        //int scaledHeight = intrinsicHeight() ? ((o->valid_rect().height()*cHeight)/intrinsicHeight()) : 0;
 
-        // don't bog down X server doing xforms
-        if(completeRepaint && cHeight >= 5 &&  o->valid_rect().height() < intrinsicHeight() &&
-           (scaledHeight / (cHeight/5) == resizeCache.height() / (cHeight/5)))
-            return;
-
-        resizeCache = QPixmap(); // for resized animations
-
-        if(completeRepaint)
-            repaintRectangle(borderLeft()+paddingLeft(), borderTop()+paddingTop(), contentWidth(), contentHeight());
-        else
-        {
-            repaintRectangle(r.x() + borderLeft() + paddingLeft(), r.y() + borderTop() + paddingTop(),
-                             r.width(), r.height());
-        }
+        repaintRectangle(r.x() + borderLeft() + paddingLeft(), r.y() + borderTop() + paddingTop(),
+                            r.width(), r.height());
     }
 }
 
@@ -234,11 +227,6 @@ void RenderImage::paint(PaintInfo& paintInfo, int _tx, int _ty)
     if (!canvas()->printImages())
         return;
 
-    CachedImage* i = m_oldImage && m_oldImage->valid_rect().size() == m_oldImage->pixmap_size() &&
-                     m_oldImage->pixmap_size().width()  == intrinsicWidth() &&
-                     m_oldImage->pixmap_size().height() == intrinsicHeight()
-                     ? m_oldImage : m_cachedImage;
-
     // paint frame around image as long as it is not completely loaded from web.
     if (bUnfinishedImageFrame && paintInfo.phase == PaintActionForeground && cWidth > 2 && cHeight > 2 && !complete()) {
         static QPixmap *loadingIcon;
@@ -247,7 +235,7 @@ void RenderImage::paint(PaintInfo& paintInfo, int _tx, int _ty)
                     (hasSufficientContrast(Qt::white, bg) ? Qt::white : Qt::black);
 	paintInfo.p->setPen(QPen(fg, 1));
 	paintInfo.p->setBrush( Qt::NoBrush );
-	paintInfo.p->drawRect(_tx, _ty, m_width, m_height);
+	paintInfo.p->drawRect(_tx, _ty, m_width - 1, m_height - 1);
         if (!(m_width <= 5 || m_height <= 5)) {
             if (!loadingIcon) {
                 loadingIcon = new QPixmap();
@@ -258,7 +246,9 @@ void RenderImage::paint(PaintInfo& paintInfo, int _tx, int _ty)
 
     }
 
-    //kdDebug( 6040 ) << "    contents (" << contentWidth << "/" << contentHeight << ") border=" << borderLeft() << " padding=" << paddingLeft() << endl;
+    CachedImage* i = m_cachedImage;
+
+    //kDebug( 6040 ) << "    contents (" << contentWidth << "/" << contentHeight << ") border=" << borderLeft() << " padding=" << paddingLeft();
     if ( !i || berrorPic)
     {
         if(cWidth > 2 && cHeight > 2)
@@ -266,15 +256,17 @@ void RenderImage::paint(PaintInfo& paintInfo, int _tx, int _ty)
             if ( !berrorPic ) {
                 //qDebug("qDrawShadePanel %d/%d/%d/%d", _tx + leftBorder, _ty + topBorder, cWidth, cHeight);
                 qDrawShadePanel( paintInfo.p, _tx + leftBorder + leftPad, _ty + topBorder + topPad, cWidth, cHeight,
-                                 KApplication::palette().inactive(), true, 1 );
+                                 QApplication::palette(), true, 1 );
             }
-            QPixmap const* pix = i ? &i->pixmap() : 0;
-            if(berrorPic && pix && (cWidth >= pix->width()+4) && (cHeight >= pix->height()+4) )
+
+            QPixmap pix = *Cache::brokenPixmap;
+            if(berrorPic && (cWidth >= pix.width()+4) && (cHeight >= pix.height()+4) )
             {
-                QRect r(pix->rect());
+                QRect r(pix.rect());
                 r = r.intersect(QRect(0, 0, cWidth-4, cHeight-4));
-                paintInfo.p->drawPixmap( QPoint( _tx + leftBorder + leftPad+2, _ty + topBorder + topPad+2), *pix, r );
+                paintInfo.p->drawPixmap( QPoint( _tx + leftBorder + leftPad+2, _ty + topBorder + topPad+2), pix, r );
             }
+
             if(!alt.isEmpty()) {
                 QString text = alt.string();
                 paintInfo.p->setFont(style()->font());
@@ -282,77 +274,69 @@ void RenderImage::paint(PaintInfo& paintInfo, int _tx, int _ty)
                 int ax = _tx + leftBorder + leftPad + 2;
                 int ay = _ty + topBorder + topPad + 2;
                 const QFontMetrics &fm = style()->fontMetrics();
+
+//BEGIN HACK
+#ifdef __GNUC__
+  #warning "KDE4: hack for testregression, remove when main branch"
+#endif
+                ax     += qMax(-fm.minLeftBearing(), 0);
+                cWidth -= qMax(-fm.minLeftBearing(), 0);
+
+//END HACK
                 if (cWidth>5 && cHeight>=fm.height())
-                    paintInfo.p->drawText(ax, ay+1, cWidth - 4, cHeight - 4, Qt::WordBreak, text );
+                    paintInfo.p->drawText(ax, ay+1, cWidth - 4, cHeight - 4, Qt::TextWordWrap, text );
             }
         }
     }
-    else if (i && !i->isTransparent())
+    else if (i && !i->isTransparent() &&
+             i->image()->size().width() && i->image()->size().height())
     {
         paintInfo.p->setPen( Qt::black ); // used for bitmaps
-        const QPixmap& pix = i->pixmap();
-        if ( (cWidth != intrinsicWidth() ||  cHeight != intrinsicHeight()) &&
-             pix.width() > 0 && pix.height() > 0 && i->valid_rect().isValid())
-        {
-            if (resizeCache.isNull() && cWidth && cHeight && intrinsicWidth() && intrinsicHeight())
-            {
-                QRect scaledrect(i->valid_rect());
-//                 kdDebug(6040) << "time elapsed: " << dt->elapsed() << endl;
-//                  kdDebug( 6040 ) << "have to scale: " << endl;
-//                  qDebug("cw=%d ch=%d  pw=%d ph=%d  rcw=%d, rch=%d",
-//                          cWidth, cHeight, intrinsicWidth(), intrinsicHeight(), resizeCache.width(), resizeCache.height());
-                QWMatrix matrix;
-                matrix.scale( (float)(cWidth)/intrinsicWidth(),
-                              (float)(cHeight)/intrinsicHeight() );
-                resizeCache = pix.xForm( matrix );
-                scaledrect.setWidth( ( cWidth*scaledrect.width() ) / intrinsicWidth() );
-                scaledrect.setHeight( ( cHeight*scaledrect.height() ) / intrinsicHeight() );
-//                   qDebug("resizeCache size: %d/%d", resizeCache.width(), resizeCache.height());
-//                   qDebug("valid: %d/%d, scaled: %d/%d",
-//                          i->valid_rect().width(), i->valid_rect().height(),
-//                          scaledrect.width(), scaledrect.height());
+        //const QPixmap& pix = i->pixmap();
+        if (!m_imagePainter)
+            m_imagePainter = new ImagePainter(i->image());
 
-                // sometimes scaledrect.width/height are off by one because
-                // of rounding errors. if the i is fully loaded, we
-                // make sure that we don't do unnecessary resizes during painting
-                QSize s(scaledrect.size());
-                if(i->valid_rect().size() == QSize( intrinsicWidth(), intrinsicHeight() )) // fully loaded
-                    s = QSize(cWidth, cHeight);
-                if(kAbs(s.width() - cWidth) < 2) // rounding errors
-                    s.setWidth(cWidth);
-                if(resizeCache.size() != s)
-                    resizeCache.resize(s);
+       // If we have a scaled  painter we want to handle the resizing ourselves, so figure out the scaled size,
+       QTransform painterTransform = paintInfo.p->transform();
 
-                paintInfo.p->drawPixmap( QPoint( _tx + leftBorder + leftPad, _ty + topBorder + topPad),
-                               resizeCache, scaledrect );
-            }
-            else
-                paintInfo.p->drawPixmap( QPoint( _tx + leftBorder + leftPad, _ty + topBorder + topPad), resizeCache );
-        }
-        else
-        {
-            // we might be just about switching images
-            QRect rect(i->valid_rect().isValid() ? i->valid_rect()
-                       : QRect(0, 0, intrinsicWidth(), intrinsicHeight()));
+       bool scaled = painterTransform.isScaling() && !painterTransform.isRotating();
 
-            QPoint offs( _tx + leftBorder + leftPad, _ty + topBorder + topPad);
-//             qDebug("normal paint rect %d/%d/%d/%d", rect.x(), rect.y(), rect.width(), rect.height());
-//             rect = rect & QRect( 0 , y - offs.y() - 10, w, 10 + y + h  - offs.y());
+       QRect scaledRect; // bounding box of the whole thing, transformed, so we also know where the origin goes.
+       if (scaled) {
+           scaledRect = painterTransform.mapRect(QRect(0, 0, contentWidth(), contentHeight()));
+           m_imagePainter->setSize(QSize(scaledRect.width(), scaledRect.height()));
+       } else {
+           m_imagePainter->setSize(QSize(contentWidth(), contentHeight()));
+       }
 
-//             qDebug("normal paint rect after %d/%d/%d/%d", rect.x(), rect.y(), rect.width(), rect.height());
-//             qDebug("normal paint: offs.y(): %d, y: %d, diff: %d", offs.y(), y, y - offs.y());
-//             qDebug("");
+        // Now, figure out the rectangle to paint (in painter coordinates), by interesting us with the painting clip rectangle.
+        int x = _tx + leftBorder + leftPad;
+        int y = _ty + topBorder + topPad;
+        QRect imageGeom   = QRect(0, 0, contentWidth(), contentHeight());
 
-//           p->setClipRect(QRect(x,y,w,h));
+        QRect clipPortion = paintInfo.r.translated(-x, -y);
+        imageGeom &= clipPortion;
 
+       QPoint destPos = QPoint(x + imageGeom.x(), y + imageGeom.y());
 
-//             p->drawPixmap( offs.x(), y, pix, rect.x(), rect.y(), rect.width(), rect.height() );
-             paintInfo.p->drawPixmap(offs, pix, rect);
+       // If we're scaling, reset the painters transform, and apply it ourselves; though
+       // being careful not apply the translation to the source rect.
+       if (scaled) {
+           paintInfo.p->resetTransform();
+           destPos   = painterTransform.map(destPos);
+           imageGeom = painterTransform.mapRect(imageGeom).translated(-scaledRect.topLeft());
+       }
+
+        m_imagePainter->paint(destPos.x(), destPos.y(), paintInfo.p,
+                                    imageGeom.x(),     imageGeom.y(),
+                                    imageGeom.width(), imageGeom.height());
+
+       if (scaled)
+           paintInfo.p->setTransform(painterTransform);
 
-        }
     }
     if (m_selectionState != SelectionNone) {
-//    kdDebug(6040) << "_tx " << _tx << " _ty " << _ty << " _x " << _x << " _y " << _y << endl;
+//    kDebug(6040) << "_tx " << _tx << " _ty " << _ty << " _x " << _x << " _y " << _y;
         // Draw in any case if inside selection. For selection borders, the
 	// offset will decide whether to draw selection or not
 	bool draw = true;
@@ -370,7 +354,7 @@ void RenderImage::paint(PaintInfo& paintInfo, int _tx, int _ty)
 	    // don't touch it unless you know what you're doing
     	    paintInfo.p->setBrushOrigin(_tx, _ty - paintInfo.r.y());
             paintInfo.p->fillRect(_tx, _ty, width(), height(),
-		    QBrush(style()->palette().active().highlight(),
+		    QBrush(style()->palette().color( QPalette::Active, QPalette::Highlight ),
 		    Qt::Dense4Pattern));
 	}
     }
@@ -381,8 +365,8 @@ void RenderImage::layout()
     KHTMLAssert( needsLayout());
     KHTMLAssert( minMaxKnown() );
 
-    short oldwidth = m_width;
-    int oldheight = m_height;
+    short m_width = 0;
+    int m_height;
 
     // minimum height
     m_height = m_cachedImage && m_cachedImage->isErrorImage() ? intrinsicHeight() : 0;
@@ -390,38 +374,9 @@ void RenderImage::layout()
     calcWidth();
     calcHeight();
 
-    // if they are variable width and we calculate a huge height or width, we assume they
-    // actually wanted the intrinsic width.
-    if ( m_width > 4096 && !style()->width().isFixed() )
-	m_width = intrinsicWidth() + paddingLeft() + paddingRight() + borderLeft() + borderRight();
-    if ( m_height > 2048 && !style()->height().isFixed() )
-	m_height = intrinsicHeight() + paddingTop() + paddingBottom() + borderTop() + borderBottom();
-
-    // limit total size to not run out of memory when doing the xform call.
-    if ( ( m_width * m_height > 4096*2048 ) &&
-         ( contentWidth() > intrinsicWidth() || contentHeight() > intrinsicHeight() ) ) {
-	float scale = sqrt( m_width*m_height / ( 4096.*2048. ) );
-	m_width = (int) (m_width/scale);
-	m_height = (int) (m_height/scale);
-    }
-
-    if ( m_width != oldwidth || m_height != oldheight )
-        resizeCache = QPixmap();
-
     setNeedsLayout(false);
 }
 
-void RenderImage::notifyFinished(CachedObject *finishedObj)
-{
-    if ( ( m_cachedImage == finishedObj || m_oldImage == finishedObj ) && m_oldImage ) {
-        m_oldImage->deref( this );
-        m_oldImage = 0;
-        repaint();
-    }
-
-    RenderReplaced::notifyFinished(finishedObj);
-}
-
 bool RenderImage::nodeAtPoint(NodeInfo& info, int _x, int _y, int _tx, int _ty, HitTestAction hitTestAction, bool inside)
 {
     inside |= RenderReplaced::nodeAtPoint(info, _x, _y, _tx, _ty, hitTestAction, inside);
@@ -434,8 +389,8 @@ bool RenderImage::nodeAtPoint(NodeInfo& info, int _x, int _y, int _tx, int _ty,
 
         HTMLImageElementImpl* i = element()->id() == ID_IMG ? static_cast<HTMLImageElementImpl*>(element()) : 0;
         HTMLMapElementImpl* map;
-        if (i && i->getDocument()->isHTMLDocument() &&
-            (map = static_cast<HTMLDocumentImpl*>(i->getDocument())->getMap(i->imageMap()))) {
+        if (i && i->document()->isHTMLDocument() &&
+            (map = static_cast<HTMLDocumentImpl*>(i->document())->getMap(i->imageMap()))) {
             // we're a client side image map
             inside = map->mapMouseEvent(_x - tx, _y - ty, contentWidth(), contentHeight(), info);
             info.setInnerNonSharedNode(element());
@@ -445,18 +400,21 @@ bool RenderImage::nodeAtPoint(NodeInfo& info, int _x, int _y, int _tx, int _ty,
     return inside;
 }
 
-void RenderImage::updateImage(CachedImage* new_image)
+void RenderImage::updateImage(CachedImage* newImage)
 {
-    CachedImage* tempimage = m_oldImage;
-    m_oldImage = m_cachedImage;
-    m_cachedImage = new_image;
-    assert( m_cachedImage != m_oldImage );
+    if (newImage == m_cachedImage)
+        return;
+
+    delete m_imagePainter; m_imagePainter = 0;
 
     if ( m_cachedImage )
-        m_cachedImage->ref(this);
+        m_cachedImage->deref(this);
 
-    if ( tempimage )
-        tempimage->deref(this);
+    // Note: this must be up-to-date before we ref, since
+    // ref can cause us to be notified of it being loaded
+    m_cachedImage  = newImage;
+    if ( m_cachedImage )
+        m_cachedImage->ref(this);
 
     // if the loading finishes we might get an error and then the image is deleted
     if ( m_cachedImage )
@@ -477,7 +435,7 @@ void RenderImage::updateFromElement()
 
     if (!u.isEmpty() &&
         ( !m_cachedImage || m_cachedImage->url() != u ) ) {
-        CachedImage *new_image = element()->getDocument()->docLoader()->
+        CachedImage *new_image = element()->document()->docLoader()->
                                  requestImage(khtml::parseURL(u));
 
         if(new_image && new_image != m_cachedImage)
@@ -489,7 +447,7 @@ bool RenderImage::complete() const
 {
      // "complete" means that the image has been loaded
      // but also that its width/height (contentWidth(),contentHeight()) have been calculated.
-     return m_cachedImage && m_cachedImage->valid_rect().size() == m_cachedImage->pixmap_size() && !needsLayout();
+     return m_cachedImage && m_cachedImage->isComplete() && !needsLayout();
 }
 
 bool RenderImage::isWidthSpecified() const
