@@ -228,6 +228,8 @@ RenderWidget::~RenderWidget()
     KHTMLAssert( refCount() <= 0 );
 
     if(m_widget) {
+        if (m_widget->hasFocus ())
+            m_widget->clearFocus ();
         m_widget->hide();
         if (m_ownsWidget)
             m_widget->deleteLater();
@@ -852,6 +854,7 @@ bool RenderWidget::eventFilter(QObject* /*o*/, QEvent* e)
 }
 
 void RenderWidget::EventPropagator::sendEvent(QEvent *e) {
+    // ### why don't we just call event()? That would be the normal route.
     switch(e->type()) {
     case QEvent::Wheel:
         wheelEvent ( static_cast<QWheelEvent *> (e) );
@@ -1223,6 +1226,16 @@ KHTMLView* KHTMLWidgetPrivate::rootViewPos(QPoint& pos)
     return last;
 }
 
+void KHTMLWidgetPrivate::setIsRedirected( bool b )
+{
+    m_redirected = b;
+    if (!b && m_rw && m_rw->widget()) {
+        setInPaintEventFlag( m_rw->widget(), false );
+        m_rw->widget()->setAttribute(Qt::WA_OpaquePaintEvent, false);
+        m_rw->widget()->removeEventFilter(m_rw->view());
+    }
+}
+
 // -----------------------------------------------------------------------------
 
 KHTMLWidget::KHTMLWidget() 
