@@ -117,7 +117,7 @@ int FixedTableLayout::calcWidthArray()
 		int effWidth = 0;
 		if ( w.isFixed() && w.value() > 0 ) {
                     effWidth = w.value();
-		    effWidth = KMIN( effWidth, 32760 );
+		    effWidth = qMin( effWidth, 32760 );
 		}
 #ifdef DEBUG_LAYOUT
 		qDebug("    col element: effCol=%d, span=%d: %d w=%d type=%d",
@@ -181,7 +181,7 @@ int FixedTableLayout::calcWidthArray()
 		int effWidth = 0;
 		if ( (w.isFixed() || w.isPercent()) && w.value() > 0 ) {
                     effWidth = w.value();
-		    effWidth = kMin( effWidth, 32760 );
+		    effWidth = qMin( effWidth, 32760 );
 		}
 #ifdef DEBUG_LAYOUT
 		qDebug("    table cell: effCol=%d, span=%d: %d",  cCol, span, effWidth);
@@ -228,7 +228,7 @@ void FixedTableLayout::calcMinMaxWidth()
     // only need to calculate the minimum width as the sum of the
     // cols/cells with a fixed width.
     //
-    // The maximum width is kMax( minWidth, tableWidth ) if table
+    // The maximum width is qMax( minWidth, tableWidth ) if table
     // width is fixed. If table width is percent, we set maxWidth to
     // unlimited.
 
@@ -239,12 +239,12 @@ void FixedTableLayout::calcMinMaxWidth()
     }
 
     int mw = calcWidthArray() + bs;
-    table->m_minWidth = kMin( kMax( mw, tableWidth ), 0x7fff );
+    table->m_minWidth = qMin( qMax( mw, tableWidth ), 0x7fff );
     table->m_maxWidth = table->m_minWidth;
 
     if ( !tableWidth ) {
 	bool haveNonFixed = false;
-	for ( unsigned int i = 0; i < width.size(); i++ ) {
+	for ( int i = 0; i < width.size(); i++ ) {
 	    if ( !width[i].isFixed() ) {
 		haveNonFixed = true;
 		break;
@@ -268,7 +268,7 @@ void FixedTableLayout::layout()
 #endif
 
 
-    QMemArray<int> calcWidth;
+    QVector<int> calcWidth;
     calcWidth.resize( nEffCols );
     calcWidth.fill( -1 );
 
@@ -297,7 +297,8 @@ void FixedTableLayout::layout()
 #endif
         for ( int i = 0; available > 0 && i < nEffCols; i++ ) {
             if ( width[i].isPercent() ) {
-                int w = base * width[i].value() / totalPercent;
+                // totalPercent may be 0 below if all %-width specifed are 0%. (#172557)
+                int w = totalPercent ? base * width[i].value() / totalPercent : 0;
                 available -= w;
                 calcWidth[i] = w;
             }
@@ -313,7 +314,8 @@ void FixedTableLayout::layout()
 
         for ( int i = 0; available > 0 && i < nEffCols; i++ ) {
             if ( width[i].isVariable() ) {
-                int w = available / totalVariable;
+                // totalVariable may be 0 below if all the variable widths specified are 0.
+                int w = totalVariable ? available / totalVariable : 0;
                 available -= w;
                 calcWidth[i] = w;
 		totalVariable--;
@@ -390,11 +392,14 @@ void AutoTableLayout::recalcColumn( int effCol )
 		RenderTableCell *cell = section->cellAt( i,  effCol );
 		if ( cell == (RenderTableCell *)-1 )
 		    continue;
+                bool cellHasContent = cell && (cell->firstChild() || cell->style()->hasBorder() || cell->style()->hasPadding());
+                if (cellHasContent)
+                    l.emptyCellsOnly = false;
 		if ( cell && cell->colSpan() == 1 ) {
                     // A cell originates in this column.  Ensure we have
                     // a min/max width of at least 1px for this column now.
-                    l.minWidth = kMax(int( l.minWidth ), 1);
-                    l.maxWidth = kMax(int( l.maxWidth ), 1);
+                    l.minWidth = qMax(int( l.minWidth ), 1);
+                    l.maxWidth = qMax(int( l.maxWidth ), cellHasContent ? 1 : 0);
 
 		    if ( !cell->minMaxKnown() )
 			cell->calcMinMaxWidth();
@@ -406,7 +411,7 @@ void AutoTableLayout::recalcColumn( int effCol )
 		    }
 
 		    Length w = cell->styleOrColWidth();
-                    w.l.value = kMin( 32767, kMax( 0, w.value() ) );
+                    w.l.value = qMin( 32767, qMax( 0, w.value() ) );
 		    switch( w.type() ) {
 		    case Fixed:
 			// ignore width=0
@@ -440,8 +445,8 @@ void AutoTableLayout::recalcColumn( int effCol )
 		    if ( cell && (!effCol || section->cellAt( i, effCol-1 ) != cell) ) {
                         // This spanning cell originates in this column.  Ensure we have
                         // a min/max width of at least 1px for this column now.
-                        l.minWidth = kMax(int( l.minWidth ), 1);
-                        l.maxWidth = kMax(int( l.maxWidth ), 1);
+                        l.minWidth = qMax(int( l.minWidth ), cellHasContent ? 1 : 0);
+                        l.maxWidth = qMax(int( l.maxWidth ), 1);
 			insertSpanCell( cell );
 		    }
 		    last = cell;
@@ -460,7 +465,7 @@ void AutoTableLayout::recalcColumn( int effCol )
 	}
     }
 
-    l.maxWidth = kMax(l.maxWidth, int(l.minWidth));
+    l.maxWidth = qMax(l.maxWidth, int(l.minWidth));
 #ifdef DEBUG_LAYOUT
     qDebug("col %d, final min=%d, max=%d, width=%d(%d)", effCol, l.minWidth, l.maxWidth, l.width.value(),  l.width.type() );
 #endif
@@ -575,26 +580,26 @@ void AutoTableLayout::calcMinMaxWidth()
     int maxNonPercent = 0;
 
     int remainingPercent = 100;
-    for ( unsigned int i = 0; i < layoutStruct.size(); i++ ) {
+    for ( int i = 0; i < layoutStruct.size(); i++ ) {
 	minWidth += layoutStruct[i].effMinWidth;
 	maxWidth += layoutStruct[i].effMaxWidth;
 	if ( layoutStruct[i].effWidth.isPercent() ) {
-            int percent = kMin(layoutStruct[i].effWidth.value(), remainingPercent);
-            int pw = ( layoutStruct[i].effMaxWidth * 100) / kMax(percent, 1);
+            int percent = qMin(layoutStruct[i].effWidth.value(), remainingPercent);
+            int pw = ( layoutStruct[i].effMaxWidth * 100) / qMax(percent, 1);
             remainingPercent -= percent;
-	    maxPercent = kMax( pw,  maxPercent );
+	    maxPercent = qMax( pw,  maxPercent );
 	} else {
 	    maxNonPercent += layoutStruct[i].effMaxWidth;
 	}
     }
 
     if (shouldScaleColumns(table)) {
-        maxNonPercent = (maxNonPercent * 100 + 50) / kMax(remainingPercent, 1);
-	maxWidth = kMax( maxNonPercent, maxWidth );
-	maxWidth = kMax( maxWidth, maxPercent );
+        maxNonPercent = (maxNonPercent * 100) / qMax(remainingPercent, 1);
+	maxWidth = qMax( maxNonPercent, maxWidth );
+	maxWidth = qMax( maxWidth, maxPercent );
     }
 
-    maxWidth = kMax( maxWidth, spanMaxWidth );
+    maxWidth = qMax( maxWidth, spanMaxWidth );
 
     int bs = table->bordersPaddingAndSpacing();
     minWidth += bs;
@@ -603,12 +608,12 @@ void AutoTableLayout::calcMinMaxWidth()
     Length tw = table->style()->width();
     if ( tw.isFixed() && tw.value() > 0 ) {
         int width = table->calcBoxWidth(tw.value());
-        minWidth = kMax( minWidth, width );
+        minWidth = qMax( minWidth, width );
 	maxWidth = minWidth;
     }
 
-    table->m_maxWidth = kMin(maxWidth, 0x7fff);
-    table->m_minWidth = kMin(minWidth, 0x7fff);
+    table->m_maxWidth = qMin(maxWidth, 0x7fff);
+    table->m_minWidth = qMin(minWidth, 0x7fff);
 #ifdef DEBUG_LAYOUT
     qDebug("    minWidth=%d, maxWidth=%d", table->m_minWidth, table->m_maxWidth );
 #endif
@@ -633,7 +638,7 @@ int AutoTableLayout::calcEffectiveWidth()
 	layoutStruct[i].effMaxWidth = layoutStruct[i].maxWidth;
     }
 
-    for ( unsigned int i = 0; i < spanCells.size(); i++ ) {
+    for ( int i = 0; i < spanCells.size(); i++ ) {
 	RenderTableCell *cell = spanCells[i];
 	if ( !cell || cell == (RenderTableCell *)-1 )
 	    break;
@@ -653,6 +658,7 @@ int AutoTableLayout::calcEffectiveWidth()
 	bool allColsArePercent = true;
 	bool allColsAreFixed = true;
 	bool haveVariable = false;
+        bool spanHasEmptyCellsOnly = true;
 	int fixedWidth = 0;
 #ifdef DEBUG_LAYOUT
 	int cSpan = span;
@@ -691,6 +697,8 @@ int AutoTableLayout::calcEffectiveWidth()
                     totalPercent += layoutStruct[lastCol].effWidth.value();
                 allColsAreFixed = false;
             }
+            if (!layoutStruct[lastCol].emptyCellsOnly)
+                spanHasEmptyCellsOnly = false;
             span -= table->spanOfEffCol( lastCol );
 	    minWidth += layoutStruct[lastCol].effMinWidth;
 	    maxWidth += layoutStruct[lastCol].effMaxWidth;
@@ -708,11 +716,11 @@ int AutoTableLayout::calcEffectiveWidth()
 		// can't satify this condition, treat as variable
 		w = Length();
 	    } else {
-		int spanMax = kMax( maxWidth, cMaxWidth );
+		int spanMax = qMax( maxWidth, cMaxWidth );
 #ifdef DEBUG_LAYOUT
 		qDebug("    adjusting tMaxWidth (%d): spanMax=%d, value=%d, totalPercent=%d", tMaxWidth, spanMax, w.value(), totalPercent );
 #endif
-		tMaxWidth = kMax( tMaxWidth, spanMax * 100 / w.value() );
+		tMaxWidth = qMax( tMaxWidth, spanMax * 100 / w.value() );
 
 		// all non percent columns in the span get percent values to sum up correctly.
 		int percentMissing = w.value() - totalPercent;
@@ -747,7 +755,7 @@ int AutoTableLayout::calcEffectiveWidth()
 		qDebug("extending minWidth of cols %d-%d to %dpx currentMin=%d accroding to fixed sum %d", col, lastCol-1, cMinWidth, minWidth, fixedWidth );
 #endif
 		for ( unsigned int pos = col; fixedWidth > 0 && pos < lastCol; pos++ ) {
-		    int w = kMax( int( layoutStruct[pos].effMinWidth ), cMinWidth * layoutStruct[pos].width.value() / fixedWidth );
+		    int w = qMax( int( layoutStruct[pos].effMinWidth ), cMinWidth * layoutStruct[pos].width.value() / fixedWidth );
 #ifdef DEBUG_LAYOUT
 		    qDebug("   col %d: min=%d, effMin=%d, new=%d", pos, layoutStruct[pos].effMinWidth, layoutStruct[pos].effMinWidth, w );
 #endif
@@ -764,8 +772,8 @@ int AutoTableLayout::calcEffectiveWidth()
                 for ( unsigned int pos = col; maxw > 0 && pos < lastCol; pos++ ) {
                     if ( layoutStruct[pos].effWidth.isPercent() && layoutStruct[pos].effWidth.value()>0 && fixedWidth <= cMinWidth) {
                         int w = layoutStruct[pos].effMinWidth;
-                        w = kMax( w, cminw*layoutStruct[pos].effWidth.value()/totalPercent );
-                        w = kMin(layoutStruct[pos].effMinWidth+(cMinWidth-minw), w);
+                        w = qMax( w, cminw*layoutStruct[pos].effWidth.value()/totalPercent );
+                        w = qMin(layoutStruct[pos].effMinWidth+(cMinWidth-minw), w);
 #ifdef DEBUG_LAYOUT
                         qDebug("   col %d: min=%d, effMin=%d, new=%d", pos, layoutStruct[pos].effMinWidth, layoutStruct[pos].effMinWidth, w );
 #endif
@@ -785,7 +793,7 @@ int AutoTableLayout::calcEffectiveWidth()
                 // Give min to variable first, to fixed second, and to others third.
 		for ( unsigned int pos = col; maxw > 0 && pos < lastCol; pos++ ) {
 		    if ( layoutStruct[pos].width.isFixed() && haveVariable && fixedWidth <= cMinWidth ) {
-			int w = kMax( int( layoutStruct[pos].effMinWidth ), layoutStruct[pos].width.value() );
+			int w = qMax( int( layoutStruct[pos].effMinWidth ), layoutStruct[pos].width.value() );
 			fixedWidth -= layoutStruct[pos].width.value();
                         minw -= layoutStruct[pos].effMinWidth;
 #ifdef DEBUG_LAYOUT
@@ -799,8 +807,8 @@ int AutoTableLayout::calcEffectiveWidth()
 
 		for ( unsigned int pos = col; maxw > 0 && pos < lastCol && minw < cMinWidth; pos++ ) {
 		    if ( !(layoutStruct[pos].width.isFixed() && haveVariable && fixedWidth <= cMinWidth) ) {
-			int w = kMax( int( layoutStruct[pos].effMinWidth ), cMinWidth * layoutStruct[pos].effMaxWidth / maxw );
-                        w = kMin(layoutStruct[pos].effMinWidth+(cMinWidth-minw), w);
+			int w = qMax( int( layoutStruct[pos].effMinWidth ), cMinWidth * layoutStruct[pos].effMaxWidth / maxw );
+                        w = qMin(layoutStruct[pos].effMinWidth+(cMinWidth-minw), w);
 
 #ifdef DEBUG_LAYOUT
 			qDebug("   col %d: min=%d, effMin=%d, new=%d", pos, layoutStruct[pos].effMinWidth, layoutStruct[pos].effMinWidth, w );
@@ -819,7 +827,7 @@ int AutoTableLayout::calcEffectiveWidth()
 		qDebug("extending maxWidth of cols %d-%d to %dpx", col, lastCol-1, cMaxWidth );
 #endif
 		for ( unsigned int pos = col; maxWidth > 0 && pos < lastCol; pos++ ) {
-		    int w = kMax( int( layoutStruct[pos].effMaxWidth ), cMaxWidth * layoutStruct[pos].effMaxWidth / maxWidth );
+		    int w = qMax( int( layoutStruct[pos].effMaxWidth ), cMaxWidth * layoutStruct[pos].effMaxWidth / maxWidth );
 #ifdef DEBUG_LAYOUT
 		    qDebug("   col %d: max=%d, effMax=%d, new=%d", pos, layoutStruct[pos].effMaxWidth, layoutStruct[pos].effMaxWidth, w );
 #endif
@@ -830,8 +838,12 @@ int AutoTableLayout::calcEffectiveWidth()
 	    }
 	} else {
 	    for ( unsigned int pos = col; pos < lastCol; pos++ )
-		layoutStruct[pos].maxWidth = kMax(layoutStruct[pos].maxWidth, int(layoutStruct[pos].minWidth) );
+		layoutStruct[pos].maxWidth = qMax(layoutStruct[pos].maxWidth, int(layoutStruct[pos].minWidth) );
 	}
+        // treat span ranges consisting of empty cells only as if they had content
+        if (spanHasEmptyCellsOnly)
+            for (unsigned int pos = col; pos < lastCol; pos++)
+                layoutStruct[pos].emptyCellsOnly = false;
     }
     effWidthDirty = false;
 
@@ -857,7 +869,7 @@ void AutoTableLayout::insertSpanCell( RenderTableCell *cell )
     }
 
     // add them in sort. This is a slow algorithm, and a binary search or a fast sorting after collection would be better
-    unsigned int pos = 0;
+    int pos = 0;
     int span = cell->colSpan();
     while ( pos < spanCells.size() && spanCells[pos] && span > spanCells[pos]->colSpan() )
 	pos++;
@@ -906,6 +918,7 @@ void AutoTableLayout::layout()
     int totalFixed = 0;
     int totalPercent = 0;
     int allocVariable = 0;
+    int numAutoEmptyCellsOnly = 0;
 
     // fill up every cell with it's minWidth
     for ( int i = 0; i < nEffCols; i++ ) {
@@ -929,9 +942,14 @@ void AutoTableLayout::layout()
             break;
         case Variable:
         case Static:
-            numVariable++;
-            totalVariable += layoutStruct[i].effMaxWidth;
-            allocVariable += w;
+            if (layoutStruct[i].emptyCellsOnly) 
+                numAutoEmptyCellsOnly++;            
+            else {
+                numVariable++;
+                totalVariable += layoutStruct[i].effMaxWidth;
+                allocVariable += w;
+            }
+            break;
         }
     }
 
@@ -940,7 +958,7 @@ void AutoTableLayout::layout()
 	for ( int i = 0; i < nEffCols; i++ ) {
 	    const Length &width = layoutStruct[i].effWidth;
 	    if ( width.isPercent() ) {
-                int w = kMax ( int( layoutStruct[i].effMinWidth ), width.minWidth( tableWidth ) );
+                int w = qMax ( int( layoutStruct[i].effMinWidth ), width.minWidth( tableWidth ) );
 		available += layoutStruct[i].calcWidth - w;
 		layoutStruct[i].calcWidth = w;
 	    }
@@ -951,10 +969,10 @@ void AutoTableLayout::layout()
 	    for ( int i = nEffCols-1; i >= 0; i-- ) {
 		if ( layoutStruct[i].effWidth.isPercent() ) {
 		    int w = layoutStruct[i].calcWidth;
-		    int reduction = kMin( w,  excess );
+		    int reduction = qMin( w,  excess );
 		    // the lines below might look inconsistent, but that's the way it's handled in mozilla
 		    excess -= reduction;
-		    int newWidth = kMax( int (layoutStruct[i].effMinWidth), w - reduction );
+		    int newWidth = qMax( int (layoutStruct[i].effMinWidth), w - reduction );
 		    available += w - newWidth;
 		    layoutStruct[i].calcWidth = newWidth;
 		    //qDebug("col %d: reducing to %d px (reduction=%d)", i, newWidth, reduction );
@@ -999,8 +1017,8 @@ void AutoTableLayout::layout()
  	//qDebug("redistributing %dpx to %d variable columns. totalVariable=%d",  available,  numVariable,  totalVariable );
 	for ( int i = 0; i < nEffCols; i++ ) {
 	    const Length &width = layoutStruct[i].effWidth;
-	    if ( width.isVariable() && totalVariable != 0 ) {
-		int w = kMax( int ( layoutStruct[i].calcWidth ),
+	    if ( width.isVariable() && totalVariable != 0 && !layoutStruct[i].emptyCellsOnly) {
+		int w = qMax( int ( layoutStruct[i].calcWidth ),
                               available * layoutStruct[i].effMaxWidth / totalVariable );
 		available -= w;
 		totalVariable -= layoutStruct[i].effMaxWidth;
@@ -1050,11 +1068,14 @@ void AutoTableLayout::layout()
 #endif
 
     // spread over the rest
-    if ( available > 0 ) {
+    if ( available > 0 && nEffCols > numAutoEmptyCellsOnly) {
         int total = nEffCols;
         // still have some width to spread
         int i = nEffCols;
         while (  i-- ) {
+            // variable columns with empty cells only don't get any width
+            if (layoutStruct[i].width.isVariable() && layoutStruct[i].emptyCellsOnly)
+                continue;
             int w = available / total;
             available -= w;
             total--;
@@ -1181,7 +1202,7 @@ void AutoTableLayout::layout()
 void AutoTableLayout::calcPercentages() const
 {
     total_percent = 0;
-    for ( unsigned int i = 0; i < layoutStruct.size(); i++ ) {
+    for ( int i = 0; i < layoutStruct.size(); i++ ) {
 	if ( layoutStruct[i].width.isPercent() )
 	    total_percent += layoutStruct[i].width.value();
     }
