@@ -27,8 +27,10 @@
 
 #include "dom/dom2_events.h"
 #include "xml/dom2_viewsimpl.h"
+#include "misc/idstring.h"
+
+#undef FOCUS_EVENT //for win32
 
-class KHTMLPart;
 class QMouseEvent;
 
 namespace DOM {
@@ -37,14 +39,11 @@ class AbstractViewImpl;
 class DOMStringImpl;
 class NodeImpl;
 
-// ### support user-defined events
-
 class EventImpl : public khtml::Shared<EventImpl>
 {
 public:
     enum EventId {
-	UNKNOWN_EVENT = 0,
-	// UI events
+       // UI events
         DOMFOCUSIN_EVENT,
         DOMFOCUSOUT_EVENT,
         DOMACTIVATE_EVENT,
@@ -63,39 +62,42 @@ public:
         DOMNODEINSERTEDINTODOCUMENT_EVENT,
         DOMATTRMODIFIED_EVENT,
         DOMCHARACTERDATAMODIFIED_EVENT,
-	// HTML events
-	LOAD_EVENT,
-	UNLOAD_EVENT,
-	ABORT_EVENT,
-	ERROR_EVENT,
-	SELECT_EVENT,
-	CHANGE_EVENT,
-	SUBMIT_EVENT,
-	RESET_EVENT,
-	FOCUS_EVENT,
-	BLUR_EVENT,
-	RESIZE_EVENT,
-	SCROLL_EVENT,
+       // HTML events
+       LOAD_EVENT,
+       UNLOAD_EVENT,
+       ABORT_EVENT,
+       ERROR_EVENT,
+       SELECT_EVENT,
+       CHANGE_EVENT,
+       SUBMIT_EVENT,
+       RESET_EVENT,
+       FOCUS_EVENT,
+       BLUR_EVENT,
+       RESIZE_EVENT,
+       SCROLL_EVENT,
         // keyboard events
-	KEYDOWN_EVENT,
-	KEYUP_EVENT,
-	KEYPRESS_EVENT, //Mostly corresponds to DOM3 textInput event.
-	// khtml events (not part of DOM)
-	KHTML_ECMA_DBLCLICK_EVENT, // for html ondblclick
-	KHTML_ECMA_CLICK_EVENT, // for html onclick
-	KHTML_DRAGDROP_EVENT,
-	KHTML_MOVE_EVENT,
+       KEYDOWN_EVENT,
+       KEYUP_EVENT,
+       KEYPRESS_EVENT, //Mostly corresponds to DOM3 textInput event.
+       // khtml events (not part of DOM)
+       KHTML_ECMA_DBLCLICK_EVENT, // for html ondblclick
+       KHTML_ECMA_CLICK_EVENT, // for html onclick
+       KHTML_DRAGDROP_EVENT,
+       KHTML_MOVE_EVENT,
+       KHTML_MOUSEWHEEL_EVENT,
+       KHTML_CONTENTLOADED_EVENT,
         // XMLHttpRequest events
         KHTML_READYSTATECHANGE_EVENT
     };
 
     EventImpl();
-    EventImpl(EventId _id, bool canBubbleArg, bool cancelableArg);
+    EventImpl(EventId id, bool canBubbleArg, bool cancelableArg);
     virtual ~EventImpl();
 
-    EventId id() const { return m_id; }
+    EventId id() const { return EventId(m_eventName.id()); }
+    DOMString type() const { return m_eventName.toString(); }
+    EventName name() const { return m_eventName; }
 
-    DOMString type() const { return m_type; }
     NodeImpl *target() const { return m_target; }
     void setTarget(NodeImpl *_target);
     NodeImpl *currentTarget() const { return m_currentTarget; }
@@ -120,24 +122,28 @@ public:
     bool propagationStopped() const { return m_propagationStopped; }
     bool defaultPrevented() const { return m_defaultPrevented; }
 
-    static EventId typeToId(DOMString type);
-    static DOMString idToType(EventId id);
-
     void setDefaultHandled() { m_defaultHandled = true; }
     bool defaultHandled() const { return m_defaultHandled; }
 
     DOMString message() const { return m_message; }
     void setMessage(const DOMString &_message) { m_message = _message; }
 
+    static khtml::IDTable<EventImpl>* idTable() {
+        if (s_idTable)
+            return s_idTable;
+        else
+            return initIdTable();
+    }
 protected:
-    DOMStringImpl *m_type;
-    bool m_canBubble;
-    bool m_cancelable;
+    static khtml::IDTable<EventImpl>* s_idTable;
+    static khtml::IDTable<EventImpl>* initIdTable();
+    EventName m_eventName;
+    bool m_canBubble  : 1;
+    bool m_cancelable : 1;
 
-    bool m_propagationStopped;
-    bool m_defaultPrevented;
-    bool m_defaultHandled;
-    EventId m_id : 6;
+    bool m_propagationStopped : 1;
+    bool m_defaultPrevented   : 1;
+    bool m_defaultHandled     : 1;
     unsigned short m_eventPhase : 2;
     NodeImpl *m_currentTarget; // ref > 0 maintained externally
     NodeImpl *m_target;
@@ -162,10 +168,19 @@ public:
     void initUIEvent(const DOMString &typeArg,
 		     bool canBubbleArg,
 		     bool cancelableArg,
-		     const AbstractView &viewArg,
+		     AbstractViewImpl* viewArg,
 		     long detailArg);
     virtual bool isUIEvent() const;
 
+    //Compat stuff
+    virtual int keyCode() const  { return 0; }
+    virtual int charCode() const { return 0; }
+
+    virtual long pageX() const { return 0; }
+    virtual long pageY() const { return 0; }
+    virtual long layerX() const { return 0; }
+    virtual long layerY() const { return 0; }
+    virtual int which() const { return 0; }
 protected:
     AbstractViewImpl *m_view;
     long m_detail;
@@ -175,6 +190,12 @@ protected:
 // Introduced in DOM Level 2: - internal
 class MouseEventImpl : public UIEventImpl {
 public:
+    enum Orientation {
+        ONone = 0,
+        OHorizontal,
+        OVertical
+    };
+
     MouseEventImpl();
     MouseEventImpl(EventId _id,
 		   bool canBubbleArg,
@@ -194,7 +215,8 @@ public:
 		   unsigned short buttonArg,
 		   NodeImpl *relatedTargetArg,
 		   QMouseEvent *qe = 0,
-                   bool isDoubleClick = false);
+                   bool isDoubleClick = false,
+                   Orientation orient = ONone);
     virtual ~MouseEventImpl();
     long screenX() const { return m_screenX; }
     long screenY() const { return m_screenY; }
@@ -204,7 +226,9 @@ public:
     long layerY() const { return m_layerY; } // non-DOM extension
     long pageX() const { return m_pageX; } // non-DOM extension
     long pageY() const { return m_pageY; } // non-DOM extension
+    virtual int which() const { return button() + 1; } // non-DOM extension
     bool isDoubleClick() const { return m_isDoubleClick; } // non-DOM extension
+    Orientation orientation() const { return KDE_CAST_BF_ENUM(Orientation, m_orientation); } // non-DOM extension
     bool ctrlKey() const { return m_ctrlKey; }
     bool shiftKey() const { return m_shiftKey; }
     bool altKey() const { return m_altKey; }
@@ -217,7 +241,7 @@ public:
     void initMouseEvent(const DOMString &typeArg,
 			bool canBubbleArg,
 			bool cancelableArg,
-			const AbstractView &viewArg,
+			AbstractViewImpl* viewArg,
 			long detailArg,
 			long screenXArg,
 			long screenYArg,
@@ -228,7 +252,8 @@ public:
 			bool shiftKeyArg,
 			bool metaKeyArg,
 			unsigned short buttonArg,
-			const Node &relatedTargetArg);
+			const Node &relatedTargetArg,
+			Orientation orient = ONone);
     virtual bool isMouseEvent() const;
 
     QMouseEvent *qEvent() const { return m_qevent; }
@@ -246,12 +271,12 @@ protected:
     bool m_shiftKey : 1;
     bool m_metaKey : 1;
     bool m_isDoubleClick : 1;
+    KDE_BF_ENUM(Orientation) m_orientation : 2;
     unsigned short m_button;
     NodeImpl *m_relatedTarget;
     QMouseEvent *m_qevent;
 };
 
-
 class KeyEventBaseImpl : public UIEventImpl {
 public:
   // VirtualKeyCode
@@ -312,15 +337,15 @@ public:
   void initKeyBaseEvent(const DOMString &typeArg,
                          bool canBubbleArg,
                          bool cancelableArg,
-                         const AbstractView &viewArg,
+                         AbstractViewImpl* viewArg,
                          unsigned long keyVal,
                          unsigned long virtKeyVal,
                          unsigned long modifiers);
 
-  bool ctrlKey()  const { return m_modifier & Qt::ControlButton; }
-  bool shiftKey() const { return m_modifier & Qt::ShiftButton; }
-  bool altKey()   const { return m_modifier & Qt::AltButton; }
-  bool metaKey()  const { return m_modifier & Qt::MetaButton; }
+  bool ctrlKey()  const { return m_modifier & Qt::ControlModifier; }
+  bool shiftKey() const { return m_modifier & Qt::ShiftModifier; }
+  bool altKey()   const { return m_modifier & Qt::AltModifier; }
+  bool metaKey()  const { return m_modifier & Qt::MetaModifier; }
 
   bool             inputGenerated() const { return m_virtKeyVal == 0; }
   unsigned long    keyVal() const     { return m_keyVal; }
@@ -328,15 +353,11 @@ public:
 
   QKeyEvent *qKeyEvent() const { if (!m_keyEvent) buildQKeyEvent(); return m_keyEvent; }
 
-  //Legacy key stuff...
-  virtual int keyCode() const  = 0;
-  virtual int charCode() const = 0;
+  ~KeyEventBaseImpl();
 
-  //### KDE4: remove these 2
-  void initModifier(unsigned long modifierArg, bool valueArg);
   bool checkModifier(unsigned long modifierArg);
 
-  ~KeyEventBaseImpl();
+  virtual int which() const { return keyCode(); } // non-DOM extension
 
   //Returns true if the event was synthesized by client use of DOM
   bool isSynthetic() const { return m_synthetic; }
@@ -368,18 +389,18 @@ public:
     TextEventImpl();
 
     TextEventImpl(QKeyEvent* key, DOM::AbstractViewImpl* view);
-    
+
     void initTextEvent(const DOMString &typeArg,
                        bool canBubbleArg,
                        bool cancelableArg,
-                       const AbstractView &viewArg,
+                       AbstractViewImpl* viewArg,
                        const DOMString& text);
 
     virtual bool isTextInputEvent() const;
 
     //Legacy key stuff...
-    int keyCode() const;
-    int charCode() const;
+    virtual int keyCode() const;
+    virtual int charCode() const;
 
     DOMString data() const { return m_outputString; }
 private:
@@ -393,13 +414,6 @@ public:
 
   virtual bool isKeyboardEvent() const;
 
-  enum KeyLocation {
-    DOM_KEY_LOCATION_STANDARD      = 0x00,
-    DOM_KEY_LOCATION_LEFT          = 0x01,
-    DOM_KEY_LOCATION_RIGHT         = 0x02,
-    DOM_KEY_LOCATION_NUMPAD        = 0x03
-  };
-
   //Legacy key stuff...
   int keyCode() const;
   int charCode() const;
@@ -412,36 +426,21 @@ public:
   void initKeyboardEvent(const DOMString &typeArg,
                          bool canBubbleArg,
                          bool cancelableArg,
-                         const AbstractView &viewArg,
+                         AbstractViewImpl* viewArg,
                          const DOMString &keyIdentifierArg,
                          unsigned long keyLocationArg,
                          const DOMString& modifiersList);
 
-  //### KDE4: remove this, it's only for compatibility with
-  //the old TextEvent wrapper
-  void initKeyboardEvent(const DOMString &typeArg,
-                         bool canBubbleArg,
-                         bool cancelableArg,
-                         const AbstractView &viewArg,
-                         unsigned long keyVal,
-                         unsigned long virtKeyVal,
-                         unsigned long modifiers,
-                         unsigned long keyLocationArg) {
-     initKeyBaseEvent(typeArg, canBubbleArg, cancelableArg, viewArg,
-        keyVal, virtKeyVal, modifiers);
-     m_keyLocation = keyLocationArg;
-  }
 private:
     unsigned long m_keyLocation;
 };
 
 
-
 class MutationEventImpl : public EventImpl {
 // ### fire these during parsing (if necessary)
 public:
     MutationEventImpl();
-    MutationEventImpl(EventId _id,
+    MutationEventImpl(EventId _id, /* for convenience */
 		      bool canBubbleArg,
 		      bool cancelableArg,
 		      const Node &relatedNodeArg,
@@ -476,27 +475,28 @@ protected:
 
 class RegisteredEventListener {
 public:
-    RegisteredEventListener() : id(EventImpl::EventId(0)), useCapture(false), listener(0) {}
+    RegisteredEventListener() : useCapture(false), listener(0) {}
 
-    RegisteredEventListener(EventImpl::EventId _id, EventListener *_listener, bool _useCapture)
-        : id(_id), useCapture(_useCapture), listener(_listener) { if (listener) listener->ref(); }
+    RegisteredEventListener(EventName _id, EventListener *_listener, bool _useCapture)
+        : eventName(_id), useCapture(_useCapture), listener(_listener) { if (listener) listener->ref(); }
 
     ~RegisteredEventListener() { if (listener) listener->deref(); listener = 0; }
 
     bool operator==(const RegisteredEventListener &other) const
-    { return id == other.id && listener == other.listener && useCapture == other.useCapture; }
+    { return eventName == other.eventName && listener == other.listener
+             && useCapture == other.useCapture; }
 
 
-    EventImpl::EventId id : 6;
+    EventName eventName;
     bool useCapture;
     EventListener *listener;
 
-    RegisteredEventListener( const RegisteredEventListener &other ) : 
-                id(other.id), useCapture(other.useCapture), listener(other.listener) 
+    RegisteredEventListener( const RegisteredEventListener &other ) :
+                eventName(other.eventName), useCapture(other.useCapture), listener(other.listener)
     { if (listener) listener->ref(); }
 
     RegisteredEventListener & operator=( const RegisteredEventListener &other ) {
-        id         = other.id;
+        eventName  = other.eventName;
         useCapture = other.useCapture;
         if (other.listener)
             other.listener->ref();
