@@ -34,27 +34,28 @@
 #include "misc/loader.h"
 #include "misc/seed.h"
 
-#include <qstringlist.h>
-#include <qptrlist.h>
-#include <qobject.h>
-#include <qintcache.h>
-#include <qintdict.h>
-#include <qdict.h>
-#include <qmap.h>
+#include <QtCore/QStringList>
+#include <QtCore/QObject>
+#include <QtCore/QList>
+#include <QtCore/QHash>
+#include <QtCore/QMap>
+
+//Added by qt3to4:
+#include <QTimerEvent>
 
 #include <kurl.h>
 
 class QPaintDevice;
 class QTextCodec;
-class QPaintDeviceMetrics;
 class KHTMLView;
+class QEventLoop;
 
 namespace khtml {
     class Tokenizer;
     class CSSStyleSelector;
     class DocLoader;
-    class CSSStyleSelectorList;
     class RenderArena;
+    class EditCommand;
     class RenderObject;
     class CounterNode;
     class CachedObject;
@@ -73,21 +74,21 @@ namespace DOM {
     class DocumentImpl;
     class DocumentType;
     class DocumentTypeImpl;
+    class EditingTextImpl;
     class ElementImpl;
     class EntityReferenceImpl;
     class EventImpl;
     class EventListener;
-    class GenericRONamedNodeMapImpl;
     class HTMLDocumentImpl;
     class HTMLElementImpl;
     class HTMLImageElementImpl;
+    class JSEditor;
     class NodeFilter;
     class NodeFilterImpl;
     class NodeIteratorImpl;
     class NodeListImpl;
     class ProcessingInstructionImpl;
     class RangeImpl;
-    class RegisteredEventListener;
     class StyleSheetImpl;
     class StyleSheetListImpl;
     class TextImpl;
@@ -104,7 +105,8 @@ public:
     DocumentTypeImpl *createDocumentType( const DOMString &qualifiedName, const DOMString &publicId,
                                           const DOMString &systemId, int &exceptioncode );
     DocumentImpl *createDocument( const DOMString &namespaceURI, const DOMString &qualifiedName,
-                                  const DocumentType &doctype, int &exceptioncode );
+                                  DocumentTypeImpl* dtype,
+                                  KHTMLView* v, int &exceptioncode );
 
     DOMImplementationImpl* getInterface(const DOMString& feature) const;
 
@@ -147,6 +149,7 @@ public:
     };
 
     ElementMappingCache();
+    ~ElementMappingCache();
 
     /**
      Add a pointer as just one of candidates, not neccesserily the proper one
@@ -173,7 +176,7 @@ public:
     */
     ItemInfo* get(const QString& id);
 private:
-    QDict<ItemInfo> m_dict;
+    QHash<QString,ItemInfo*> m_dict;
 };
 
 
@@ -190,15 +193,19 @@ public:
     // DOM methods & attributes for Document
 
     DocumentTypeImpl *doctype() const;
+    void setDocType(DocumentTypeImpl* dt);
 
     DOMImplementationImpl *implementation() const;
     ElementImpl *documentElement() const;
+    virtual void childrenChanged();
     virtual ElementImpl *createElement ( const DOMString &tagName, int* pExceptioncode = 0 );
     virtual AttrImpl *createAttribute( const DOMString &tagName, int* pExceptioncode = 0 );
     DocumentFragmentImpl *createDocumentFragment ();
     TextImpl *createTextNode ( DOMStringImpl* data ) { return new TextImpl( docPtr(), data); }
     TextImpl *createTextNode ( const QString& data )
         { return createTextNode(new DOMStringImpl(data.unicode(), data.length())); }
+    TextImpl *createTextNode ( const DOMString &data ) { return createTextNode(data.implementation()); }
+    TextImpl *createTextNode ( const char *latin1 ) { return createTextNode(DOMString(latin1)); }
     CommentImpl *createComment ( DOMStringImpl* data );
     CDATASectionImpl *createCDATASection ( DOMStringImpl* data );
     ProcessingInstructionImpl *createProcessingInstruction ( const DOMString &target, DOMStringImpl* data );
@@ -210,6 +217,7 @@ public:
                                            int* pExceptioncode = 0 );
     ElementImpl *getElementById ( const DOMString &elementId ) const;
 
+
     // Actually part of HTMLDocument, but used for giving XML documents a window title as well
     DOMString title() const { return m_title; }
     void setTitle(const DOMString& _title);
@@ -219,9 +227,6 @@ public:
     virtual DOMString nodeName() const;
     virtual unsigned short nodeType() const;
 
-    virtual DOMStringImpl* textContent() const;
-    virtual void           setTextContent( const DOMString &text, int& exceptioncode );
-
     // Other methods (not part of DOM)
     virtual bool isDocumentNode() const { return true; }
     virtual bool isHTMLDocument() const { return false; }
@@ -239,13 +244,13 @@ public:
      * This method returns true if all top-level stylesheets have loaded (including
      * any \@imports that they may be loading).
      */
-    bool haveStylesheetsLoaded() { return m_pendingStylesheets <= 0 || m_ignorePendingStylesheets; }
+    bool haveStylesheetsLoaded() const { return m_pendingStylesheets <= 0 || m_ignorePendingStylesheets; }
 
     /**
      * Increments the number of pending sheets.  The \<link\> elements
      * invoke this to add themselves to the loading list.
      */
-    void addPendingSheet() { m_pendingStylesheets++; }
+    void addPendingSheet();
 
     /**
      * Returns true if the document has pending stylesheets
@@ -262,38 +267,42 @@ public:
      * constructed from these which is used to create the a new style selector which collates all of the stylesheets
      * found and is used to calculate the derived styles for all rendering objects.
      *
-     * @param shallow If the stylesheet list for the document is unchanged, with only added or removed rules 
+     * @param shallow If the stylesheet list for the document is unchanged, with only added or removed rules
      * in existing sheets, then set this argument to true for efficiency.
      */
     void updateStyleSelector(bool shallow=false);
 
+    bool readyForLayout() const;
     void recalcStyleSelector();
     void rebuildStyleSelector();
 
-    QString nextState();
+    // Tries to restore the elements value from the doc state,
+    // if it seems like the same thing
+    void attemptRestoreState(NodeImpl* e);
 
     // Query all registered elements for their state
     QStringList docState();
     bool unsubmittedFormChanges();
     void registerMaintainsState(NodeImpl* e) { m_maintainsState.append(e); }
-    void deregisterMaintainsState(NodeImpl* e) { m_maintainsState.removeRef(e); }
+    void deregisterMaintainsState(NodeImpl* e) { int i; if ((i = m_maintainsState.indexOf(e)) != -1) m_maintainsState.removeAt(i); }
 
     // Set the state the document should restore to
-    void setRestoreState( const QStringList &s) { m_state = s; }
+    void setRestoreState( const QStringList &s);
 
-    KHTMLView *view() const { return m_view; }
+    KHTMLView *view() const;
     KHTMLPart* part() const;
 
     RangeImpl *createRange();
 
     NodeIteratorImpl *createNodeIterator(NodeImpl *root, unsigned long whatToShow,
-                                    NodeFilter &filter, bool entityReferenceExpansion, int &exceptioncode);
+                                    NodeFilterImpl* filter, bool entityReferenceExpansion, int &exceptioncode);
 
     TreeWalkerImpl *createTreeWalker(NodeImpl *root, unsigned long whatToShow, NodeFilterImpl *filter,
                             bool entityReferenceExpansion, int &exceptioncode);
 
+    EditingTextImpl *createEditingTextNode(const DOMString &text);
+
     virtual void recalcStyle( StyleChange = NoChange );
-    static QPtrList<DocumentImpl> * changedDocuments;
     virtual void updateRendering();
     void updateLayout();
     static void updateDocumentsRendering();
@@ -307,28 +316,31 @@ public:
     // to get visually ordered hebrew and arabic pages right
     void setVisuallyOrdered();
     // to get URL decoding right
-    void setDecoderCodec(const QTextCodec *codec);
+    //void setDecoderCodec(const QTextCodec *codec);
 
+    // ### elide the two after designMode merge
     void setSelection(NodeImpl* s, int sp, NodeImpl* e, int ep);
     void clearSelection();
+    void updateSelection();
 
     void open ( bool clearEventListeners = true );
     virtual void close (  );
+    virtual void contentLoaded() {}
     void write ( const DOMString &text );
     void write ( const QString &text );
     void writeln ( const DOMString &text );
     void finishParsing (  );
 
-    KURL URL() const { return m_url; }
+    KUrl URL() const { return m_url; }
     void setURL(const QString& url) { m_url = url; }
 
-    KURL baseURL() const { return m_baseURL.isEmpty() ? m_url : m_baseURL; }
-    void setBaseURL(const KURL& baseURL) { m_baseURL = baseURL; }
+    KUrl baseURL() const { return m_baseURL.isEmpty() ? m_url : m_baseURL; }
+    void setBaseURL(const KUrl& baseURL);
 
     QString baseTarget() const { return m_baseTarget; }
     void setBaseTarget(const QString& baseTarget) { m_baseTarget = baseTarget; }
 
-    QString completeURL(const QString& url) const { return KURL(baseURL(),url,m_decoderMibEnum).url(); };
+    QString completeURL(const QString& url) const { return KUrl(baseURL(),url /*,m_decoderMibEnum*/).url(); }
     DOMString canonURL(const DOMString& url) const { return url.isEmpty() ? url : completeURL(url.string()); }
 
     void setUserStyleSheet(const QString& sheet);
@@ -340,9 +352,9 @@ public:
     virtual khtml::Tokenizer *createTokenizer();
     khtml::Tokenizer *tokenizer() { return m_tokenizer; }
 
-    QPaintDeviceMetrics *paintDeviceMetrics() { return m_paintDeviceMetrics; }
-    QPaintDevice *paintDevice() const { return m_paintDevice; }
-    void setPaintDevice( QPaintDevice *dev );
+    void setPaintDevice(QPaintDevice *dev){m_paintDevice = dev;}
+    QPaintDevice *paintDevice() const {return m_paintDevice;}
+    int logicalDpiY();
 
     enum HTMLMode {
         Html3 = 0,
@@ -356,7 +368,7 @@ public:
         Transitional,
         Strict
     };
-    virtual void determineParseMode( const QString &str );
+    virtual void determineParseMode();
     void setParseMode( ParseMode m ) { pMode = m; }
     ParseMode parseMode() const { return pMode; }
 
@@ -380,7 +392,7 @@ public:
     bool prepareMouseEvent( bool readonly, int x, int y, MouseEvent *ev );
 
     virtual bool childTypeAllowed( unsigned short nodeType );
-    virtual NodeImpl *cloneNode ( bool deep );
+    virtual WTF::PassRefPtr<NodeImpl> cloneNode ( bool deep );
 
     NodeImpl::Id getId( NodeImpl::IdType _type, DOMStringImpl* _nsURI, DOMStringImpl *_localName,
                         DOMStringImpl *_prefix, bool readonly, bool lookupHTML, int *pExceptioncode = 0);
@@ -388,7 +400,7 @@ public:
                                       int *pExceptioncode = 0);
     DOMString getName( NodeImpl::IdType _type, NodeImpl::Id _id ) const;
 
-    StyleSheetListImpl* styleSheets() { return m_styleSheets; };
+    StyleSheetListImpl* styleSheets() { return m_styleSheets; }
 
     DOMString preferredStylesheetSet() const { return m_preferredStylesheetSet; }
     DOMString selectedStylesheetSet() const;
@@ -403,6 +415,7 @@ public:
     NodeImpl* hoverNode() const { return m_hoverNode; }
     void setHoverNode(NodeImpl *newHoverNode);
     NodeImpl *focusNode() const { return m_focusNode; }
+    void quietResetFocus(); // Removes focus from active node without attempting to emit any events
     void setFocusNode(NodeImpl *newFocusNode);
     NodeImpl* activeNode() const { return m_activeNode; }
     void setActiveNode(NodeImpl *newActiveNode);
@@ -442,20 +455,25 @@ public:
     void load(const DOMString &uri);
     void loadXML(const DOMString &source);
     // from cachedObjectClient
-    void setStyleSheet(const DOM::DOMString &url, const DOM::DOMString &sheet, const DOM::DOMString &charset);
+    void setStyleSheet(const DOM::DOMString &url, const DOM::DOMString &sheet, const DOM::DOMString &charset, const DOM::DOMString &mimetype);
     void error(int err, const QString &text);
 
     typedef QMap<QString, ProcessingInstructionImpl*> LocalStyleRefs;
     LocalStyleRefs* localStyleRefs() { return &m_localStyleRefs; }
 
     virtual void defaultEventHandler(EventImpl *evt);
-    virtual void setHTMLWindowEventListener(int id, EventListener *listener);
-    EventListener *getHTMLWindowEventListener(int id);
+
+    void setHTMLWindowEventListener(EventName id, EventListener *listener);
+    void setHTMLWindowEventListener(unsigned id, EventListener *listener);
+
+    EventListener *getHTMLWindowEventListener(EventName id);
+    EventListener *getHTMLWindowEventListener(unsigned id);
+
     EventListener *createHTMLEventListener(const QString& code, const QString& name, NodeImpl* node);
 
-    void addWindowEventListener(int id, EventListener *listener, const bool useCapture);
-    void removeWindowEventListener(int id, EventListener *listener, bool useCapture);
-    bool hasWindowEventListener(int id);
+    void addWindowEventListener(EventName id, EventListener *listener, const bool useCapture);
+    void removeWindowEventListener(EventName id, EventListener *listener, bool useCapture);
+    bool hasWindowEventListener(EventName id);
 
     EventListener *createHTMLEventListener(QString code);
 
@@ -515,16 +533,25 @@ public:
 
     bool isURLAllowed(const QString& url) const;
 
-    HTMLElementImpl* body();
+    HTMLElementImpl* body() const;
 
     DOMString toString() const;
+ 
+    bool execCommand(const DOMString &command, bool userInterface, const DOMString &value);
+    bool queryCommandEnabled(const DOMString &command);
+    bool queryCommandIndeterm(const DOMString &command);
+    bool queryCommandState(const DOMString &command);
+    bool queryCommandSupported(const DOMString &command);
+    DOMString queryCommandValue(const DOMString &command);
 
     void incDOMTreeVersion() { ++m_domtree_version; }
     unsigned int domTreeVersion() const { return m_domtree_version; }
 
-    QDict<khtml::CounterNode>* counters(const khtml::RenderObject* o) { return m_counterDict[(void*)o]; }
-    void setCounters(const khtml::RenderObject* o, QDict<khtml::CounterNode> *dict) { m_counterDict.insert((void*)o, dict);}
-    void removeCounters(const khtml::RenderObject* o) { m_counterDict.remove((void*)o); }
+    JSEditor *jsEditor();
+
+    QHash<QString,khtml::CounterNode*>* counters(const khtml::RenderObject* o) { return m_counterDict.value(o); }
+    void setCounters(const khtml::RenderObject* o, QHash<QString,khtml::CounterNode*> *dict) { m_counterDict.insert(o, dict);}
+    void removeCounters(const khtml::RenderObject* o) { delete m_counterDict.take(o); }
 
 
     ElementMappingCache& underDocNamedCache() {
@@ -545,18 +572,19 @@ public:
     khtml::DynamicDomRestyler& dynamicDomRestyler() { return *m_dynamicDomRestyler; }
     const khtml::DynamicDomRestyler& dynamicDomRestyler() const { return *m_dynamicDomRestyler; }
 
-signals:
+Q_SIGNALS:
     void finishedParsing();
 
 protected:
     khtml::CSSStyleSelector *m_styleSelector;
     KHTMLView *m_view;
     QStringList m_state;
+    int         m_stateRestorePos;
 
     khtml::DocLoader *m_docLoader;
     khtml::Tokenizer *m_tokenizer;
-    KURL m_url;
-    KURL m_baseURL;
+    KUrl m_url;
+    KUrl m_baseURL;
     QString m_baseTarget;
 
     DocumentTypeImpl *m_doctype;
@@ -578,7 +606,6 @@ protected:
     CSSStyleSheetImpl *m_elemSheet;
 
     QPaintDevice *m_paintDevice;
-    QPaintDeviceMetrics *m_paintDeviceMetrics;
     ParseMode pMode;
     HTMLMode hMode;
 
@@ -594,33 +621,25 @@ protected:
         IdNameMapping(unsigned short _start)
             : idStart(_start), count(0) {}
         ~IdNameMapping() {
-            QIntDictIterator<DOM::DOMStringImpl> it(names);
-            for (; it.current() ; ++it)
-                it.current()->deref();
+            QHashIterator<long,DOM::DOMStringImpl*> it(names);
+            while (it.hasNext())
+                it.next().value()->deref();
         }
         unsigned short idStart;
         unsigned short count;
-        QIntDict<DOM::DOMStringImpl> names;
-        QDict<void> ids;
-
-        void expandIfNeeded() {
-            if (ids.size() <= ids.count() && ids.size() != khtml_MaxSeed)
-                ids.resize( khtml::nextSeed(ids.count()) );
-            if (names.size() <= names.count() && names.size() != khtml_MaxSeed)
-                names.resize( khtml::nextSeed(names.count()) );
-        }
+        QHash<long,DOM::DOMStringImpl*> names;
+        QHash<QString,void*> ids;
 
         void addAlias(DOMStringImpl* _prefix, DOMStringImpl* _name, bool cs, NodeImpl::Id id) {
             if(_prefix && _prefix->l) {
-                QConstString n(_name->s, _name->l);
-                QConstString px( _prefix->s, _prefix->l );
-                QString name = cs ? n.string() : n.string().upper();
-                QString qn("aliases: " + (cs ? px.string() : px.string().upper()) + ":" + name);
-                if (!ids.find( qn )) {
+                QString n(_name->s, _name->l);
+                QString px( _prefix->s, _prefix->l );
+                QString name = cs ? n : n.toUpper();
+                QString qn("aliases: " + (cs ? px : px.toUpper()) + ":" + name);
+                if (!ids.contains( qn )) {
                     ids.insert( qn, (void*)id );
                 }
             }
-            expandIfNeeded();
         }
 
     };
@@ -629,7 +648,7 @@ protected:
     IdNameMapping *m_elementMap;
     IdNameMapping *m_namespaceMap;
 
-    QPtrList<NodeIteratorImpl> m_nodeIterators;
+    QList<NodeIteratorImpl*> m_nodeIterators;
     AbstractViewImpl *m_defaultView;
 
     unsigned short m_listenerTypes;
@@ -637,10 +656,10 @@ protected:
     StyleSheetListImpl *m_addedStyleSheets; // programmatically added style sheets
     LocalStyleRefs m_localStyleRefs; // references to inlined style elements
     RegisteredListenerList m_windowEventListeners;
-    QPtrList<NodeImpl> m_maintainsState;
+    QList<NodeImpl*> m_maintainsState;
 
     // ### evaluate for placement in RenderStyle
-    QPtrDict<QDict<khtml::CounterNode> > m_counterDict;
+    QHash<const khtml::RenderObject*,QHash<QString,khtml::CounterNode*> *> m_counterDict;
 
     khtml::DynamicDomRestyler *m_dynamicDomRestyler;
 
@@ -652,29 +671,33 @@ protected:
     bool m_async;
     bool m_hadLoadError;
     bool m_docLoading;
-    bool m_inSyncLoad;
+
+    QEventLoop* m_inSyncLoad;
 
     DOMString m_title;
     DOMString m_preferredStylesheetSet;
     khtml::CachedCSSStyleSheet *m_loadingXMLDoc;
 
-    int m_decoderMibEnum;
+    mutable ElementImpl* m_documentElement;
+
+    //int m_decoderMibEnum;
 
     //Forms, images, etc., must be quickly accessible via document.name.
     ElementMappingCache m_underDocNamedCache;
 
     //Cache for nodelists and collections.
-    QIntDict<NodeListImpl::Cache> m_nodeListCache;
+    QHash<long,NodeListImpl::Cache*> m_nodeListCache;
 
-    QPtrList<HTMLImageElementImpl> m_imageLoadEventDispatchSoonList;
-    QPtrList<HTMLImageElementImpl> m_imageLoadEventDispatchingList;
+    QLinkedList<HTMLImageElementImpl*> m_imageLoadEventDispatchSoonList;
+    QLinkedList<HTMLImageElementImpl*> m_imageLoadEventDispatchingList;
     int m_imageLoadEventTimer;
 
     //Cache for getElementById
     mutable ElementMappingCache m_getElementByIdCache;
 
-    khtml::SharedPtr<khtml::RenderArena> m_renderArena;
+    SharedPtr<khtml::RenderArena> m_renderArena;
 private:
+    JSEditor *m_jsEditor;
     mutable DOMString m_domain;
     int m_selfOnlyRefCount;
 public:
@@ -690,7 +713,7 @@ public:
         if (!m_selfOnlyRefCount && !refCount())
             delete this;
     }
-    
+
     // This is called when our last outside reference dies
     virtual void removedLastRef();
 };
@@ -704,7 +727,7 @@ public:
     // DOM methods overridden from  parent classes
     virtual DOMString nodeName() const;
     virtual unsigned short nodeType() const;
-    virtual NodeImpl *cloneNode ( bool deep );
+    virtual WTF::PassRefPtr<NodeImpl> cloneNode ( bool deep );
 
     // Other methods (not part of DOM)
     virtual bool childTypeAllowed( unsigned short type );
@@ -734,17 +757,14 @@ public:
     virtual DOMString nodeName() const;
     virtual unsigned short nodeType() const;
     virtual bool childTypeAllowed( unsigned short type );
-    virtual NodeImpl *cloneNode ( bool deep );
-
-    virtual DOMStringImpl* textContent() const;
-    virtual void           setTextContent( const DOMString &text, int& exceptioncode );
+    virtual WTF::PassRefPtr<NodeImpl> cloneNode ( bool deep );
 
     // Other methods (not part of DOM)
     void setName(const DOMString& n) { m_qualifiedName = n; }
     void setPublicId(const DOMString& publicId) { m_publicId = publicId; }
     void setSystemId(const DOMString& systemId) { m_systemId = systemId; }
+    void setInternalSubset(const DOMString& subset) { m_subset = subset; }
     DOMImplementationImpl *implementation() const { return m_implementation; }
-    void copyFrom(const DocumentTypeImpl&);
 
     virtual DOMString toString() const;
 
