@@ -824,7 +824,8 @@ static bool startAndEndLineNodesIncludingNode(NodeImpl *node, int offset, Select
         while (renderNode && renderNode->isInline())
             renderNode = renderNode->parent();
 
-        renderNode = renderNode->firstChild();
+        if (renderNode)
+            renderNode = renderNode->firstChild();
 
         NodeImpl *startNode = 0;
         NodeImpl *endNode = 0;
@@ -937,32 +938,32 @@ void Selection::debugPosition() const
         Position upstream = pos.equivalentUpstreamPosition();
         Position downstream = pos.equivalentDownstreamPosition();
         fprintf(stderr, "upstream:   %s %p:%ld\n", getTagName(upstream.node()->id())
-            , upstream.node(), upstream.offset());
+            , (void*)upstream.node(), upstream.offset());
         fprintf(stderr, "pos:        %s %p:%ld\n", getTagName(pos.node()->id())
-            , pos.node(), pos.offset());
+            , (void*)pos.node(), pos.offset());
         fprintf(stderr, "downstream: %s %p:%ld\n", getTagName(downstream.node()->id())
-            , downstream.node(), downstream.offset());
+            , (void*)downstream.node(), downstream.offset());
     }
     else {
         Position pos = start();
         Position upstream = pos.equivalentUpstreamPosition();
         Position downstream = pos.equivalentDownstreamPosition();
         fprintf(stderr, "upstream:   %s %p:%ld\n", getTagName(upstream.node()->id())
-            , upstream.node(), upstream.offset());
+            , (void*)upstream.node(), upstream.offset());
         fprintf(stderr, "start:      %s %p:%ld\n", getTagName(pos.node()->id())
-            , pos.node(), pos.offset());
+            , (void*)pos.node(), pos.offset());
         fprintf(stderr, "downstream: %s %p:%ld\n", getTagName(downstream.node()->id())
-            , downstream.node(), downstream.offset());
+            , (void*)downstream.node(), downstream.offset());
         fprintf(stderr, "-----------------------------------\n");
         pos = end();
         upstream = pos.equivalentUpstreamPosition();
         downstream = pos.equivalentDownstreamPosition();
         fprintf(stderr, "upstream:   %s %p:%ld\n", getTagName(upstream.node()->id())
-            , upstream.node(), upstream.offset());
+            , (void*)upstream.node(), upstream.offset());
         fprintf(stderr, "end:        %s %p:%ld\n", getTagName(pos.node()->id())
-            , pos.node(), pos.offset());
+            , (void*)pos.node(), pos.offset());
         fprintf(stderr, "downstream: %s %p:%ld\n", getTagName(downstream.node()->id())
-            , downstream.node(), downstream.offset());
+            , (void*)downstream.node(), downstream.offset());
         fprintf(stderr, "-----------------------------------\n");
     }
 
