@@ -20,6 +20,7 @@
 */
 
 #include <config.h>
+#include <config-kstandarddirs.h>
 
 #include "klauncher_cmds.h"
 
@@ -46,6 +47,7 @@ static char *getDisplay()
    char *screen;
    char *colon;
    char *i;
+
 /*
  don't test for a value from qglobal.h but instead distinguish
  Qt/X11 from Qt/Embedded by the fact that Qt/E apps have -DQWS
@@ -53,7 +55,9 @@ static char *getDisplay()
  but we don't want to include that here) (Simon)
 #ifdef Q_WS_X11
  */
-#if !defined(QWS)
+#if defined(NO_DISPLAY)
+   display = "NODISPLAY";
+#elif !defined(QWS)
    display = getenv("DISPLAY");
 #else
    display = getenv("QWS_DISPLAY");
@@ -62,7 +66,7 @@ static char *getDisplay()
    {
       display = ":0";
    }
-   result = malloc(strlen(display)+1);
+   result = (char*)malloc(strlen(display)+1);
    if (result == NULL)
       return NULL;
 
@@ -139,7 +143,7 @@ static int openSocket()
 
   if (!kde_home || !kde_home[0])
   {
-     kde_home = "~/.kde/";
+     kde_home = "~/" KDE_DEFAULT_HOME "/";
   }
 
   if (kde_home[0] == '~')
@@ -173,21 +177,25 @@ static int openSocket()
 
   /* append $DISPLAY */
   display = getDisplay();
+#if !defined (NO_DISPLAY)
   if (display == NULL)
   {
      fprintf(stderr, "Error: Could not determine display.\n");
      return -1;
   }
+#endif
 
-  if (strlen(sock_file)+strlen(display)+strlen("/kdeinit_")+2 > MAX_SOCK_FILE)
+  if (strlen(sock_file)+strlen(display)+strlen("/kdeinit4_")+2 > MAX_SOCK_FILE)
   {
      fprintf(stderr, "Warning: Socket name will be too long.\n");
      free (display);
      return -1;
   }
-  strcat(sock_file, "/kdeinit_");
+  strcat(sock_file, "/kdeinit4_");
+#if !defined (NO_DISPLAY)
   strcat(sock_file, display);
   free(display);
+#endif
 
   if (strlen(sock_file) >= sizeof(server.sun_path))
   {
@@ -210,7 +218,8 @@ static int openSocket()
   socklen = sizeof(server);
   if(connect(s, (struct sockaddr *)&server, socklen) == -1) 
   {
-     perror("Warning: connect() failed: ");
+     fprintf(stderr, "kdeinit4_wrapper: Warning: connect(%s) failed:", sock_file);
+     perror(" ");
      close(s);
      return -1;
   }
@@ -218,8 +227,6 @@ static int openSocket()
 }
 
 static pid_t kwrapper_pid;
-static volatile pid_t got_sig; /* = 0 */
-static pid_t last_sig; /* = 0 */
 
 static void sig_pass_handler( int signo );
 static void setup_signals( void );
@@ -244,9 +251,17 @@ static void sig_pass_handler( int signo )
         kill( kwrapper_pid, SIGSTOP ); /* pass the signal to the real process */
     else                               /* SIGTSTP wouldn't work ... I don't think is much */
         kill( kwrapper_pid, signo );   /* of a problem */
-    got_sig = signo;
-    if( got_sig == SIGCONT )
+
+    if( signo == SIGCONT )
         setup_signals(); /* restore signals */
+    else if( signo == SIGCHLD )
+        ; /* nothing, ignore */
+    else /* do the default action ( most of them quit the app ) */
+    {
+        setup_signal_handler( signo, 1 );
+        raise( signo ); /* handle the signal again */
+    }
+        
     errno = save_errno;
 }
 
@@ -274,30 +289,40 @@ static void setup_signals()
     /* some more ? */
 }
 
-static void kwrapper_run( pid_t pid )
+static int kwrapper_run( pid_t wrapped, int sock )
 {
-    kwrapper_pid = pid;
+    klauncher_header header;
+    char *buffer;
+    long pid, status;
+
+    kwrapper_pid = wrapped;
     setup_signals();
-    for(;;)
+
+    read_socket(sock, (char *)&header, sizeof(header));
+
+    if (header.cmd != LAUNCHER_DIED)
     {
-        sleep( 1 ); /* poll and see if the real process is still alive */
-        if( got_sig != last_sig ) /* did we get a signal ? */
-            {
-            last_sig = got_sig;
-            if( got_sig == SIGCHLD )
-                ; /* nothing, ignore */
-            else if( got_sig == SIGCONT )
-                ; /* done in signal handler */
-            else /* do the default action ( most of them quit the app ) */
-                {
-                setup_signal_handler( got_sig, 1 );
-                raise( got_sig ); /* handle the signal again */
-                }
-            }
-        if( kill( pid, 0 ) < 0 )
-            break;
+       fprintf(stderr, "Unexpected response from KInit (response = %ld).\n", header.cmd);
+       exit(255);
     }
- /* I'm afraid it won't be that easy to get the return value ... */
+
+    buffer = (char *) malloc(header.arg_length);
+    if (buffer == NULL)
+    {
+        fprintf(stderr, "Error: malloc() failed\n");
+        exit(255);
+    }
+
+    read_socket(sock, buffer, header.arg_length);
+    pid = ((long *) buffer)[0];
+    if( pid !=  kwrapper_pid)
+    {
+       fprintf(stderr, "Unexpected LAUNCHER_DIED from KInit - pid = %ld\n", pid);
+       exit(255);
+    }
+
+    status = ((long *) buffer)[1];
+    return (int) status;
 }
 
 int main(int argc, char **argv)
@@ -328,24 +353,24 @@ int main(int argc, char **argv)
       p++;
    start = p;
 
-   if (strcmp(start, "kdeinit_wrapper") == 0)
+   if (strcmp(start, "kdeinit4_wrapper") == 0)
       wrapper = 1;
-   else if (strcmp(start, "kshell") == 0)
+   else if (strcmp(start, "kshell4") == 0)
       ext_wrapper = 1;
-   else if (strcmp(start, "kwrapper") == 0)
+   else if (strcmp(start, "kwrapper4") == 0)
       kwrapper = 1;
-   else if (strcmp(start, "kdeinit_shutdown") == 0)
+   else if (strcmp(start, "kdeinit4_shutdown") == 0)
    {
       if( argc > 1)
       {
          fprintf(stderr, "Usage: %s\n\n", start);
-         fprintf(stderr, "Shuts down kdeinit master process and terminates all processes spawned from it.\n");
+         fprintf(stderr, "Shuts down kdeinit4 master process and terminates all processes spawned from it.\n");
          exit( 255 );
       }
       sock = openSocket();
       if( sock < 0 )
       {
-          fprintf( stderr, "Error: Can't contact kdeinit!\n" );
+          fprintf( stderr, "Error: Can not contact kdeinit4!\n" );
           exit( 255 );
       }
       header.cmd = LAUNCHER_TERMINATE_KDE;
@@ -368,10 +393,10 @@ int main(int argc, char **argv)
    }
 
    sock = openSocket();
-   if( sock < 0 ) /* couldn't contact kdeinit, start argv[ 0 ] directly */
+   if( sock < 0 ) /* couldn't contact kdeinit4, start argv[ 0 ] directly */
    {
       execvp( argv[ 0 ], argv );
-      fprintf( stderr, "Error: Can't run %s !\n", argv[ 0 ] );
+      fprintf( stderr, "Error: Can not run %s !\n", argv[ 0 ] );
       exit( 255 );
    }
    
@@ -531,7 +556,7 @@ int main(int argc, char **argv)
       if( !kwrapper ) /* kwrapper shouldn't print any output */
           printf("Launched ok, pid = %ld\n", pid);
       else
-          kwrapper_run( pid );
+          exit( kwrapper_run( pid, sock ) );
    }
    else if (header.cmd == LAUNCHER_ERROR)
    {
