@@ -20,6 +20,7 @@
    Boston, MA 02110-1301, USA.
 */
 
+#include "kbookmarkimporter_ns.h"
 #include "kbookmarkimporter.h"
 #include "kbookmarkexporter.h"
 #include "kbookmarkmanager.h"
@@ -27,15 +28,14 @@
 #include <kstringhandler.h>
 #include <klocale.h>
 #include <kdebug.h>
+#include <kde_file.h>
 #include <kcharsets.h>
-#include <qtextcodec.h>
-#include <qstylesheet.h>
 
-#include <sys/types.h>
-#include <stddef.h>
-#include <dirent.h>
-#include <sys/stat.h>
-#include <assert.h>
+#include <qtextcodec.h>
+#include <qtextdocument.h> // Qt::escape
+#include <QtGui/QApplication>
+
+
 
 void KNSBookmarkImporterImpl::parse()
 {
@@ -45,55 +45,59 @@ void KNSBookmarkImporterImpl::parse()
     if (!codec)
         return;
 
-    if(f.open(IO_ReadOnly)) {
+    if(f.open(QIODevice::ReadOnly)) {
 
         static const int g_lineLimit = 16*1024;
-        QCString s(g_lineLimit);
+        QByteArray s(g_lineLimit,0);
         // skip header
-        while(f.readLine(s.data(), g_lineLimit) >= 0 && !s.contains("<DL>"));
+        while(f.readLine(s.data(), g_lineLimit) >= 1 && !s.contains("<DL>")) {
+            ;
+        }
 
-        while(f.readLine(s.data(), g_lineLimit)>=0) {
-            if ( s[s.length()-1] != '\n' ) // Gosh, this line is longer than g_lineLimit. Skipping.
+        while( int size = f.readLine(s.data(), g_lineLimit)>=1) {
+            if ( size == g_lineLimit ) // Gosh, this line is longer than g_lineLimit. Skipping.
             {
-               kdWarning() << "Netscape bookmarks contain a line longer than " << g_lineLimit << ". Skipping." << endl;
+               kWarning() << "Netscape bookmarks contain a line longer than " << g_lineLimit << ". Skipping.";
                continue;
             }
-            QCString t = s.stripWhiteSpace();
-            if(t.left(12).upper() == "<DT><A HREF=" ||
-               t.left(16).upper() == "<DT><H3><A HREF=") {
-              int firstQuotes = t.find('"')+1;
-              int secondQuotes = t.find('"', firstQuotes);
+            QByteArray t = s.trimmed();
+            if(t.left(12).toUpper() == "<DT><A HREF=" ||
+               t.left(16).toUpper() == "<DT><H3><A HREF=") {
+
+              int firstQuotes = t.indexOf('"')+1;
+              int secondQuotes = t.indexOf('"', firstQuotes);
               if (firstQuotes != -1 && secondQuotes != -1)
               {
-                QCString link = t.mid(firstQuotes, secondQuotes-firstQuotes);
-                int endTag = t.find('>', secondQuotes+1);
-                QCString name = t.mid(endTag+1);
-                name = name.left(name.findRev('<'));
-                if ( name.right(4) == "</A>" )
-                    name = name.left( name.length() - 4 );
+                QByteArray link = t.mid(firstQuotes, secondQuotes-firstQuotes);
+                int endTag = t.indexOf('>', secondQuotes+1);
+
+                int closeTag = t.indexOf('<', endTag + 1);
+
+                QByteArray name = t.mid(endTag + 1, closeTag - endTag - 1);
                 QString qname = KCharsets::resolveEntities( codec->toUnicode( name ) );
-                QCString additionalInfo = t.mid( secondQuotes+1, endTag-secondQuotes-1 );
+                QByteArray additionalInfo = t.mid( secondQuotes+1, endTag-secondQuotes-1 );
 
                 emit newBookmark( qname,
-                                  link, codec->toUnicode(additionalInfo) );
+                                  codec->toUnicode(link),
+                                  QByteArray() );
               }
             }
-            else if(t.left(7).upper() == "<DT><H3") {
-                int endTag = t.find('>', 7);
-                QCString name = t.mid(endTag+1);
-                name = name.left(name.findRev('<'));
+            else if(t.left(7).toUpper() == "<DT><H3") {
+                int endTag = t.indexOf('>', 7);
+                QByteArray name = t.mid(endTag+1);
+                name = name.left(name.indexOf('<'));
                 QString qname = KCharsets::resolveEntities( codec->toUnicode( name ) );
-                QCString additionalInfo = t.mid( 8, endTag-8 );
+                QByteArray additionalInfo = t.mid( 8, endTag-8 );
                 bool folded = (additionalInfo.left(6) == "FOLDED");
                 if (folded) additionalInfo.remove(0,7);
 
                 emit newFolder( qname,
                                 !folded,
-                                codec->toUnicode(additionalInfo) );
+                                QByteArray() );
             }
-            else if(t.left(4).upper() == "<HR>")
+            else if(t.left(4).toUpper() == "<HR>")
                 emit newSeparator();
-            else if(t.left(8).upper() == "</DL><P>")
+            else if(t.left(8).toUpper() == "</DL><P>")
                 emit endFolder();
         }
 
@@ -103,68 +107,21 @@ void KNSBookmarkImporterImpl::parse()
 
 QString KNSBookmarkImporterImpl::findDefaultLocation(bool forSaving) const
 {
-    if (m_utf8) 
+    if (m_utf8)
     {
        if ( forSaving )
-           return KFileDialog::getSaveFileName( QDir::homeDirPath() + "/.mozilla",
-                                                i18n("*.html|HTML Files (*.html)") );
+           return KFileDialog::getSaveFileName( QDir::homePath() + "/.mozilla",
+                                                i18n("*.html|HTML Files (*.html)"),
+                                                QApplication::activeWindow() );
        else
-           return KFileDialog::getOpenFileName( QDir::homeDirPath() + "/.mozilla",
-                                                i18n("*.html|HTML Files (*.html)") );
-    } 
-    else 
-    {
-       return QDir::homeDirPath() + "/.netscape/bookmarks.html";
+           return KFileDialog::getOpenFileName( QDir::homePath() + "/.mozilla",
+                                                i18n("*.html|HTML Files (*.html)"),
+                                                QApplication::activeWindow() );
     }
-}
-
-////////////////////////////////////////////////////////////////
-
-
-void KNSBookmarkImporter::parseNSBookmarks( bool utf8 )
-{
-    KNSBookmarkImporterImpl importer;
-    importer.setFilename(m_fileName);
-    importer.setUtf8(utf8);
-    importer.setupSignalForwards(&importer, this);
-    importer.parse();
-}
-
-QString KNSBookmarkImporter::netscapeBookmarksFile( bool forSaving )
-{
-    static KNSBookmarkImporterImpl *p = 0;
-    if (!p)
+    else
     {
-        p = new KNSBookmarkImporterImpl;
-        p->setUtf8(false);
+       return QDir::homePath() + "/.netscape/bookmarks.html";
     }
-    return p->findDefaultLocation(forSaving);
-}
-
-QString KNSBookmarkImporter::mozillaBookmarksFile( bool forSaving )
-{
-    static KNSBookmarkImporterImpl *p = 0;
-    if (!p)
-    {
-        p = new KNSBookmarkImporterImpl;
-        p->setUtf8(true);
-    }
-    return p->findDefaultLocation(forSaving);
-}
-
-
-////////////////////////////////////////////////////////////////
-//                   compat only
-////////////////////////////////////////////////////////////////
-
-void KNSBookmarkExporter::write(bool utf8) {
-   KNSBookmarkExporterImpl exporter(m_pManager, m_fileName);
-   exporter.setUtf8(utf8);
-   exporter.write(m_pManager->root());
-}
-
-void KNSBookmarkExporter::writeFolder(QTextStream &/*stream*/, KBookmarkGroup /*gp*/) {
-   // TODO - requires a d pointer workaround hack?
 }
 
 ////////////////////////////////////////////////////////////////
@@ -173,29 +130,29 @@ void KNSBookmarkExporterImpl::setUtf8(bool utf8) {
    m_utf8 = utf8;
 }
 
-void KNSBookmarkExporterImpl::write(KBookmarkGroup parent) {
+void KNSBookmarkExporterImpl::write(const KBookmarkGroup &parent) {
    if (QFile::exists(m_fileName)) {
-      ::rename(
-         QFile::encodeName(m_fileName), 
+      KDE_rename(
+         QFile::encodeName(m_fileName),
          QFile::encodeName(m_fileName + ".beforekde"));
    }
 
    QFile file(m_fileName);
 
-   if (!file.open(IO_WriteOnly)) {
-      kdError(7043) << "Can't write to file " << m_fileName << endl;
+   if (!file.open(QIODevice::WriteOnly)) {
+      kError(7043) << "Can't write to file " << m_fileName << endl;
       return;
    }
 
    QTextStream fstream(&file);
-   fstream.setEncoding(m_utf8 ? QTextStream::UnicodeUTF8 : QTextStream::Locale);
+   fstream.setCodec(m_utf8 ? QTextCodec::codecForName("UTF-8") : QTextCodec::codecForLocale());
 
-   QString charset 
-      = m_utf8 ? "UTF-8" : QString::fromLatin1(QTextCodec::codecForLocale()->name()).upper();
+   QString charset
+      = m_utf8 ? "UTF-8" : QString::fromLatin1(QTextCodec::codecForLocale()->name()).toUpper();
 
    fstream << "<!DOCTYPE NETSCAPE-Bookmark-file-1>" << endl
            << i18n("<!-- This file was generated by Konqueror -->") << endl
-           << "<META HTTP-EQUIV=\"Content-Type\" CONTENT=\"text/html; charset=" 
+           << "<META HTTP-EQUIV=\"Content-Type\" CONTENT=\"text/html; charset="
               << charset << "\">" << endl
            << "<TITLE>" << i18n("Bookmarks") << "</TITLE>" << endl
            << "<H1>" << i18n("Bookmarks") << "</H1>" << endl
@@ -204,9 +161,9 @@ void KNSBookmarkExporterImpl::write(KBookmarkGroup parent) {
            << "</DL><P>" << endl;
 }
 
-QString KNSBookmarkExporterImpl::folderAsString(KBookmarkGroup parent) const {
+QString KNSBookmarkExporterImpl::folderAsString(const KBookmarkGroup &parent) const {
    QString str;
-   QTextStream fstream(&str, IO_WriteOnly);
+   QTextStream fstream(&str, QIODevice::WriteOnly);
 
    for (KBookmark bk = parent.first(); !bk.isNull(); bk = parent.next(bk)) {
       if (bk.isSeparator()) {
@@ -214,12 +171,12 @@ QString KNSBookmarkExporterImpl::folderAsString(KBookmarkGroup parent) const {
          continue;
       }
 
-      QString text = QStyleSheet::escape(bk.fullText());
+      QString text = Qt::escape(bk.fullText());
 
       if (bk.isGroup() ) {
-         fstream << "<DT><H3 " 
+         fstream << "<DT><H3 "
                     << (!bk.toGroup().isOpen() ? "FOLDED " : "")
-                    << bk.internalElement().attribute("netscapeinfo") << ">" 
+                    << bk.internalElement().attribute("netscapeinfo") << ">"
                  << text << "</H3>" << endl
                  << "<DL><P>" << endl
                  << folderAsString(bk.toGroup())
@@ -229,7 +186,7 @@ QString KNSBookmarkExporterImpl::folderAsString(KBookmarkGroup parent) const {
       } else {
          // note - netscape seems to use local8bit for url...
          fstream << "<DT><A HREF=\"" << bk.url().url() << "\""
-                    << bk.internalElement().attribute("netscapeinfo") << ">" 
+                    << bk.internalElement().attribute("netscapeinfo") << ">"
                  << text << "</A>" << endl;
          continue;
       }
