@@ -36,7 +36,7 @@
 class KDirLister;
 namespace KIO { class Job; class ListJob; }
 class OrgKdeKDirNotifyInterface;
-
+struct KDirListerCacheDirectoryData;
 
 class KDirLister::Private
 {
@@ -47,8 +47,6 @@ public:
     complete = false;
 
     autoUpdate = false;
-    isShowingDotFiles = false;
-    dirOnlyMode = false;
 
     autoErrorHandling = false;
     errorParent = 0;
@@ -63,8 +61,7 @@ public:
     lstRemoveItems = 0;
 
     refreshItemWasFiltered = false;
-
-    changes = NONE;
+    hasPendingChanges = false;
 
     window = 0;
     m_cachedItemsJob = 0;
@@ -91,6 +88,21 @@ public:
   void emitItemsDeleted(const KFileItemList &items);
   void redirect( const KUrl& oldUrl, const KUrl& newUrl );
 
+    /**
+     * Should this item be visible according to the current filter settings?
+     */
+    bool isItemVisible(const KFileItem& item) const;
+
+    void prepareForSettingsChange() {
+        if (!hasPendingChanges) {
+            hasPendingChanges = true;
+            oldSettings = settings;
+        }
+    }
+
+    void emitChanges();
+
+
   KDirLister *m_parent;
 
   /**
@@ -105,12 +117,11 @@ public:
   bool complete:1;
 
   bool autoUpdate:1;
-  bool isShowingDotFiles:1;
-  bool dirOnlyMode:1;
 
   bool delayedMimeTypes:1;
 
   bool refreshItemWasFiltered:1;
+    bool hasPendingChanges:1; // i.e. settings != oldSettings
 
   bool autoErrorHandling:2;
   QWidget *errorParent;
@@ -129,16 +140,24 @@ public:
   QList<QPair<KFileItem,KFileItem> > *lstRefreshItems;
   KFileItemList *lstMimeFilteredItems, *lstRemoveItems;
 
-  int changes;
-
     QWidget *window; // Main window this lister is associated with
     class CachedItemsJob;
     CachedItemsJob* m_cachedItemsJob;
 
-  QString nameFilter;
-  QList<QRegExp> lstFilters, oldFilters;
-  QStringList mimeFilter, oldMimeFilter;
-  QStringList mimeExcludeFilter, oldMimeExcludeFilter;
+    QString nameFilter; // parsed into lstFilters
+
+    struct FilterSettings {
+        FilterSettings() : isShowingDotFiles(false), dirOnlyMode(false) {}
+        bool isShowingDotFiles;
+        bool dirOnlyMode;
+        QList<QRegExp> lstFilters;
+        QStringList mimeFilter;
+        QStringList mimeExcludeFilter;
+    };
+    FilterSettings settings;
+    FilterSettings oldSettings;
+
+    friend class KDirListerCache;
 };
 
 /**
@@ -236,8 +255,7 @@ private:
     bool validUrl( const KDirLister *lister, const KUrl& _url ) const;
 
     // helper for both stop methods
-    struct DirectoryData;
-    void stopLister(KDirLister* lister, const QString& url, DirectoryData& dirData);
+    void stopLister(KDirLister* lister, const QString& url, KDirListerCacheDirectoryData& dirData);
 
     KIO::ListJob *jobForUrl( const QString& url, KIO::ListJob *not_job = 0 );
     const KUrl& joburl( KIO::ListJob *job );
@@ -271,8 +289,9 @@ private:
   void printDebug();
 #endif
 
-  struct DirItem
+  class DirItem
   {
+	public:
     DirItem( const KUrl &dir )
       : url(dir)
     {
@@ -367,36 +386,43 @@ private:
     QHash<QString /*url*/, DirItem*> itemsInUse;
     QCache<QString /*url*/, DirItem> itemsCached;
 
-    // Data associated with a directory url
-    // This could be in DirItem but only in the itemsInUse dict...
-    struct DirectoryData
-    {
-        // A lister can be EITHER in listersCurrentlyListing OR listersCurrentlyHolding
-        // but NOT in both at the same time.
-        // But both lists can have different listers at the same time; this
-        // happens if more listers are requesting url at the same time and
-        // one lister was stopped during the listing of files.
-
-        // Listers that are currently listing this url
-        QList<KDirLister *> listersCurrentlyListing;
-        // Listers that are currently holding this url
-        QList<KDirLister *> listersCurrentlyHolding;
-
-        void moveListersWithoutCachedItemsJob();
-    };
-
-    typedef QHash<QString /*url*/, DirectoryData> DirectoryDataHash;
+    typedef QHash<QString /*url*/, KDirListerCacheDirectoryData> DirectoryDataHash;
     DirectoryDataHash directoryData;
 
-    // List of files that we have changed recently
+    // Set of local files that we have changed recently (according to KDirWatch)
+    // We temporize the notifications by keeping them 500ms in this list.
     QSet<QString /*url*/> pendingUpdates;
     // The timer for doing the delayed updates
     QTimer pendingUpdateTimer;
 
+    // Set of remote files that have changed recently -- but we can't emit those
+    // changes yet, we need to wait for the "update" directory listing.
+    // The cmp() call can't differ mimetypes since they are determined on demand,
+    // this is why we need to remember those files here.
+    QSet<KFileItem*> pendingRemoteUpdates;
+
     // the KDirNotify signals
     OrgKdeKDirNotifyInterface *kdirnotify;
 };
 
+// Data associated with a directory url
+// This could be in DirItem but only in the itemsInUse dict...
+struct KDirListerCacheDirectoryData
+{
+    // A lister can be EITHER in listersCurrentlyListing OR listersCurrentlyHolding
+    // but NOT in both at the same time.
+    // But both lists can have different listers at the same time; this
+    // happens if more listers are requesting url at the same time and
+    // one lister was stopped during the listing of files.
+
+    // Listers that are currently listing this url
+    QList<KDirLister *> listersCurrentlyListing;
+    // Listers that are currently holding this url
+    QList<KDirLister *> listersCurrentlyHolding;
+
+    void moveListersWithoutCachedItemsJob();
+};
+
 //const unsigned short KDirListerCache::MAX_JOBS_PER_LISTER = 5;
 
 // This job tells KDirListerCache to emit cached items asynchronously from listDir()
