@@ -166,6 +166,7 @@ KDirModelNode* KDirModelPrivate::nodeForUrl(const KUrl& _url, bool returnLastPar
 {
     KUrl url(_url);
     url.adjustPath(KUrl::RemoveTrailingSlash); // KDirLister does this too, so we remove the slash before comparing with the root node url.
+    url.cleanPath(); // remove double slashes in the path
     url.setQuery(QString());
     url.setRef(QString());
 
@@ -203,7 +204,7 @@ KDirModelNode* KDirModelPrivate::nodeForUrl(const KUrl& _url, bool returnLastPar
 
         // E.g. pathStr is /a/b/c and nodePath is /a. We want to find the child "b" in dirNode.
         const QString relativePath = pathStr.mid(nodePath.length());
-        Q_ASSERT(!relativePath.startsWith('/')); // huh? we need double-slash simplification?
+        Q_ASSERT(!relativePath.startsWith('/')); // check if multiple slashes have been removed
         const int nextSlash = relativePath.indexOf('/');
         const QString fileName = relativePath.left(nextSlash); // works even if nextSlash==-1
         KDirModelNode* node = dirNode->m_childNodesByName.value(fileName);
@@ -410,8 +411,10 @@ void KDirModelPrivate::_k_slotDeleteItems(const KFileItemList& items)
     Q_ASSERT(!item.isNull());
     KUrl url = item.url();
     KDirModelNode* node = nodeForUrl(url); // O(depth)
-    if (!node)
+    if (!node) {
+        kWarning(7008) << "No node found for item that was just removed:" << url;
         return;
+    }
 
     KDirModelDirNode* dirNode = node->parent();
     if (!dirNode)
@@ -438,6 +441,9 @@ void KDirModelPrivate::_k_slotDeleteItems(const KFileItemList& items)
         if (!node) { // don't lookup the first item twice
             url = item.url();
             node = nodeForUrl(url);
+            if (!node) {
+                kWarning(7008) << "No node found for item that was just removed:" << url;
+            }
             Q_ASSERT(node);
         }
         rowNumbers.setBit(node->rowNumber(), 1); // O(n)
@@ -674,7 +680,11 @@ bool KDirModel::setData( const QModelIndex & index, const QVariant & value, int
 
 int KDirModel::rowCount( const QModelIndex & parent ) const
 {
-    KDirModelDirNode* parentNode = static_cast<KDirModelDirNode *>(d->nodeForIndex(parent));
+    KDirModelNode* node = d->nodeForIndex(parent);
+    if (!node || !d->isDir(node)) // #176555
+        return 0;
+
+    KDirModelDirNode* parentNode = static_cast<KDirModelDirNode *>(node);
     Q_ASSERT(parentNode);
     const int count = parentNode->m_childNodes.count();
 #if 0
@@ -701,20 +711,21 @@ QModelIndex KDirModel::parent( const QModelIndex & index ) const
 
 QStringList KDirModel::mimeTypes( ) const
 {
-    return QStringList() << QLatin1String("text/uri-list")
-                         << QLatin1String( "application/x-kde-cutselection" ) // TODO
-                         << QLatin1String( "text/plain" )
-                         << QLatin1String( "application/x-kde-urilist" );
+    return KUrl::List::mimeDataTypes()
+        << QLatin1String( "application/x-kde-cutselection" ); // TODO
 }
 
 QMimeData * KDirModel::mimeData( const QModelIndexList & indexes ) const
 {
-    KUrl::List urls;
+    KUrl::List urls, mostLocalUrls;
     foreach ( const QModelIndex &index, indexes ) {
-        urls << d->nodeForIndex( index )->item().url();
+        const KFileItem& item = d->nodeForIndex( index )->item();
+        urls << item.url();
+        bool dummy;
+        mostLocalUrls << item.mostLocalUrl(dummy);
     }
     QMimeData *data = new QMimeData();
-    urls.populateMimeData( data );
+    urls.populateMimeData(mostLocalUrls, data);
     return data;
 }
 
