@@ -15,8 +15,8 @@
 
     You should have received a copy of the GNU Library General Public License
     along with this library; see the file COPYING.LIB.  If not, write to
-    the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
-    Boston, MA 02111-1307, USA.
+    the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+    Boston, MA 02110-1301, USA.
 */
 
 #ifndef _kio_scheduler_h
@@ -24,19 +24,18 @@
 
 #include "kio/job.h"
 #include "kio/jobclasses.h"
-#include <qtimer.h>
-#include <qptrdict.h>
-#include <qmap.h>
-
-#include <dcopobject.h>
+#include <QtCore/QTimer>
+#include <QtCore/QMap>
+#include <QtGui/QWidgetList>
+#include <sys/types.h> // pid_t
 
 namespace KIO {
 
     class Slave;
-    class SlaveList;
     class SlaveConfig;
     class SessionData;
 
+    class SchedulerPrivate;
     /**
      * The KIO::Scheduler manages io-slaves for the application.
      * It also queues jobs and assigns the job to a slave when one
@@ -51,7 +50,7 @@ namespace KIO {
      *
      * Example:
      * \code
-     *    TransferJob *job = KIO::get(KURL("http://www.kde.org"));
+     *    TransferJob *job = KIO::get(KUrl("http://www.kde.org"));
      * \endcode
      *
      *
@@ -64,7 +63,7 @@ namespace KIO {
      *
      * Example:
      * \code
-     *    TransferJob *job = KIO::get(KURL("http://www.kde.org"));
+     *    TransferJob *job = KIO::get(KUrl("http://www.kde.org"));
      *    KIO::Scheduler::scheduleJob(job);
      * \endcode
      *
@@ -81,15 +80,15 @@ namespace KIO {
      * Example:
      * \code
      *    Slave *slave = KIO::Scheduler::getConnectedSlave(
-     *            KURL("pop3://bastian:password@mail.kde.org"));
+     *            KUrl("pop3://bastian:password@mail.kde.org"));
      *    TransferJob *job1 = KIO::get(
-     *            KURL("pop3://bastian:password@mail.kde.org/msg1"));
+     *            KUrl("pop3://bastian:password@mail.kde.org/msg1"));
      *    KIO::Scheduler::assignJobToSlave(slave, job1);
      *    TransferJob *job2 = KIO::get(
-     *            KURL("pop3://bastian:password@mail.kde.org/msg2"));
+     *            KUrl("pop3://bastian:password@mail.kde.org/msg2"));
      *    KIO::Scheduler::assignJobToSlave(slave, job2);
      *    TransferJob *job3 = KIO::get(
-     *            KURL("pop3://bastian:password@mail.kde.org/msg3"));
+     *            KUrl("pop3://bastian:password@mail.kde.org/msg3"));
      *    KIO::Scheduler::assignJobToSlave(slave, job3);
      *
      *    // ... Wait for jobs to finish...
@@ -97,7 +96,7 @@ namespace KIO {
      *    KIO::Scheduler::disconnectSlave(slave);
      * \endcode
      *
-     * Note that you need to explicitly disconnect the slave when the 
+     * Note that you need to explicitly disconnect the slave when the
      * connection goes down, so your error handler should contain:
      * \code
      *    if (error == KIO::ERR_CONNECTION_BROKEN)
@@ -108,50 +107,39 @@ namespace KIO {
      * @see KIO::Job
      **/
 
-    class KIO_EXPORT Scheduler : public QObject, virtual public DCOPObject {
+    class KIO_EXPORT Scheduler : public QObject
+    {
         Q_OBJECT
-
+        Q_CLASSINFO("D-Bus Interface", "org.kde.KIO.Scheduler")
     public:
-        typedef QPtrList<SimpleJob> JobList;
-
-        // InfoDict needs Info, so we can't declare it private
-        class ProtocolInfo;
-        class JobData;
-
-        ~Scheduler();
-
         /**
-         * Register @p job with the scheduler. 
+         * Register @p job with the scheduler.
          * The default is to create a new slave for the job if no slave
          * is available. This can be changed by calling scheduleJob.
-	 * @param job the job to register
+         * @param job the job to register
          */
-        static void doJob(SimpleJob *job)
-        { self()->_doJob(job); }
+        static void doJob(SimpleJob *job);
 
         /**
          * Calling ths function makes that @p job gets scheduled for later
          * execution, if multiple jobs are registered it might wait for
          * other jobs to finish.
-	 * @param job the job to schedule
+         * @param job the job to schedule
          */
-        static void scheduleJob(SimpleJob *job)
-        { self()->_scheduleJob(job); }
+        static void scheduleJob(SimpleJob *job);
 
         /**
          * Stop the execution of a job.
-	 * @param job the job to cancel
+         * @param job the job to cancel
          */
-        static void cancelJob(SimpleJob *job)
-        { self()->_cancelJob(job); }
+        static void cancelJob(SimpleJob *job);
 
         /**
          * Called when a job is done.
-	 * @param job the finished job
-	 * @param slave the slave that executed the @p job
+         * @param job the finished job
+         * @param slave the slave that executed the @p job
          */
-        static void jobFinished(KIO::SimpleJob *job, KIO::Slave *slave)
-        { self()->_jobFinished(job, slave); }
+        static void jobFinished(KIO::SimpleJob *job, KIO::Slave *slave);
 
         /**
          * Puts a slave on notice. A next job may reuse this slave if it
@@ -161,26 +149,23 @@ namespace KIO {
          * Based on the mimetype, the program can give control to another
          * component in the same process which can then resume the job
          * by simply asking for the same URL again.
-	 * @param job the job that should be stopped
-	 * @param url the URL that is handled by the @p url
+         * @param job the job that should be stopped
+         * @param url the URL that is handled by the @p url
          */
-        static void putSlaveOnHold(KIO::SimpleJob *job, const KURL &url)
-        { self()->_putSlaveOnHold(job, url); }
+        static void putSlaveOnHold(KIO::SimpleJob *job, const KUrl &url);
 
         /**
-         * Removes any slave that might have been put on hold. If a slave 
+         * Removes any slave that might have been put on hold. If a slave
          * was put on hold it will be killed.
          */
-        static void removeSlaveOnHold()
-        { self()->_removeSlaveOnHold(); }
+        static void removeSlaveOnHold();
 
         /**
          * Send the slave that was put on hold back to KLauncher. This
          * allows another process to take over the slave and resume the job
          * that was started.
          */
-        static void publishSlaveOnHold()
-        { self()->_publishSlaveOnHold(); }
+        static void publishSlaveOnHold();
 
         /**
          * Requests a slave for use in connection-oriented mode.
@@ -193,8 +178,8 @@ namespace KIO {
          * @see assignJobToSlave()
          * @see disconnectSlave()
          */
-        static KIO::Slave *getConnectedSlave(const KURL &url, const KIO::MetaData &config = MetaData() )
-        { return self()->_getConnectedSlave(url, config); }
+        static KIO::Slave *getConnectedSlave(const KUrl &url,
+                const KIO::MetaData &config = MetaData() );
 
         /*
          * Uses @p slave to do @p job.
@@ -212,8 +197,7 @@ namespace KIO {
          * @see slaveConnected()
          * @see slaveError()
          */
-        static bool assignJobToSlave(KIO::Slave *slave, KIO::SimpleJob *job)
-        { return self()->_assignJobToSlave(slave, job); }
+        static bool assignJobToSlave(KIO::Slave *slave, KIO::SimpleJob *job);
 
         /*
          * Disconnects @p slave.
@@ -227,28 +211,21 @@ namespace KIO {
          * @see getConnectedSlave
          * @see assignJobToSlave
          */
-        static bool disconnectSlave(KIO::Slave *slave)
-        { return self()->_disconnectSlave(slave); }
+        static bool disconnectSlave(KIO::Slave *slave);
 
         /**
-         * Send the slave that was put on hold back to KLauncher. This
-         * allows another process to take over the slave and resume the job
-         * the that was started.
          * Register the mainwindow @p wid with the KIO subsystem
          * Do not call this, it is called automatically from
          * void KIO::Job::setWindow(QWidget*).
-	 * @param wid the window to register
-	 * @since 3.1
+         * @param wid the window to register
          */
-        static void registerWindow(QWidget *wid)
-        { self()->_registerWindow(wid); }
-        
+        static void registerWindow(QWidget *wid);
+
         /**
          * @internal
          * Unregisters the window registered by registerWindow().
          */
-        static void unregisterWindow(QObject *wid)
-        { self()->slotUnregisterWindow(wid); }
+        static void unregisterWindow(QObject *wid);
 
         /**
          * Function to connect signals emitted by the scheduler.
@@ -257,107 +234,56 @@ namespace KIO {
          * @see slaveError()
          */
         static bool connect( const char *signal, const QObject *receiver,
-                             const char *member)
-        { return QObject::connect(self(), signal, receiver, member); }
+                             const char *member);
 
         static bool connect( const QObject* sender, const char* signal,
-                             const QObject* receiver, const char* member )
-        { return QObject::connect(sender, signal, receiver, member); }
+                             const QObject* receiver, const char* member );
 
         static bool disconnect( const QObject* sender, const char* signal,
-                                const QObject* receiver, const char* member )
-        { return QObject::disconnect(sender, signal, receiver, member); }
+                                const QObject* receiver, const char* member );
 
         bool connect( const QObject *sender, const char *signal,
-                      const char *member )
-        { return QObject::connect(sender, signal, member); }
+                      const char *member );
 
         /**
-         * When true, the next job will check whether KLauncher has a slave 
+         * When true, the next job will check whether KLauncher has a slave
          * on hold that is suitable for the job.
-	 * @param b true when KLauncher has a job on hold
+         * @param b true when KLauncher has a job on hold
          */
-        static void checkSlaveOnHold(bool b) { self()->_checkSlaveOnHold(b); }
+        static void checkSlaveOnHold(bool b);
 
-        void debug_info();
+        static void emitReparseSlaveConfiguration();
 
-        virtual bool process(const QCString &fun, const QByteArray &data,
-                             QCString& replyType, QByteArray &replyData);
-
-        virtual QCStringList functions();
-
-    public slots:
-        void slotSlaveDied(KIO::Slave *slave);
-        void slotSlaveStatus(pid_t pid, const QCString &protocol,
-                             const QString &host, bool connected);
-    signals:
+    Q_SIGNALS:
         void slaveConnected(KIO::Slave *slave);
         void slaveError(KIO::Slave *slave, int error, const QString &errorMsg);
 
-    protected:
-        void setupSlave(KIO::Slave *slave, const KURL &url, const QString &protocol, const QString &proxy , bool newSlave, const KIO::MetaData *config=0);
-        bool startJobScheduled(ProtocolInfo *protInfo);
-        bool startJobDirect();
+        // DBUS
+        Q_SCRIPTABLE void reparseSlaveConfiguration(const QString &);
+
+    private:
+        Q_DISABLE_COPY(Scheduler)
         Scheduler();
+        ~Scheduler();
 
-    protected slots:
-        void startStep();
-        void slotCleanIdleSlaves();
-        void slotSlaveConnected();
-        void slotSlaveError(int error, const QString &errorMsg);
-        void slotScheduleCoSlave();
-      /// @since 3.1
-        void slotUnregisterWindow(QObject *);
-
-    private:
-        class ProtocolInfoDict;
-        class ExtraJobData;
-
-        Scheduler(const Scheduler&);
         static Scheduler *self();
-        static Scheduler *instance;
-        void _doJob(SimpleJob *job);
-        void _scheduleJob(SimpleJob *job);
-        void _cancelJob(SimpleJob *job);
-        void _jobFinished(KIO::SimpleJob *job, KIO::Slave *slave);
-        void _scheduleCleanup();
-        void _putSlaveOnHold(KIO::SimpleJob *job, const KURL &url);
-        void _removeSlaveOnHold();
-        Slave *_getConnectedSlave(const KURL &url, const KIO::MetaData &metaData );
-        bool _assignJobToSlave(KIO::Slave *slave, KIO::SimpleJob *job);
-        bool _disconnectSlave(KIO::Slave *slave);
-        void _checkSlaveOnHold(bool b);
-        void _publishSlaveOnHold();
-        void _registerWindow(QWidget *wid);
-        
-        Slave *findIdleSlave(ProtocolInfo *protInfo, SimpleJob *job, bool &exact);
-        Slave *createSlave(ProtocolInfo *protInfo, SimpleJob *job, const KURL &url);
-        
 
-        QTimer slaveTimer;
-        QTimer coSlaveTimer;
-        QTimer cleanupTimer;
-        bool busy;
+        Q_PRIVATE_SLOT(schedulerPrivate, void slotSlaveDied(KIO::Slave *slave))
+        Q_PRIVATE_SLOT(schedulerPrivate, void slotSlaveStatus(pid_t pid, const QByteArray &protocol,
+                                               const QString &host, bool connected))
 
-        SlaveList *slaveList;
-        SlaveList *idleSlaves;
-        SlaveList *coIdleSlaves;
+        // connected to D-Bus signal:
+        Q_PRIVATE_SLOT(schedulerPrivate, void slotReparseSlaveConfiguration(const QString &))
 
-        ProtocolInfoDict *protInfoDict;
-        Slave *slaveOnHold;
-        KURL urlOnHold;
-        JobList newJobs;
-
-        QPtrDict<JobList> coSlaves;
-        ExtraJobData *extraJobData;
-        SlaveConfig *slaveConfig;
-        SessionData *sessionData;
-        bool checkOnHold;
-        QMap<QObject *,WId> m_windowList;
-    protected:
-	virtual void virtual_hook( int id, void* data );
+        Q_PRIVATE_SLOT(schedulerPrivate, void startStep())
+        Q_PRIVATE_SLOT(schedulerPrivate, void slotCleanIdleSlaves())
+        Q_PRIVATE_SLOT(schedulerPrivate, void slotSlaveConnected())
+        Q_PRIVATE_SLOT(schedulerPrivate, void slotSlaveError(int error, const QString &errorMsg))
+        Q_PRIVATE_SLOT(schedulerPrivate, void slotScheduleCoSlave())
+        Q_PRIVATE_SLOT(schedulerPrivate, void slotUnregisterWindow(QObject *))
     private:
-	class SchedulerPrivate* d;
+        friend class SchedulerPrivate;
+        SchedulerPrivate * const d;
 };
 
 }
