@@ -376,6 +376,16 @@ namespace KJS {
      */
     virtual bool implementsCall() const;
 
+    /**
+     * Whether or not this object should be considered a function for the purpose
+     * of the typeof operator. Normally this is the same as implementsCall(),
+     * which is what the default implementation delegates too, 
+     * but in some cases compatibility dictates that the object both be callable
+     * and call itself an object and not a function. In this case, this method should
+     * be overriden as well
+    */
+    virtual bool isFunctionType() const;
+
     /**
      * Calls this object as if it is a function.
      *
@@ -439,6 +449,12 @@ namespace KJS {
     JSValue **getDirectLocation(const Identifier& propertyName)
         { return _prop.getLocation(propertyName); }
 
+    // If this method returns non-0, there is already a property
+    // with name propertyName that's not readonly and not a setter-getter
+    // which can be updated via the returned pointer.
+    JSValue **getDirectWriteLocation(const Identifier& propertyName)
+        { return _prop.getWriteLocation(propertyName); }
+
     void putDirect(const Identifier &propertyName, JSValue *value, int attr = 0)
         { _prop.put(propertyName, value, attr); }
     void putDirect(const Identifier &propertyName, int value, int attr = 0);
