@@ -25,13 +25,13 @@
 #include <kdebug.h>
 #include <kglobal.h>
 #include <klocale.h>
-#include <kmdcodec.h>
+#include <kcodecs.h>
 #include <ksavefile.h>
 #include <kstandarddirs.h>
 
-#include <qfile.h>
-#include <qfileinfo.h>
-#include <qregexp.h>
+#include <QtCore/QFile>
+#include <QtCore/QFileInfo>
+#include <QtCore/QRegExp>
 
 #include "blowfish.h"
 #include "sha1.h"
@@ -39,6 +39,12 @@
 
 #include <assert.h>
 
+// quick fix to get random numbers on win32
+#ifdef Q_OS_WIN32
+    #include <windows.h>
+    #include <wincrypt.h>
+#endif
+
 #define KWALLET_VERSION_MAJOR		0
 #define KWALLET_VERSION_MINOR		0
 
@@ -54,17 +60,21 @@ using namespace KWallet;
 #define KWMAGIC "KWALLET\n\r\0\r\n"
 #define KWMAGIC_LEN 12
 
+class Backend::BackendPrivate
+{
+};
+
 static void initKWalletDir()
 {
-    KGlobal::dirs()->addResourceType("kwallet", "share/apps/kwallet");
+    KGlobal::dirs()->addResourceType("kwallet", 0, "share/apps/kwallet");
 }
 
-Backend::Backend(const QString& name, bool isPath) : _name(name), _ref(0) {
+Backend::Backend(const QString& name, bool isPath) : d(0), _name(name), _ref(0) {
 	initKWalletDir();
 	if (isPath) {
 		_path = name;
 	} else {
-		_path = KGlobal::dirs()->saveLocation("kwallet") + "/" + _name + ".kwl";
+		_path = KGlobal::dirs()->saveLocation("kwallet") + _name + ".kwl";
 	}
 
 	_open = false;
@@ -75,13 +85,14 @@ Backend::~Backend() {
 	if (_open) {
 		close();
 	}
+	delete d;
 }
 
 
 int Backend::close() {
 	for (FolderMap::ConstIterator i = _entries.begin(); i != _entries.end(); ++i) {
-		for (EntryMap::ConstIterator j = i.data().begin(); j != i.data().end(); ++j) {
-			delete j.data();
+		for (EntryMap::ConstIterator j = i.value().begin(); j != i.value().end(); ++j) {
+			delete j.value();
 		}
 	}
 	_entries.clear();
@@ -91,11 +102,30 @@ return 0;
 
 
 static int getRandomBlock(QByteArray& randBlock) {
+
+#ifdef Q_OS_WIN32
+
+    // use windows crypto API to get randomness on win32
+    // HACK: this should be done using qca
+    HCRYPTPROV hProv;
+    
+    if(!CryptAcquireContext(&hProv, NULL, NULL, PROV_RSA_FULL, CRYPT_VERIFYCONTEXT | CRYPT_SILENT)) return -1; // couldn't get random data
+    
+    if(!CryptGenRandom(hProv, static_cast<DWORD>(randBlock.size()), (BYTE*)randBlock.data())) {
+        return -3; // read error
+    }
+    
+    // release the crypto context
+    CryptReleaseContext(hProv, 0);
+    
+    return randBlock.size();
+    
+#else
 	// First try /dev/urandom
 	if (QFile::exists("/dev/urandom")) {
 		QFile devrand("/dev/urandom");
-		if (devrand.open(IO_ReadOnly)) {
-			unsigned int rc = devrand.readBlock(randBlock.data(), randBlock.size());
+		if (devrand.open(QIODevice::ReadOnly)) {
+			int rc = devrand.read(randBlock.data(), randBlock.size());
 
 			if (rc != randBlock.size()) {
 				return -3;		// not enough data read
@@ -109,12 +139,12 @@ static int getRandomBlock(QByteArray& randBlock) {
 	// FIXME: open in noblocking mode!
 	if (QFile::exists("/dev/random")) {
 		QFile devrand("/dev/random");
-		if (devrand.open(IO_ReadOnly)) {
-		unsigned int rc = 0;
-		unsigned int cnt = 0;
+		if (devrand.open(QIODevice::ReadOnly)) {
+    		int rc = 0;
+    		int cnt = 0;
 
 			do {
-				int rc2 = devrand.readBlock(randBlock.data() + rc, randBlock.size());
+				int rc2 = devrand.read(randBlock.data() + rc, randBlock.size());
 
 				if (rc2 < 0) {
 					return -3;	// read error
@@ -132,12 +162,12 @@ static int getRandomBlock(QByteArray& randBlock) {
 	}
 
 	// EGD method
-	char *randFilename;
-	if ((randFilename = getenv("RANDFILE"))) {
+	QString randFilename = QString::fromLocal8Bit(qgetenv("RANDFILE"));
+	if (!randFilename.isEmpty()) {
 		if (QFile::exists(randFilename)) {
 			QFile devrand(randFilename);
-			if (devrand.open(IO_ReadOnly)) {
-				unsigned int rc = devrand.readBlock(randBlock.data(), randBlock.size());
+			if (devrand.open(QIODevice::ReadOnly)) {
+				int rc = devrand.read(randBlock.data(), randBlock.size());
 				if (rc != randBlock.size()) {
 					return -3;      // not enough data read
 				}
@@ -147,8 +177,10 @@ static int getRandomBlock(QByteArray& randBlock) {
 	}
 
 	// Couldn't get any random data!!
-
+    
 	return -1;
+
+#endif
 }
 
 
@@ -159,9 +191,9 @@ static int password2hash(const QByteArray& password, QByteArray& hash) {
 
 	assert(shasz >= 20);
 
-	QByteArray block1(shasz);
+	QByteArray block1(shasz, 0);
 
-	sha.process(password.data(), QMIN(password.size(), 16));
+	sha.process(password.data(), qMin(password.size(), 16));
 
 	// To make brute force take longer
 	for (int i = 0; i < 2000; i++) {
@@ -173,8 +205,8 @@ static int password2hash(const QByteArray& password, QByteArray& hash) {
 	sha.reset();
 
 	if (password.size() > 16) {
-		sha.process(password.data() + 16, QMIN(password.size() - 16, 16));
-		QByteArray block2(shasz);
+		sha.process(password.data() + 16, qMin(password.size() - 16, 16));
+		QByteArray block2(shasz, 0);
 		// To make brute force take longer
 		for (int i = 0; i < 2000; i++) {
 			memcpy(block2.data(), sha.hash(), shasz);
@@ -185,9 +217,9 @@ static int password2hash(const QByteArray& password, QByteArray& hash) {
 		sha.reset();
 
 		if (password.size() > 32) {
-			sha.process(password.data() + 32, QMIN(password.size() - 32, 16));
+			sha.process(password.data() + 32, qMin(password.size() - 32, 16));
 
-			QByteArray block3(shasz);
+			QByteArray block3(shasz, 0);
 			// To make brute force take longer
 			for (int i = 0; i < 2000; i++) {
 				memcpy(block3.data(), sha.hash(), shasz);
@@ -200,7 +232,7 @@ static int password2hash(const QByteArray& password, QByteArray& hash) {
 			if (password.size() > 48) {
 				sha.process(password.data() + 48, password.size() - 48);
 
-				QByteArray block4(shasz);
+				QByteArray block4(shasz, 0);
 				// To make brute force take longer
 				for (int i = 0; i < 2000; i++) {
 					memcpy(block4.data(), sha.hash(), shasz);
@@ -245,10 +277,10 @@ static int password2hash(const QByteArray& password, QByteArray& hash) {
 
 bool Backend::exists(const QString& wallet) {
 	initKWalletDir();
-	QString path = KGlobal::dirs()->saveLocation("kwallet") + "/" + wallet + ".kwl";
+	QString path = KGlobal::dirs()->saveLocation("kwallet") + '/' + wallet + ".kwl";
 	// Note: 60 bytes is presently the minimum size of a wallet file.
 	//       Anything smaller is junk.
-return QFile::exists(path) && QFileInfo(path).size() >= 60;
+	return QFile::exists(path) && QFileInfo(path).size() >= 60;
 }
 
 
@@ -275,7 +307,7 @@ QString Backend::openRCToString(int rc) {
 		case -6:
 			return i18n("Decryption error.");
 		default:
-			return QString::null;
+			return QString();
 	}
 }
 
@@ -293,7 +325,7 @@ int Backend::open(const QByteArray& password) {
 	//       Anything smaller is junk and should be deleted.
 	if (!QFile::exists(_path) || QFileInfo(_path).size() < 60) {
 		QFile newfile(_path);
-		if (!newfile.open(IO_ReadWrite)) {
+		if (!newfile.open(QIODevice::ReadWrite)) {
 			return -2;   // error opening file
 		}
 		newfile.close();
@@ -304,17 +336,17 @@ int Backend::open(const QByteArray& password) {
 
 	QFile db(_path);
 
-	if (!db.open(IO_ReadOnly)) {
+	if (!db.open(QIODevice::ReadOnly)) {
 		return -2;         // error opening file
 	}
 
 	char magicBuf[KWMAGIC_LEN];
-	db.readBlock(magicBuf, KWMAGIC_LEN);
+	db.read(magicBuf, KWMAGIC_LEN);
 	if (memcmp(magicBuf, KWMAGIC, KWMAGIC_LEN) != 0) {
 		return -3;         // bad magic
 	}
 
-	db.readBlock(magicBuf, 4);
+	db.read(magicBuf, 4);
 
 	// First byte is major version, second byte is minor version
 	if (magicBuf[0] != KWALLET_VERSION_MAJOR) {
@@ -336,7 +368,7 @@ int Backend::open(const QByteArray& password) {
 	_hashes.clear();
 	// Read in the hashes
 	QDataStream hds(&db);
-	Q_UINT32 n;
+	quint32 n;
 	hds >> n;
 	if (n > 0xffff) { // sanity check
 		return -43;
@@ -345,16 +377,16 @@ int Backend::open(const QByteArray& password) {
 	for (size_t i = 0; i < n; ++i) {
 		KMD5::Digest d, d2; // judgment day
 		MD5Digest ba;
-		QMap<MD5Digest,QValueList<MD5Digest> >::iterator it;
-		Q_UINT32 fsz;
+		QMap<MD5Digest,QList<MD5Digest> >::iterator it;
+		quint32 fsz;
 		if (hds.atEnd()) return -43;
-		hds.readRawBytes(reinterpret_cast<char *>(d), 16);
+		hds.readRawData(reinterpret_cast<char *>(d), 16);
 		hds >> fsz;
-		ba.duplicate(reinterpret_cast<char *>(d), 16);
-		it = _hashes.insert(ba, QValueList<MD5Digest>());
+		ba = MD5Digest(reinterpret_cast<char *>(d));
+		it = _hashes.insert(ba, QList<MD5Digest>());
 		for (size_t j = 0; j < fsz; ++j) {
-			hds.readRawBytes(reinterpret_cast<char *>(d2), 16);
-			ba.duplicate(reinterpret_cast<char *>(d2), 16);
+			hds.readRawData(reinterpret_cast<char *>(d2), 16);
+			ba = MD5Digest(reinterpret_cast<char *>(d2));
 			(*it).append(ba);
 		}
 	}
@@ -409,7 +441,7 @@ int Backend::open(const QByteArray& password) {
 	t++;
 
 	if (fsize < 0 || fsize > long(encrypted.size()) - blksz - 4) {
-		//kdDebug() << "fsize: " << fsize << " encrypted.size(): " << encrypted.size() << " blksz: " << blksz << endl;
+		//kDebug() << "fsize: " << fsize << " encrypted.size(): " << encrypted.size() << " blksz: " << blksz;
 		encrypted.fill(0);
 		return -9;         // file structure error.
 	}
@@ -432,18 +464,16 @@ int Backend::open(const QByteArray& password) {
 	sha.reset();
 
 	// chop off the leading blksz+4 bytes
-	QByteArray tmpenc;
-	tmpenc.duplicate(encrypted.data()+blksz+4, fsize);
-	encrypted.fill(0);
-	encrypted.duplicate(tmpenc.data(), tmpenc.size());
+	QByteArray tmpenc(encrypted.data()+blksz+4, fsize);
+	encrypted = tmpenc;
 	tmpenc.fill(0);
 
 	// Load the data structures up
-	QDataStream eStream(encrypted, IO_ReadOnly);
+	QDataStream eStream(encrypted);
 
 	while (!eStream.atEnd()) {
 		QString folder;
-		Q_UINT32 n;
+		quint32 n;
 
 		eStream >> folder;
 		eStream >> n;
@@ -456,7 +486,7 @@ int Backend::open(const QByteArray& password) {
 			KWallet::Wallet::EntryType et = KWallet::Wallet::Unknown;
 			Entry *e = new Entry;
 			eStream >> key;
-			Q_INT32 x = 0; // necessary to read properly
+			qint32 x = 0; // necessary to read properly
 			eStream >> x;
 			et = static_cast<KWallet::Wallet::EntryType>(x);
 
@@ -480,6 +510,7 @@ int Backend::open(const QByteArray& password) {
 	}
 
 	_open = true;
+	encrypted.fill(0);
 	return 0;
 }
 
@@ -489,29 +520,28 @@ int Backend::sync(const QByteArray& password) {
 		return -255;  // not open yet
 	}
 
-	KSaveFile sf(_path, 0600);
-	QFile *qf = sf.file();
+	KSaveFile sf(_path);
 
-	if (!qf) {
-		sf.abort();
+	if (!sf.open()) {
 		return -1;		// error opening file
 	}
+	sf.setPermissions(QFile::ReadUser|QFile::WriteUser);
 
-	qf->writeBlock(KWMAGIC, KWMAGIC_LEN);
+	sf.write(KWMAGIC, KWMAGIC_LEN);
 
 	// Write the version number
-	QByteArray version(4);
+	QByteArray version(4, 0);
 	version[0] = KWALLET_VERSION_MAJOR;
 	version[1] = KWALLET_VERSION_MINOR;
 	version[2] = KWALLET_CIPHER_BLOWFISH_CBC;
 	version[3] = KWALLET_HASH_SHA1;
-	qf->writeBlock(version, 4);
+	sf.write(version, 4);
 
 	// Holds the hashes we write out
 	QByteArray hashes;
-	QDataStream hashStream(hashes, IO_WriteOnly);
+	QDataStream hashStream(&hashes, QIODevice::WriteOnly);
 	KMD5 md5;
-	hashStream << static_cast<Q_UINT32>(_entries.count());
+	hashStream << static_cast<quint32>(_entries.count());
 
 	// Holds decrypted data prior to encryption
 	QByteArray decrypted;
@@ -521,28 +551,28 @@ int Backend::sync(const QByteArray& password) {
 	// resizes.
 
 	// populate decrypted
-	QDataStream dStream(decrypted, IO_WriteOnly);
+	QDataStream dStream(&decrypted, QIODevice::WriteOnly);
 	for (FolderMap::ConstIterator i = _entries.begin(); i != _entries.end(); ++i) {
 		dStream << i.key();
-		dStream << static_cast<Q_UINT32>(i.data().count());
+		dStream << static_cast<quint32>(i.value().count());
 
 		md5.reset();
-		md5.update(i.key().utf8());
-		hashStream.writeRawBytes(reinterpret_cast<const char*>(&(md5.rawDigest()[0])), 16);
-		hashStream << static_cast<Q_UINT32>(i.data().count());
+		md5.update(i.key().toUtf8());
+		hashStream.writeRawData(reinterpret_cast<const char*>(&(md5.rawDigest()[0])), 16);
+		hashStream << static_cast<quint32>(i.value().count());
 
-		for (EntryMap::ConstIterator j = i.data().begin(); j != i.data().end(); ++j) {
+		for (EntryMap::ConstIterator j = i.value().begin(); j != i.value().end(); ++j) {
 			dStream << j.key();
-			dStream << static_cast<Q_INT32>(j.data()->type());
-			dStream << j.data()->value();
+			dStream << static_cast<qint32>(j.value()->type());
+			dStream << j.value()->value();
 
 			md5.reset();
-			md5.update(j.key().utf8());
-			hashStream.writeRawBytes(reinterpret_cast<const char*>(&(md5.rawDigest()[0])), 16);
+			md5.update(j.key().toUtf8());
+			hashStream.writeRawData(reinterpret_cast<const char*>(&(md5.rawDigest()[0])), 16);
 		}
 	}
 
-	qf->writeBlock(hashes, hashes.size());
+	sf.write(hashes, hashes.size());
 
 	// calculate the hash of the file
 	SHA1 sha;
@@ -580,7 +610,7 @@ int Backend::sync(const QByteArray& password) {
 		wholeFile[(int)(i+blksz)] = (decrypted.size() >> 8*(3-i))&0xff;
 	}
 
-	for (unsigned int i = 0; i < decrypted.size(); i++) {
+	for (int i = 0; i < decrypted.size(); i++) {
 		wholeFile[(int)(i+blksz+4)] = decrypted[i];
 	}
 
@@ -619,16 +649,15 @@ int Backend::sync(const QByteArray& password) {
 	passhash.fill(0);        // passhash is UNUSABLE NOW
 
 	// write the file
-	qf->writeBlock(wholeFile, wholeFile.size());
-	if (!sf.close()) {
+	sf.write(wholeFile, wholeFile.size());
+	if (!sf.finalize()) {
 		wholeFile.fill(0);
-		sf.abort();
 		return -4; // write error
 	}
 
 	wholeFile.fill(0);
 
-return 0;
+	return 0;
 }
 
 
@@ -663,29 +692,29 @@ QStringList Backend::entryList() const {
 
 
 Entry *Backend::readEntry(const QString& key) {
-Entry *rc = 0L;
+	Entry *rc = 0L;
 
 	if (_open && hasEntry(key)) {
 		rc = _entries[_folder][key];
 	}
 
-return rc;
+	return rc;
 }
 
 
-QPtrList<Entry> Backend::readEntryList(const QString& key) {
-	QPtrList<Entry> rc;
+QList<Entry*> Backend::readEntryList(const QString& key) {
+	QList<Entry*> rc;
 
 	if (!_open) {
 		return rc;
 	}
 
-	QRegExp re(key, true, true);
+	QRegExp re(key, Qt::CaseSensitive, QRegExp::Wildcard);
 
 	const EntryMap& map = _entries[_folder];
 	for (EntryMap::ConstIterator i = map.begin(); i != map.end(); ++i) {
 		if (re.exactMatch(i.key())) {
-			rc.append(i.data());
+			rc.append(i.value());
 		}
 	}
 	return rc;
@@ -700,38 +729,38 @@ bool Backend::createFolder(const QString& f) {
 	_entries.insert(f, EntryMap());
 
 	KMD5 folderMd5;
-	folderMd5.update(f.utf8());
-	_hashes.insert(MD5Digest(folderMd5.rawDigest()), QValueList<MD5Digest>());
+	folderMd5.update(f.toUtf8());
+	_hashes.insert(MD5Digest(folderMd5.rawDigest()), QList<MD5Digest>());
 
-return true;
+	return true;
 }
 
 
 int Backend::renameEntry(const QString& oldName, const QString& newName) {
-EntryMap& emap = _entries[_folder];
-EntryMap::Iterator oi = emap.find(oldName);
-EntryMap::Iterator ni = emap.find(newName);
+	EntryMap& emap = _entries[_folder];
+	EntryMap::Iterator oi = emap.find(oldName);
+	EntryMap::Iterator ni = emap.find(newName);
 
 	if (oi != emap.end() && ni == emap.end()) {
-		Entry *e = oi.data();
-		emap.remove(oi);
+		Entry *e = oi.value();
+		emap.erase(oi);
 		emap[newName] = e;
 
 		KMD5 folderMd5;
-		folderMd5.update(_folder.utf8());
+		folderMd5.update(_folder.toUtf8());
 
 		HashMap::iterator i = _hashes.find(MD5Digest(folderMd5.rawDigest()));
 		if (i != _hashes.end()) {
 			KMD5 oldMd5, newMd5;
-			oldMd5.update(oldName.utf8());
-			newMd5.update(newName.utf8());
-			i.data().remove(MD5Digest(oldMd5.rawDigest()));
-			i.data().append(MD5Digest(newMd5.rawDigest()));
+			oldMd5.update(oldName.toUtf8());
+			newMd5.update(newName.toUtf8());
+			i.value().removeAll(MD5Digest(oldMd5.rawDigest()));
+			i.value().append(MD5Digest(newMd5.rawDigest()));
 		}
 		return 0;
 	}
 
-return -1;
+	return -1;
 }
 
 
@@ -745,13 +774,13 @@ void Backend::writeEntry(Entry *e) {
 	_entries[_folder][e->key()]->copy(e);
 
 	KMD5 folderMd5;
-	folderMd5.update(_folder.utf8());
+	folderMd5.update(_folder.toUtf8());
 
 	HashMap::iterator i = _hashes.find(MD5Digest(folderMd5.rawDigest()));
 	if (i != _hashes.end()) {
 		KMD5 md5;
-		md5.update(e->key().utf8());
-		i.data().append(MD5Digest(md5.rawDigest()));
+		md5.update(e->key().toUtf8());
+		i.value().append(MD5Digest(md5.rawDigest()));
 	}
 }
 
@@ -767,24 +796,24 @@ bool Backend::removeEntry(const QString& key) {
 	}
 
 	FolderMap::Iterator fi = _entries.find(_folder);
-	EntryMap::Iterator ei = fi.data().find(key);
+	EntryMap::Iterator ei = fi.value().find(key);
 
-	if (fi != _entries.end() && ei != fi.data().end()) {
-		delete ei.data();
-		fi.data().remove(ei);
+	if (fi != _entries.end() && ei != fi.value().end()) {
+		delete ei.value();
+		fi.value().erase(ei);
 		KMD5 folderMd5;
-		folderMd5.update(_folder.utf8());
+		folderMd5.update(_folder.toUtf8());
 
 		HashMap::iterator i = _hashes.find(MD5Digest(folderMd5.rawDigest()));
 		if (i != _hashes.end()) {
 			KMD5 md5;
-			md5.update(key.utf8());
-			i.data().remove(MD5Digest(md5.rawDigest()));
+			md5.update(key.toUtf8());
+			i.value().removeAll(MD5Digest(md5.rawDigest()));
 		}
 		return true;
 	}
 
-return false;
+	return false;
 }
 
 
@@ -797,40 +826,40 @@ bool Backend::removeFolder(const QString& f) {
 
 	if (fi != _entries.end()) {
 		if (_folder == f) {
-			_folder = QString::null;
+			_folder.clear();
 		}
 
-		for (EntryMap::Iterator ei = fi.data().begin(); ei != fi.data().end(); ++ei) {
-			delete ei.data();
+		for (EntryMap::Iterator ei = fi.value().begin(); ei != fi.value().end(); ++ei) {
+			delete ei.value();
 		}
 
-		_entries.remove(fi);
+		_entries.erase(fi);
 
 		KMD5 folderMd5;
-		folderMd5.update(f.utf8());
-		_hashes.erase(MD5Digest(folderMd5.rawDigest()));
+		folderMd5.update(f.toUtf8());
+		_hashes.remove(MD5Digest(folderMd5.rawDigest()));
 		return true;
 	}
 
-return false;
+	return false;
 }
 
 
 bool Backend::folderDoesNotExist(const QString& folder) const {
 	KMD5 md5;
-	md5.update(folder.utf8());
+	md5.update(folder.toUtf8());
 	return !_hashes.contains(MD5Digest(md5.rawDigest()));
 }
 
 
 bool Backend::entryDoesNotExist(const QString& folder, const QString& entry) const {
 	KMD5 md5;
-	md5.update(folder.utf8());
+	md5.update(folder.toUtf8());
 	HashMap::const_iterator i = _hashes.find(MD5Digest(md5.rawDigest()));
 	if (i != _hashes.end()) {
 		md5.reset();
-		md5.update(entry.utf8());
-		return !i.data().contains(MD5Digest(md5.rawDigest()));
+		md5.update(entry.toUtf8());
+		return !i.value().contains(MD5Digest(md5.rawDigest()));
 	}
 	return true;
 }
